@@ -65,8 +65,11 @@
 #include "lib/getdate.h"
 #include "lib/glib-missing.h"
 #include "lib/halloc.h"
+#include "lib/hashing.h"
 #include "lib/hashlist.h"
 #include "lib/header.h"
+#include "lib/hikset.h"
+#include "lib/htable.h"
 #include "lib/parse.h"
 #include "lib/random.h"
 #include "lib/strtok.h"
@@ -86,14 +89,14 @@ dmesh_url_error_t dmesh_url_errno;	/**< Error from dmesh_url_parse() */
  * It is implemented as a big hash table, where SHA1 are keys, each value
  * being a struct dmesh pointer.
  */
-static GHashTable *mesh = NULL;
+static hikset_t *mesh = NULL;
 
 struct dmesh {				/**< A download mesh bucket */
 	list_t *entries;		/**< The download mesh entries, dmesh_entry data */
-	GHashTable *by_host;	/**< Entries indexed by host (IP:port) */
-	GHashTable *by_guid;	/**< Entries indexed by GUID (firewalled entries) */
+	htable_t *by_host;		/**< Entries indexed by host (IP:port) */
+	htable_t *by_guid;		/**< Entries indexed by GUID (firewalled entries) */
 	time_t last_update;		/**< Timestamp of last insert/expire in the mesh */
-	const struct sha1 *sha1;	/**< The SHA1 of this mesh */
+	const sha1_t *sha1;		/**< The SHA1 of this mesh */
 };
 
 struct dmesh_entry {
@@ -104,8 +107,8 @@ struct dmesh_entry {
 		dmesh_fwinfo_t fwh;		/**< Firewalled host */
 	} e;
 	hash_list_t *bad;		/**< Keeps track of IPs reporting entry as bad */
-	guint8 good;			/**< Whether marked as being a good entry */
-	guint8 fw_entry;		/**< Whether entry is that of a firewalled host */
+	uint8 good;				/**< Whether marked as being a good entry */
+	uint8 fw_entry;			/**< Whether entry is that of a firewalled host */
 };
 
 #define MAX_LIFETIME	43200		/**< half a day */
@@ -135,7 +138,7 @@ static cqueue_t *dmesh_cq;			/**< Download mesh callout queue */
  *
  * The table is persisted at regular intervals.
  */
-static GHashTable *ban_mesh = NULL;
+static hikset_t *ban_mesh = NULL;
 
 struct dmesh_banned {
 	dmesh_urlinfo_t *info;	/**< The banned URL (same as key) */
@@ -145,12 +148,12 @@ struct dmesh_banned {
 };
 
 typedef void (*dmesh_add_cb)(
-	const struct sha1 *sha1, host_addr_t addr, guint16 port, gpointer udata);
+	const struct sha1 *sha1, host_addr_t addr, uint16 port, void *udata);
 
 /**
  * This table stores the banned entries by SHA1.
  */
-static GHashTable *ban_mesh_by_sha1 = NULL;
+static htable_t *ban_mesh_by_sha1 = NULL;
 
 #define BAN_LIFETIME	7200		/**< 2 hours */
 
@@ -164,16 +167,16 @@ static char *dmesh_fwinfo_to_string(const dmesh_fwinfo_t *info);
 /**
  * Hash a URL info.
  */
-static guint
-urlinfo_hash(gconstpointer key)
+static uint
+urlinfo_hash(const void *key)
 {
 	const dmesh_urlinfo_t *info = key;
-	guint hash;
+	uint hash;
 
 	hash = host_addr_hash(info->addr);
-	hash ^= ((guint32) info->port << 16) | info->port;
-	hash ^= info->idx;
-	hash ^= g_str_hash(info->name);
+	hash ^= port_hash(info->port);
+	hash ^= integer_hash(info->idx);
+	hash ^= string_mix_hash(info->name);
 
 	return hash;
 }
@@ -182,7 +185,7 @@ urlinfo_hash(gconstpointer key)
  * Test equality of two URL infos.
  */
 static int
-urlinfo_eq(gconstpointer a, gconstpointer b)
+urlinfo_eq(const void *a, const void *b)
 {
 	const dmesh_urlinfo_t *ia = a, *ib = b;
 
@@ -198,10 +201,12 @@ urlinfo_eq(gconstpointer a, gconstpointer b)
 G_GNUC_COLD void
 dmesh_init(void)
 {
-	mesh = g_hash_table_new(sha1_hash, sha1_eq);
-	ban_mesh = g_hash_table_new(urlinfo_hash, urlinfo_eq);
-	ban_mesh_by_sha1 = g_hash_table_new(sha1_hash, sha1_eq);
-	dmesh_cq = cq_submake("dmesh", callout_queue, DMESH_CALLOUT);
+	mesh = hikset_create(offsetof(struct dmesh, sha1),
+		HASH_KEY_FIXED, SHA1_RAW_SIZE);
+	ban_mesh = hikset_create_any(offsetof(struct dmesh_banned, info),
+		urlinfo_hash, urlinfo_eq);
+	ban_mesh_by_sha1 = htable_create(HASH_KEY_FIXED, SHA1_RAW_SIZE);
+	dmesh_cq = cq_main_submake("dmesh", DMESH_CALLOUT);
 	dmesh_retrieve();
 	dmesh_ban_retrieve();
 }
@@ -234,7 +239,7 @@ dmesh_entry_free(struct dmesh_entry *dme)
 static void
 dmesh_fill_info(dmesh_urlinfo_t *info,
 	const struct sha1 *sha1, const host_addr_t addr,
-	guint16 port, guint idx, const char *name)
+	uint16 port, uint idx, const char *name)
 {
 	static const char urnsha1[] = "urn:sha1:";
 	static char urn[SHA1_BASE32_SIZE + sizeof urnsha1];
@@ -270,7 +275,7 @@ static void
 dmesh_ban_remove_entry(struct dmesh_banned *dmb)
 {
 	g_assert(dmb);
-	g_assert(dmb == g_hash_table_lookup(ban_mesh, dmb->info));
+	g_assert(dmb == hikset_lookup(ban_mesh, dmb->info));
 
 	/*
 	 * Also remove the banned entry from the IP list by SHA1 which is ussed
@@ -280,26 +285,25 @@ dmesh_ban_remove_entry(struct dmesh_banned *dmb)
 	if (dmb->sha1 != NULL) {
 		GSList *by_addr;
 		GSList *head;
-		gpointer key;			/* The SHA1 atom used for key in table */
-		gpointer x;
-		gboolean found;
+		const void *key;		/* The SHA1 atom used for key in table */
+		void *x;
+		bool found;
 
-		found = g_hash_table_lookup_extended(
-			ban_mesh_by_sha1, dmb->sha1, &key, &x);
+		found = htable_lookup_extended(ban_mesh_by_sha1, dmb->sha1, &key, &x);
 		g_assert(found);
 		head = by_addr = x;
 		by_addr = g_slist_remove(by_addr, dmb);
 
 		if (by_addr == NULL) {
-			g_hash_table_remove(ban_mesh_by_sha1, key);
+			htable_remove(ban_mesh_by_sha1, key);
 			atom_sha1_free(key);
 		} else if (by_addr != head)
-			g_hash_table_insert(ban_mesh_by_sha1, key, by_addr);
+			htable_insert(ban_mesh_by_sha1, key, by_addr);
 
 		atom_sha1_free(dmb->sha1);
 	}
 
-	g_hash_table_remove(ban_mesh, dmb->info);
+	hikset_remove(ban_mesh, dmb->info);
 	dmesh_urlinfo_free(dmb->info);
 	WFREE(dmb);
 }
@@ -308,7 +312,7 @@ dmesh_ban_remove_entry(struct dmesh_banned *dmb)
  * Called from callout queue when it's time to expire the URL ban.
  */
 static void
-dmesh_ban_expire(cqueue_t *unused_cq, gpointer obj)
+dmesh_ban_expire(cqueue_t *unused_cq, void *obj)
 {
 	struct dmesh_banned *dmb = obj;
 
@@ -344,7 +348,7 @@ dmesh_ban_add(const struct sha1 *sha1, dmesh_urlinfo_t *info, time_t stamp)
 	 * Insert new entry, or update old entry if the new one is more recent.
 	 */
 
-	dmb = g_hash_table_lookup(ban_mesh, info);
+	dmb = hikset_lookup(ban_mesh, info);
 
 	if (dmb == NULL) {
 		dmesh_urlinfo_t *ui;
@@ -361,7 +365,7 @@ dmesh_ban_add(const struct sha1 *sha1, dmesh_urlinfo_t *info, time_t stamp)
 		dmb->cq_ev = cq_insert(dmesh_cq, lifetime*1000, dmesh_ban_expire, dmb);
 		dmb->sha1 = NULL;
 
-		g_hash_table_insert(ban_mesh, dmb->info, dmb);
+		hikset_insert(ban_mesh, dmb);
 
 		/*
 		 * Keep record of banned hosts by SHA1 Hash. We will use this to send
@@ -371,23 +375,24 @@ dmesh_ban_add(const struct sha1 *sha1, dmesh_urlinfo_t *info, time_t stamp)
 
 		if (sha1 != NULL) {
 			GSList *by_addr;
-			gboolean existed;
+			bool existed;
 
 			dmb->sha1 = atom_sha1_get(sha1);
 
 			/*
              * Don't fear for duplicates here. The dmb lookup above
              * makes sure that if a XNalt with the IP already exists,
-             * the appropriate dmb will be updates (else-case below).
+             * the appropriate dmb will be updated (else-case below).
              *     -- BLUE 16/01/2004
              */
-			by_addr = g_hash_table_lookup(ban_mesh_by_sha1, sha1);
+			by_addr = htable_lookup(ban_mesh_by_sha1, sha1);
 			existed = by_addr != NULL;
 			by_addr = g_slist_append(by_addr, dmb);
 
-			if (!existed)
-				gm_hash_table_insert_const(ban_mesh_by_sha1,
+			if (!existed) {
+				htable_insert_const(ban_mesh_by_sha1,
 					atom_sha1_get(sha1), by_addr);
+			}
 		}
 	}
 	else if (delta_time(dmb->created, stamp) < 0) {
@@ -400,13 +405,13 @@ dmesh_ban_add(const struct sha1 *sha1, dmesh_urlinfo_t *info, time_t stamp)
  * Forcefully remove an entry from the banned mesh.
  */
 static void
-dmesh_ban_remove(const struct sha1 *sha1, host_addr_t addr, guint16 port)
+dmesh_ban_remove(const struct sha1 *sha1, host_addr_t addr, uint16 port)
 {
 	dmesh_urlinfo_t info;
 	struct dmesh_banned *dmb;
 
 	dmesh_fill_info(&info, sha1, addr, port, URN_INDEX, NULL);
-	dmb = g_hash_table_lookup(ban_mesh, &info);
+	dmb = hikset_lookup(ban_mesh, &info);
 
 	if (dmb) {
 		cq_cancel(&dmb->cq_ev);
@@ -417,16 +422,16 @@ dmesh_ban_remove(const struct sha1 *sha1, host_addr_t addr, guint16 port)
 /**
  * Check whether URL is banned from the mesh.
  */
-static gboolean
+static bool
 dmesh_is_banned(const dmesh_urlinfo_t *info)
 {
-	return NULL != g_hash_table_lookup(ban_mesh, info);
+	return hikset_contains(ban_mesh, info);
 }
 
 /**
  * Are we capable of using firewalled alternate locations?
  */
-gboolean
+bool
 dmesh_can_use_fwalt(void)
 {
 	return !GNET_PROPERTY(is_firewalled) && GNET_PROPERTY(send_pushes);
@@ -473,12 +478,12 @@ dmesh_url_strerror(dmesh_url_error_t errnum)
  * @return TRUE if OK, FALSE if we could not parse it.
  * The variable `dmesh_url_errno' is set accordingly.
  */
-gboolean
+bool
 dmesh_url_parse(const char *url, dmesh_urlinfo_t *info)
 {
 	host_addr_t addr;
-	guint16 port;
-	guint idx;
+	uint16 port;
+	uint idx;
 	const char *endptr, *file, *host = NULL, *path = NULL;
 
 	if (!http_url_parse(url, &port, &host, &path)) {
@@ -546,7 +551,7 @@ dmesh_url_parse(const char *url, dmesh_urlinfo_t *info)
 	 */
 
 	if (idx != URN_INDEX) {
-		char *unescaped = url_unescape(deconstify_gchar(file), FALSE);
+		char *unescaped = url_unescape(deconstify_char(file), FALSE);
 		if (!unescaped) {
 			dmesh_url_errno = DMESH_URL_BAD_ENCODING;
 			return FALSE;
@@ -583,8 +588,9 @@ dm_alloc(const struct sha1 *sha1)
 	dm->last_update = 0;
 	dm->entries = list_new();
 	dm->sha1 = atom_sha1_get(sha1);
-	dm->by_host = g_hash_table_new(packed_host_hash_func, packed_host_eq_func);
-	dm->by_guid = g_hash_table_new(guid_hash, guid_eq);
+	dm->by_host = htable_create_any(packed_host_hash_func,
+		packed_host_hash_func2, packed_host_eq_func);
+	dm->by_guid = htable_create(HASH_KEY_FIXED, GUID_RAW_SIZE);
 
 	return dm;
 }
@@ -603,11 +609,11 @@ dm_free(struct dmesh *dm)
 	 * we just disposed of above, so we only need to get rid of the keys.
 	 */
 	
-	gm_hash_table_foreach_key(dm->by_host, wfree_packed_host, NULL);
-	gm_hash_table_destroy_null(&dm->by_host);
+	htable_foreach_key(dm->by_host, wfree_packed_host, NULL);
+	htable_free_null(&dm->by_host);
 
 	/* Keys were GUID in the dmesh_entry, no need to free them */
-	gm_hash_table_destroy_null(&dm->by_guid);
+	htable_free_null(&dm->by_guid);
 
 	atom_sha1_free_null(&dm->sha1);
 	WFREE(dm);
@@ -620,9 +626,9 @@ static void
 dm_remove_entry(struct dmesh *dm, struct dmesh_entry *dme)
 {
 	struct packed_host packed;
-	gpointer key;
-	gpointer value;
-	gboolean found;
+	const void *key;
+	void *value;
+	bool found;
 
 	g_assert(dm);
 	g_assert(list_length(dm->entries) > 0);
@@ -636,16 +642,16 @@ dm_remove_entry(struct dmesh *dm, struct dmesh_entry *dme)
 	}
 
 	if (dme->fw_entry) {
-		found = g_hash_table_lookup_extended(dm->by_guid,
+		found = htable_lookup_extended(dm->by_guid,
 					dme->e.fwh.guid, &key, &value);
 	} else {
 		packed = host_pack(dme->e.url.addr, dme->e.url.port);
-		found = g_hash_table_lookup_extended(dm->by_host,
+		found = htable_lookup_extended(dm->by_host,
 					&packed, &key, &value);
 	}
 
 	g_assert(found);
-	g_assert(value == (gpointer) dme);
+	g_assert(value == (void *) dme);
 
 	found = list_remove(dm->entries, dme);		/* Remove from list... */
 
@@ -654,10 +660,10 @@ dm_remove_entry(struct dmesh *dm, struct dmesh_entry *dme)
 	/* ...and from the proper hash table */
 
 	if (dme->fw_entry) {
-		g_hash_table_remove(dm->by_guid, dme->e.fwh.guid);
+		htable_remove(dm->by_guid, dme->e.fwh.guid);
 	} else {
-		g_hash_table_remove(dm->by_host, &packed);
-		wfree_packed_host(key, NULL);
+		htable_remove(dm->by_host, &packed);
+		wfree_packed_host(deconstify_pointer(key), NULL);
 	}
 
 	dmesh_entry_free(dme);
@@ -667,18 +673,18 @@ dm_remove_entry(struct dmesh *dm, struct dmesh_entry *dme)
  * Remove the addr:port entry from mesh bucket, if present.
  */
 static void
-dm_remove(struct dmesh *dm, const host_addr_t addr, guint16 port)
+dm_remove(struct dmesh *dm, const host_addr_t addr, uint16 port)
 {
 	struct packed_host packed;
 	struct dmesh_entry *dme;
-	gpointer key;
-	gpointer value;
-	gboolean found;
+	const void *key;
+	void *value;
+	bool found;
 
 	g_assert(dm);
 
 	packed = host_pack(addr, port);
-	found = g_hash_table_lookup_extended(dm->by_host, &packed, &key, &value);
+	found = htable_lookup_extended(dm->by_host, &packed, &key, &value);
 
 	if (!found)
 		return;
@@ -694,8 +700,8 @@ dm_remove(struct dmesh *dm, const host_addr_t addr, guint16 port)
 	g_assert(found);
 	g_assert(!dme->fw_entry);
 
-	g_hash_table_remove(dm->by_host, &packed);	/* And from hash table */
-	wfree_packed_host(key, NULL);
+	htable_remove(dm->by_host, &packed);	/* And from hash table */
+	wfree_packed_host(deconstify_pointer(key), NULL);
 
 	dmesh_entry_free(dme);
 }
@@ -704,7 +710,7 @@ dm_remove(struct dmesh *dm, const host_addr_t addr, guint16 port)
  * Is the SHA1 that of a finished file (either shared in the library or
  * seeded after completion)?
  */
-static gboolean
+static bool
 sha1_of_finished_file(const struct sha1 *sha1)
 {
 	const shared_file_t *sf = shared_file_by_sha1(sha1);
@@ -719,7 +725,7 @@ sha1_of_finished_file(const struct sha1 *sha1)
  * we get are only from uploaders.  To keep only the ones that are fresh-enough, 
  * reduce the lifetime of each entry.
  */
-static glong
+static long
 dm_lifetime(const struct dmesh *dm)
 {
 	g_assert(dm);
@@ -737,7 +743,7 @@ dm_expire(struct dmesh *dm)
 	GSList *expired = NULL;
 	GSList *sl;
 	time_t now = tm_time();
-	glong agemax;
+	long agemax;
 	list_iter_t *iter;
 
 	agemax = dm_lifetime(dm);
@@ -787,30 +793,26 @@ dm_expire(struct dmesh *dm)
 static void
 dmesh_dispose(const struct sha1 *sha1)
 {
-	gpointer key;
-	gpointer value;
-	gboolean found;
+	void *value;
+	bool found;
 	struct dmesh *dm;
 
-	found = g_hash_table_lookup_extended(mesh, sha1, &key, &value);
+	found = hikset_lookup_extended(mesh, sha1, &value);
 
 	dm = value;
 	g_assert(found);
 	g_assert(list_length(dm->entries) == 0);
 
-	/* Remove it from the hashtable before freeing key, just in case
-	 * that sha1 == key. */
-	g_hash_table_remove(mesh, sha1);
-	atom_sha1_free(key);
+	hikset_remove(mesh, sha1);
 	dm_free(dm);
 }
 
 /**
  * Remove entry from mesh due to a failed download attempt.
  */
-gboolean
-dmesh_remove(const struct sha1 *sha1, const host_addr_t addr, guint16 port,
-	guint idx, const char *name)
+bool
+dmesh_remove(const struct sha1 *sha1, const host_addr_t addr, uint16 port,
+	uint idx, const char *name)
 {
 	struct dmesh *dm;
 	dmesh_urlinfo_t info;
@@ -827,7 +829,7 @@ dmesh_remove(const struct sha1 *sha1, const host_addr_t addr, guint16 port,
 	 * Lookup SHA1 in the mesh to see if we already have entries for it.
 	 */
 
-	dm = g_hash_table_lookup(mesh, sha1);
+	dm = hikset_lookup(mesh, sha1);
 
 	if (dm == NULL)				/* Nothing for this SHA1 key */
 		return FALSE;
@@ -856,7 +858,7 @@ dmesh_count(const struct sha1 *sha1)
 
 	g_assert(sha1);
 
-	dm = g_hash_table_lookup(mesh, sha1);
+	dm = hikset_lookup(mesh, sha1);
 
 	/*
 	 * If we have an entry and the last update was done more than
@@ -894,10 +896,10 @@ dmesh_get(const struct sha1 *sha1)
 	 * than the one we're trying to add).
 	 */
 
-	dm =  g_hash_table_lookup(mesh, sha1);
+	dm =  hikset_lookup(mesh, sha1);
 	if (dm == NULL) {
 		dm = dm_alloc(sha1);
-		gm_hash_table_insert_const(mesh, atom_sha1_get(sha1), dm);
+		hikset_insert(mesh, dm);
 	} else {
 		dm_expire(dm);
 	}
@@ -915,16 +917,16 @@ dmesh_get(const struct sha1 *sha1)
  * @return whether the entry was added in the mesh, or was discarded because
  * it was the oldest record and we have enough already.
  */
-static gboolean
+static bool
 dmesh_raw_add(const struct sha1 *sha1, const dmesh_urlinfo_t *info,
-	time_t stamp, gboolean swarm)
+	time_t stamp, bool swarm)
 {
 	struct dmesh_entry *dme;
 	struct dmesh *dm;
 	time_t now = tm_time();
 	host_addr_t addr = info->addr;
-	guint16 port = info->port;
-	guint idx = info->idx;
+	uint16 port = info->port;
+	uint idx = info->idx;
 	const char *name = info->name;
 	struct packed_host packed;
 	const char *reason = NULL;
@@ -990,7 +992,7 @@ dmesh_raw_add(const struct sha1 *sha1, const dmesh_urlinfo_t *info,
 	 */
 
 	packed = host_pack(addr, port);
-	dme = g_hash_table_lookup(dm->by_host, &packed);
+	dme = htable_lookup(dm->by_host, &packed);
 
 	if (dme) {
 		/*
@@ -1044,7 +1046,7 @@ dmesh_raw_add(const struct sha1 *sha1, const dmesh_urlinfo_t *info,
 		list_append(dm->entries, dme);
 		dm->last_update = now;
 
-		g_hash_table_insert(dm->by_host, walloc_packed_host(addr, port), dme);
+		htable_insert(dm->by_host, walloc_packed_host(addr, port), dme);
 
 		if (list_length(dm->entries) == MAX_ENTRIES) {
 			struct dmesh_entry *oldest = list_head(dm->entries);
@@ -1071,7 +1073,7 @@ rejected:
 	if (GNET_PROPERTY(dmesh_debug) > 4)
 		g_debug("MESH %s: rejecting \"%s\", stamp=%u age=%u: %s",
 			sha1_base32(sha1),
-			dmesh_urlinfo_to_string(info), (guint) stamp,
+			dmesh_urlinfo_to_string(info), (uint) stamp,
 			(unsigned) delta_time(now, stamp),
 			reason);
 
@@ -1087,9 +1089,9 @@ rejected:
  * @return whether the entry was added in the mesh, or was discarded because
  * it was the oldest record and we have enough already.
  */
-static gboolean
+static bool
 dmesh_raw_fw_add(const struct sha1 *sha1, const dmesh_fwinfo_t *info,
-	time_t stamp, gboolean swarm)
+	time_t stamp, bool swarm)
 {
 	struct dmesh_entry *dme;
 	struct dmesh *dm;
@@ -1121,7 +1123,7 @@ dmesh_raw_fw_add(const struct sha1 *sha1, const dmesh_fwinfo_t *info,
 	 * See whether we knew something about this host already.
 	 */
 
-	dme = g_hash_table_lookup(dm->by_guid, info->guid);
+	dme = htable_lookup(dm->by_guid, info->guid);
 
 	if (dme) {
 		/*
@@ -1175,7 +1177,7 @@ dmesh_raw_fw_add(const struct sha1 *sha1, const dmesh_fwinfo_t *info,
 		list_append(dm->entries, dme);
 		dm->last_update = now;
 
-		gm_hash_table_insert_const(dm->by_guid, dme->e.fwh.guid, dme);
+		htable_insert(dm->by_guid, dme->e.fwh.guid, dme);
 
 		if (list_length(dm->entries) == MAX_ENTRIES) {
 			struct dmesh_entry *oldest = list_head(dm->entries);
@@ -1197,7 +1199,7 @@ rejected:
 	if (GNET_PROPERTY(dmesh_debug) > 4)
 		g_debug("MESH %s: rejecting \"%s\", stamp=%u age=%u: %s",
 			sha1_base32(sha1),
-			dmesh_fwinfo_to_string(info), (guint) stamp,
+			dmesh_fwinfo_to_string(info), (uint) stamp,
 			(unsigned) delta_time(now, stamp),
 			reason);
 
@@ -1207,9 +1209,9 @@ rejected:
 /**
  * Same as dmesh_raw_add(), but this is for public consumption.
  */
-gboolean
+bool
 dmesh_add(const struct sha1 *sha1, const host_addr_t addr,
-	guint16 port, guint idx, const char *name, time_t stamp)
+	uint16 port, uint idx, const char *name, time_t stamp)
 {
 	dmesh_urlinfo_t info;
 
@@ -1227,7 +1229,7 @@ dmesh_add(const struct sha1 *sha1, const host_addr_t addr,
  * Add addr:port as a known alternate location for given sha1.
  */
 void
-dmesh_add_alternate(const struct sha1 *sha1, host_addr_t addr, guint16 port)
+dmesh_add_alternate(const struct sha1 *sha1, host_addr_t addr, uint16 port)
 {
 	dmesh_urlinfo_t info;
 
@@ -1240,7 +1242,7 @@ dmesh_add_alternate(const struct sha1 *sha1, host_addr_t addr, guint16 port)
  */
 void
 dmesh_add_good_alternate(const struct sha1 *sha1,
-	host_addr_t addr, guint16 port)
+	host_addr_t addr, uint16 port)
 {
 	dmesh_urlinfo_t info;
 
@@ -1260,7 +1262,7 @@ dmesh_add_alternates(const struct sha1 *sha1, const gnet_host_vec_t *alt)
 	for (i = gnet_host_vec_count(alt) - 1; i >= 0; i--) {
 		struct gnutella_host host;
 		host_addr_t addr;
-		guint16 port;
+		uint16 port;
 
 		host = gnet_host_vec_get(alt, i);
 		addr = gnet_host_get_addr(&host);
@@ -1275,7 +1277,7 @@ dmesh_add_alternates(const struct sha1 *sha1, const gnet_host_vec_t *alt)
  * This inserts the location into the banned mesh.
  */
 void
-dmesh_remove_alternate(const struct sha1 *sha1, host_addr_t addr, guint16 port)
+dmesh_remove_alternate(const struct sha1 *sha1, host_addr_t addr, uint16 port)
 {
 	dmesh_remove(sha1, addr, port, URN_INDEX, NULL);
 }
@@ -1290,7 +1292,7 @@ dmesh_remove_alternate(const struct sha1 *sha1, host_addr_t addr, guint16 port)
  */
 void
 dmesh_negative_alt(const struct sha1 *sha1, host_addr_t reporter,
-	host_addr_t addr, guint16 port)
+	host_addr_t addr, uint16 port)
 {
 	struct dmesh *dm;
 	struct packed_host packed;
@@ -1300,13 +1302,13 @@ dmesh_negative_alt(const struct sha1 *sha1, host_addr_t reporter,
 	 * Lookup SHA1 in the mesh to see if we already have entries for it.
 	 */
 
-	dm = g_hash_table_lookup(mesh, sha1);
+	dm = hikset_lookup(mesh, sha1);
 
 	if (dm == NULL)				/* Nothing for this SHA1 key */
 		return;
 
 	packed = host_pack(addr, port);
-	dme = g_hash_table_lookup(dm->by_host, &packed);
+	dme = htable_lookup(dm->by_host, &packed);
 
 	if (dme == NULL)
 		return;
@@ -1343,21 +1345,21 @@ dmesh_negative_alt(const struct sha1 *sha1, host_addr_t reporter,
  */
 void
 dmesh_good_mark(const struct sha1 *sha1,
-	host_addr_t addr, guint16 port, gboolean good)
+	host_addr_t addr, uint16 port, bool good)
 {
 	struct dmesh *dm;
 	struct packed_host packed;
 	struct dmesh_entry *dme;
-	gboolean retried = FALSE;
+	bool retried = FALSE;
 
-	dm = g_hash_table_lookup(mesh, sha1);
+	dm = hikset_lookup(mesh, sha1);
 	if (dm == NULL)
 		return;			/* Weird, but it doesn't matter */
 
 	packed = host_pack(addr, port);
 
 retry:
-	dme = g_hash_table_lookup(dm->by_host, &packed);
+	dme = htable_lookup(dm->by_host, &packed);
 
 	if (dme == NULL) {
 		/*
@@ -1417,16 +1419,16 @@ retry:
  */
 static void
 dmesh_good_fw_mark(const struct sha1 *sha1,
-	const struct guid *guid, gboolean good)
+	const struct guid *guid, bool good)
 {
 	struct dmesh *dm;
 	struct dmesh_entry *dme;
 
-	dm = g_hash_table_lookup(mesh, sha1);
+	dm = hikset_lookup(mesh, sha1);
 	if (dm == NULL)
 		return;			/* Weird, but it doesn't matter */
 
-	dme = g_hash_table_lookup(dm->by_guid, guid);
+	dme = htable_lookup(dm->by_guid, guid);
 
 	if (dme == NULL)
 		return;
@@ -1498,7 +1500,7 @@ dmesh_add_good_firewalled(const struct sha1 *sha1, const struct guid *guid)
  */
 static size_t
 dmesh_urlinfo_to_string_buf(const dmesh_urlinfo_t *info, char *buf,
-	size_t len, gboolean *quoting)
+	size_t len, bool *quoting)
 {
 	size_t rw;
 	size_t maxslen = len - 1;			/* Account for trailing NUL */
@@ -1658,7 +1660,7 @@ static size_t
 dmesh_entry_url_stamp(const struct dmesh_entry *dme, char *buf, size_t size)
 {
 	size_t rw;
-	gboolean quoting;
+	bool quoting;
 
 	g_assert(!dme->fw_entry);
 	g_assert(size > 0);
@@ -1764,14 +1766,14 @@ dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, int hcnt)
 	int nselected;
 	int i;
 	int j;
-	gboolean complete_file;
+	bool complete_file;
 	list_iter_t *iter;
 
 	/*
 	 * Fetch the mesh entry for this SHA1.
 	 */
 
-	dm = g_hash_table_lookup(mesh, sha1);
+	dm = hikset_lookup(mesh, sha1);
 	if (dm == NULL)						/* SHA1 unknown */
 		return 0;
 
@@ -1878,7 +1880,7 @@ dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, int hcnt)
  */
 static size_t
 dmesh_fwalt_string(char *buf, size_t size,
-	const guid_t *guid, host_addr_t addr, guint16 port, sequence_t *proxies,
+	const guid_t *guid, host_addr_t addr, uint16 port, sequence_t *proxies,
 	host_net_t net)
 {
 	size_t rw;
@@ -1957,7 +1959,7 @@ int
 dmesh_alternate_location(const struct sha1 *sha1,
 	char *buf, size_t size, const host_addr_t addr,
 	time_t last_sent, const char *vendor,
-	fileinfo_t *fi, gboolean request, const struct guid *guid,
+	fileinfo_t *fi, bool request, const struct guid *guid,
 	host_net_t net)
 {
 	char url[1024];
@@ -1970,10 +1972,10 @@ dmesh_alternate_location(const struct sha1 *sha1,
 	GSList *by_addr;
 	size_t maxlinelen = 0;
 	header_fmt_t *fmt;
-	gboolean added;
+	bool added;
 	list_iter_t *iter;
-	gboolean complete_file;
-	gboolean can_share_partials;
+	bool complete_file;
+	bool can_share_partials;
 
 	g_assert(sha1);
 	g_assert(buf);
@@ -2021,7 +2023,7 @@ dmesh_alternate_location(const struct sha1 *sha1,
 	 *		 -- JA, 1/11/2003
 	 */
 
-	by_addr = g_hash_table_lookup(ban_mesh_by_sha1, sha1);
+	by_addr = htable_lookup(ban_mesh_by_sha1, sha1);
 
 	if (by_addr != NULL) {
 		fmt = header_fmt_make("X-Nalt", ", ", size, size / 3);
@@ -2067,7 +2069,7 @@ dmesh_alternate_location(const struct sha1 *sha1,
 	}
 
 	/* Find mesh entry for this SHA1 */
-	dm = (struct dmesh *) g_hash_table_lookup(mesh, sha1);
+	dm = hikset_lookup(mesh, sha1);
 
 	/*
 	 * Start filling the buffer.
@@ -2317,7 +2319,7 @@ dmesh_alternate_location(const struct sha1 *sha1,
 		struct dmesh_entry *dme = list_iter_next(iter);
 		sequence_t *proxies;
 		host_addr_t servent_addr;
-		guint16 servent_port;
+		uint16 servent_port;
 
 		if (!dme->fw_entry)
 			continue;
@@ -2413,12 +2415,12 @@ nomore:
  *
  * @return whether we successfully extracted the SHA1.
  */
-gboolean
+bool
 dmesh_collect_sha1(const char *value, struct sha1 *sha1)
 {
 	strtok_t *st;
 	const char *tok;
-	gboolean found = FALSE;
+	bool found = FALSE;
 
 	st = strtok_make_strip(value);
 
@@ -2446,7 +2448,7 @@ dmesh_collect_sha1(const char *value, struct sha1 *sha1)
  */
 static void
 dmesh_parse_addr_port_list(const struct sha1 *sha1, const char *value,
-	dmesh_add_cb func, gpointer udata)
+	dmesh_add_cb func, void *udata)
 {
 	const char *tls_hex, *p, *next;
 
@@ -2456,8 +2458,8 @@ dmesh_parse_addr_port_list(const struct sha1 *sha1, const char *value,
 	while (NULL != (p = next)) {
 		const char *start, *endptr;
 		host_addr_t addr;
-		guint16 port;
-		gboolean ok;
+		uint16 port;
+		bool ok;
 
 		start = skip_ascii_blanks(p);
 		if ('\0' == *start)
@@ -2501,8 +2503,8 @@ dmesh_parse_addr_port_list(const struct sha1 *sha1, const char *value,
 
 static void
 dmesh_collect_compact_locations_cback(
-	const struct sha1 *sha1, host_addr_t addr, guint16 port,
-	gpointer unused_udata)
+	const struct sha1 *sha1, host_addr_t addr, uint16 port,
+	void *unused_udata)
 {
 	(void) unused_udata;
 	(void) dmesh_add_alternate(sha1, addr, port);
@@ -2521,8 +2523,8 @@ dmesh_collect_compact_locations(const struct sha1 *sha1, const char *value)
 
 static void
 dmesh_collect_negative_locations_cback(
-	const struct sha1 *sha1, host_addr_t addr, guint16 port,
-	gpointer udata)
+	const struct sha1 *sha1, host_addr_t addr, uint16 port,
+	void *udata)
 {
 	host_addr_t *reporter = udata;
 
@@ -2551,17 +2553,17 @@ void
 dmesh_collect_locations(const struct sha1 *sha1, const char *value)
 {
 	const char *p = value;
-	guchar c;
+	uchar c;
 	time_t now = tm_time();
-	gboolean finished = FALSE;
+	bool finished = FALSE;
 
 	do {
 		const char *date_start, *url_start;
 		time_t stamp;
-		gboolean ok;
+		bool ok;
 		dmesh_urlinfo_t info;
-		gboolean skip_date;
-		gboolean in_quote;
+		bool skip_date;
+		bool in_quote;
 
 		/*
 		 * Find next space, colon or EOS (End of String).
@@ -2625,7 +2627,7 @@ dmesh_collect_locations(const struct sha1 *sha1, const char *value)
 		 * Parse URL.
 		 */
 
-		g_assert((guchar) *p == c);
+		g_assert((uchar) *p == c);
 
 		if (*url_start == '"') {				/* URL enclosed in quotes? */
 			url_start++;						/* Skip that needless quote */
@@ -2719,7 +2721,7 @@ dmesh_collect_locations(const struct sha1 *sha1, const char *value)
 		if (p != date_start) {
 			char *date;
 
-			g_assert((guchar) *p == c);
+			g_assert((uchar) *p == c);
 			date = h_strndup(date_start, p - date_start);
 			stamp = date2time(date, now);
 
@@ -2770,7 +2772,7 @@ dmesh_collect_locations(const struct sha1 *sha1, const char *value)
 			g_debug("MESH %s: %s \"%s\", stamp=%u age=%u",
 				sha1_base32(sha1),
 				ok ? "added" : "rejected",
-				dmesh_urlinfo_to_string(&info), (guint) stamp,
+				dmesh_urlinfo_to_string(&info), (uint) stamp,
 				(unsigned) delta_time(now, stamp));
 
 		if (c == '\0')				/* Reached end of string */
@@ -2801,7 +2803,7 @@ dmesh_alt_loc_fill(const struct sha1 *sha1, dmesh_urlinfo_t *buf, int count)
 	g_assert(buf);
 	g_assert(count > 0);
 
-	dm = g_hash_table_lookup(mesh, sha1);
+	dm = hikset_lookup(mesh, sha1);
 	if (dm == NULL)					/* SHA1 unknown */
 		return 0;
 
@@ -2833,9 +2835,9 @@ void
 dmesh_collect_fw_host(const struct sha1 *sha1, const char *value)
 {
 	struct guid guid;
-	gboolean seen_proxy = FALSE;
-	gboolean seen_guid = FALSE;
-	gboolean seen_pptls = FALSE;
+	bool seen_proxy = FALSE;
+	bool seen_guid = FALSE;
+	bool seen_pptls = FALSE;
 	time_t stamp = 0;
 	const char *tok;
 	strtok_t *st;
@@ -2864,7 +2866,7 @@ dmesh_collect_fw_host(const struct sha1 *sha1, const char *value)
 
 	while ((tok = strtok_next(st, ";"))) {
 		host_addr_t addr;
-		guint16 port;
+		uint16 port;
 		gnet_host_t host;
 
 		/* GUID is the first item we expect */
@@ -2975,7 +2977,7 @@ dmesh_check_results_set(gnet_results_set_t *rs)
 	for (sl = rs->records; sl; sl = g_slist_next(sl)) {
 		gnet_record_t *rc = sl->data;
 		dmesh_urlinfo_t info;
-		gboolean has = FALSE;
+		bool has = FALSE;
 
 		if (rc->sha1 == NULL)
 			continue;
@@ -2988,7 +2990,7 @@ dmesh_check_results_set(gnet_results_set_t *rs)
 		 * sharing this SHA1.
 		 */
 
-		has = NULL != g_hash_table_lookup(mesh, rc->sha1);
+		has = hikset_contains(mesh, rc->sha1);
 
 		if (!has) {
 			const shared_file_t *sf = shared_file_by_sha1(rc->sha1);
@@ -3082,14 +3084,13 @@ dmesh_multiple_downloads(const struct sha1 *sha1,
  * Store key/value pair in file.
  */
 static void
-dmesh_store_kv(gpointer key, gpointer value, gpointer udata)
+dmesh_store_kv(void *value, void *udata)
 {
 	const struct dmesh *dm = value;
 	FILE *out = udata;
 	list_iter_t *iter;
 
-	g_assert(key);
-	fprintf(out, "%s\n", sha1_base32(key));
+	fprintf(out, "%s\n", sha1_base32(dm->sha1));
 
 	iter = list_iter_before_head(dm->entries);
 
@@ -3113,8 +3114,8 @@ typedef void (*header_func_t)(FILE *out);
  * The storing callback for each item is `store_cb'.
  */
 static void
-dmesh_store_hash(const char *what, GHashTable *hash, const char *file,
-	header_func_t header_cb, GHFunc store_cb)
+dmesh_store_hikset(const char *what, hikset_t *hash, const char *file,
+	header_func_t header_cb, data_fn_t store_cb)
 {
 	FILE *out;
 	file_path_t fp;
@@ -3126,7 +3127,7 @@ dmesh_store_hash(const char *what, GHashTable *hash, const char *file,
 		return;
 
 	header_cb(out);
-	g_hash_table_foreach(hash, store_cb, out);
+	hikset_foreach(hash, store_cb, out);
 
 	file_config_close(out, &fp);
 }
@@ -3156,7 +3157,7 @@ dmesh_header_print(FILE *out)
 void
 dmesh_store(void)
 {
-	dmesh_store_hash("download mesh",
+	dmesh_store_hikset("download mesh",
 		mesh, dmesh_file, dmesh_header_print, dmesh_store_kv);
 }
 
@@ -3170,8 +3171,8 @@ dmesh_retrieve(void)
 	FILE *f;
 	char tmp[4096];
 	struct sha1 sha1;
-	gboolean has_sha1 = FALSE;
-	gboolean skip = FALSE, truncated = FALSE;
+	bool has_sha1 = FALSE;
+	bool skip = FALSE, truncated = FALSE;
 	int line = 0;
 	file_path_t fp[1];
 
@@ -3243,15 +3244,13 @@ dmesh_retrieve(void)
  * Store key/value pair in file.
  */
 static void
-dmesh_ban_store_kv(gpointer key, gpointer value, gpointer udata)
+dmesh_ban_store_kv(void *value, void *udata)
 {
 	const struct dmesh_banned *dmb = value;
 	FILE *out = udata;
 
-	g_assert(key == dmb->info);
-
 	fprintf(out, "%lu %s\n",
-		(gulong) dmb->created, dmesh_urlinfo_to_string(dmb->info));
+		(ulong) dmb->created, dmesh_urlinfo_to_string(dmb->info));
 }
 
 /**
@@ -3274,7 +3273,7 @@ dmesh_ban_header_print(FILE *out)
 void
 dmesh_ban_store(void)
 {
-	dmesh_store_hash("banned mesh",
+	dmesh_store_hikset("banned mesh",
 		ban_mesh, dmesh_ban_file, dmesh_ban_header_print, dmesh_ban_store_kv);
 }
 
@@ -3343,29 +3342,24 @@ dmesh_ban_retrieve(void)
 /**
  * Free key/value pair in download mesh hash.
  */
-static gboolean
-dmesh_free_kv(gpointer key, gpointer value, gpointer unused_udata)
+static void
+dmesh_free_kv(void *value, void *unused_udata)
 {
 	struct dmesh *dm = value;
 
 	(void) unused_udata;
-	atom_sha1_free(key);
 	dm_free(dm);
-
-	return TRUE;
 }
 
 /**
  * Prepend the value to the list, given by reference.
  */
 static void
-dmesh_ban_prepend_list(gpointer key, gpointer value, gpointer user)
+dmesh_ban_prepend_list(void *value, void *user)
 {
 	struct dmesh_banned *dmb = value;
 	GSList **listref = user;
 
-	g_assert(key == dmb->info);
-
 	*listref = g_slist_prepend(*listref, dmb);
 }
 
@@ -3381,8 +3375,8 @@ dmesh_close(void)
 	dmesh_store();
 	dmesh_ban_store();
 
-	g_hash_table_foreach_remove(mesh, dmesh_free_kv, NULL);
-	gm_hash_table_destroy_null(&mesh);
+	hikset_foreach(mesh, dmesh_free_kv, NULL);
+	hikset_free_null(&mesh);
 
 	/*
 	 * Construct a list of banned mesh entries to remove, then manually
@@ -3390,7 +3384,7 @@ dmesh_close(void)
 	 * and `ban_mesh_by_sha1' as well.
 	 */
 
-	g_hash_table_foreach(ban_mesh, dmesh_ban_prepend_list, &banned);
+	hikset_foreach(ban_mesh, dmesh_ban_prepend_list, &banned);
 
 	for (sl = banned; sl; sl = g_slist_next(sl)) {
 		struct dmesh_banned *dmb = sl->data;
@@ -3399,8 +3393,8 @@ dmesh_close(void)
 	}
 
 	gm_slist_free_null(&banned);
-	gm_hash_table_destroy_null(&ban_mesh);
-	gm_hash_table_destroy_null(&ban_mesh_by_sha1);
+	hikset_free_null(&ban_mesh);
+	htable_free_null(&ban_mesh_by_sha1);
 
 	cq_free_null(&dmesh_cq);
 }
