@@ -59,6 +59,10 @@
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
 #include "lib/halloc.h"
+#include "lib/hevset.h"
+#include "lib/hikset.h"
+#include "lib/hset.h"
+#include "lib/htable.h"
 #include "lib/nid.h"
 #include "lib/stringify.h"
 #include "lib/tm.h"
@@ -133,33 +137,33 @@ typedef enum {
  */
 typedef struct dquery {
 	dquery_magic_t magic;
-	guint32 flags;			/**< Operational flags */
+	uint32 flags;			/**< Operational flags */
 	struct nid *node_id;	/**< ID of the node that originated the query */
 	struct nid qid;			/**< Unique query ID, to detect ghosts */
 	gnet_search_t sh;		/**< Search handle, if node ID = NODE_ID_SELF */
 	pmsg_t *mb;				/**< The search messsage "template" */
 	query_hashvec_t *qhv;	/**< Query hash vector for QRP filtering */
-	GHashTable *queried;	/**< Contains node IDs that we queried so far */
-	GHashTable *enqueued;	/**< Contains node IDs with enqueued queries */
+	htable_t *queried;		/**< Contains node IDs that we queried so far */
+	hset_t *enqueued;		/**< Contains node IDs with enqueued queries */
 	const struct guid *lmuid;/**< For proxied query: the original leaf MUID */
-	guint16 query_flags;	/**< Flags from the marked query speed field */
-	guint8 ttl;				/**< Initial query TTL */
-	guint32 horizon;		/**< Theoretical horizon reached thus far */
-	guint32 up_sent;		/**< # of UPs to which we really sent our query */
-	guint32 last_status;	/**< How many UP queried last time we got status */
-	guint32 pending;		/**< Pending query messages not ACK'ed yet by mq */
-	guint32 max_results;	/**< Max results we're targetting for */
-	guint32 fin_results;	/**< # of results terminating leaf-guided query */
-	guint32 oob_results;	/**< Amount of unclaimed OOB results reported */
-	guint32 results;		/**< Results we got so far for the query */
-	guint32 linger_results;	/**< Results we got whilst lingering */
-	guint32 new_results;	/**< New we got since last query status request */
-	guint32 kept_results;	/**< Results they say they kept after filtering */
-	guint32 result_timeout;	/**< The current timeout for getting results */
-	guint32 stat_timeouts;	/**< The amount of status request timeouts we had */
+	uint16 query_flags;		/**< Flags from the marked query speed field */
+	uint8 ttl;				/**< Initial query TTL */
+	uint32 horizon;			/**< Theoretical horizon reached thus far */
+	uint32 up_sent;			/**< # of UPs to which we really sent our query */
+	uint32 last_status;		/**< How many UP queried last time we got status */
+	uint32 pending;			/**< Pending query messages not ACK'ed yet by mq */
+	uint32 max_results;		/**< Max results we're targetting for */
+	uint32 fin_results;		/**< # of results terminating leaf-guided query */
+	uint32 oob_results;		/**< Amount of unclaimed OOB results reported */
+	uint32 results;			/**< Results we got so far for the query */
+	uint32 linger_results;	/**< Results we got whilst lingering */
+	uint32 new_results;		/**< New we got since last query status request */
+	uint32 kept_results;	/**< Results they say they kept after filtering */
+	uint32 result_timeout;	/**< The current timeout for getting results */
+	uint32 stat_timeouts;	/**< The amount of status request timeouts we had */
 	cevent_t *expire_ev;	/**< Callout queue global expiration event */
 	cevent_t *results_ev;	/**< Callout queue results expiration event */
-	gpointer alive;			/**< Alive ping stats for computing timeouts */
+	void *alive;			/**< Alive ping stats for computing timeouts */
 	time_t start;			/**< Time at which it started */
 	time_t stop;			/**< Time at which it was terminated */
 	struct next_up *nv;		/**< Previous "next UP vector" */
@@ -184,14 +188,14 @@ enum {
  * This table keeps track of all the dynamic query objects that we have
  * created and which are alive.
  */
-static GHashTable *dqueries;
+static hevset_t *dqueries;
 
 /**
  * This table keeps track of all the dynamic query objects created
  * for a given node ID.  The key is the node ID (converted to a pointer) and
  * the value is a GSList containing all the queries for that node.
  */
-static GHashTable *by_node_id;
+static htable_t *by_node_id;
 
 /**
  * This table keeps track of the association between a MUID and the
@@ -200,7 +204,7 @@ static GHashTable *by_node_id;
  *
  * The keys are MUIDs (GUID atoms), the values are the dquery_t object.
  */
-static GHashTable *by_muid;
+static htable_t *by_muid;
 
 /**
  * This table keeps track of the association between a leaf MUID and the
@@ -208,7 +212,7 @@ static GHashTable *by_muid;
  * account them for the relevant query (since for OOB-proxied query, the
  * MUID we'll get is the one the leaf knows about).
  */
-static GHashTable *by_leaf_muid;
+static hikset_t *by_leaf_muid;
 
 /**
  * Information about query messages sent.
@@ -227,9 +231,9 @@ static GHashTable *by_leaf_muid;
 struct dq_pmsg_info {
 	struct nid qid;		/**< Query ID of the dynamic query */
 	struct nid *node_id;/**< The ID of the node we sent it to */
-	guint16 degree;		/**< The advertised degree of the destination node */
-	guint8 ttl;			/**< The TTL used for that query */
-	guint8 probe;		/**< Whether query is just a probe */
+	uint16 degree;		/**< The advertised degree of the destination node */
+	uint8 ttl;			/**< The TTL used for that query */
+	uint8 probe;		/**< Whether query is just a probe */
 };
 
 /*
@@ -243,7 +247,7 @@ struct dq_pmsg_info {
 #define MAX_DEGREE		50
 #define MAX_TTL			5
 
-static guint32 hosts[MAX_DEGREE][MAX_TTL];	/**< Pre-computed horizon */
+static uint32 hosts[MAX_DEGREE][MAX_TTL];	/**< Pre-computed horizon */
 
 static void dq_send_next(dquery_t *dq);
 static void dq_terminate(dquery_t *dq);
@@ -289,7 +293,7 @@ fill_hosts(void)
  * We adjust the horizon by DQ_FUZZY_FACTOR, assuming that at each hop there
  * is deperdition due to flow-control, network cycles, etc...
  */
-static guint32
+static uint32
 dq_get_horizon(int degree, int ttl)
 {
 	int i;
@@ -308,7 +312,7 @@ dq_get_horizon(int degree, int ttl)
  * Compute amount of results "kept" for the query, if we have this
  * information available.
  */
-static guint32
+static uint32
 dq_kept_results(dquery_t *dq)
 {
 	dquery_check(dq);
@@ -345,8 +349,8 @@ dq_kept_results(dquery_t *dq)
 static unsigned
 dq_select_ttl(dquery_t *dq, gnutella_node_t *node, int connections)
 {
-	guint32 needed;
-	guint32 results;
+	uint32 needed;
+	uint32 results;
 	double results_per_up;
 	double hosts_to_reach;
 	double hosts_to_reach_via_node;
@@ -393,8 +397,8 @@ dq_select_ttl(dquery_t *dq, gnutella_node_t *node, int connections)
  * @param probe	  whether the query is just a probe, with a lower TTL
  */
 static struct dq_pmsg_info *
-dq_pmi_alloc(dquery_t *dq, guint16 degree, guint8 ttl,
-	const struct nid *node_id, gboolean probe)
+dq_pmi_alloc(dquery_t *dq, uint16 degree, uint8 ttl,
+	const struct nid *node_id, bool probe)
 {
 	struct dq_pmsg_info *pmi;
 	const struct nid *key = nid_ref(node_id);	
@@ -413,7 +417,7 @@ dq_pmi_alloc(dquery_t *dq, guint16 degree, guint8 ttl,
 	 * Remember that we've queried this node, and with which TTL.
 	 */
 
-	gm_hash_table_insert_const(dq->queried, key,
+	htable_insert(dq->queried, key,
 		uint_to_pointer(ttl | (pmi->probe ? DQ_TTL_PROBE : 0)));
 
 	/*
@@ -421,7 +425,7 @@ dq_pmi_alloc(dquery_t *dq, guint16 degree, guint8 ttl,
 	 * forbid further sending should this query be stuck in the TX queue.
 	 */
 
-	gm_hash_table_insert_const(dq->enqueued, pmi->node_id, int_to_pointer(1));
+	hset_insert(dq->enqueued, pmi->node_id);
 
 	return pmi;
 }
@@ -448,7 +452,7 @@ dq_alive(struct nid qid)
 	/* NOTE: dqueries might have been freed already, as dq_pmsg_free()
 	 *		 might still call this function after dq_close().
 	 */
-	dq = dqueries ? g_hash_table_lookup(dqueries, &qid) : NULL;
+	dq = dqueries ? hevset_lookup(dqueries, &qid) : NULL;
 	if (dq) {
 		dquery_check(dq);
 	}
@@ -459,7 +463,7 @@ dq_alive(struct nid qid)
  * Free routine for an extended message block.
  */
 static void
-dq_pmsg_free(pmsg_t *mb, gpointer arg)
+dq_pmsg_free(pmsg_t *mb, void *arg)
 {
 	struct dq_pmsg_info *pmi = arg;
 	dquery_t *dq;
@@ -482,12 +486,13 @@ dq_pmsg_free(pmsg_t *mb, gpointer arg)
 
 	g_assert(dq->pending > 0);
 	dq->pending--;
-	g_hash_table_remove(dq->enqueued, pmi->node_id);
+	hset_remove(dq->enqueued, pmi->node_id);
 
 	if (!pmsg_was_sent(mb)) {
-		struct nid *key;
-		gboolean found;
-		void *knid, *ttlv;
+		const struct nid *key;
+		bool found;
+		const void *knid;
+		void *ttlv;
 		unsigned ttl;
 
 		/*
@@ -496,8 +501,7 @@ dq_pmsg_free(pmsg_t *mb, gpointer arg)
 		 * make it through the network.
 		 */
 
-		found = g_hash_table_lookup_extended(dq->queried, pmi->node_id,
-			&knid, &ttlv);
+		found = htable_lookup_extended(dq->queried, pmi->node_id, &knid, &ttlv);
 
 		g_assert(found);		/* Or something is seriously corrupted */
 
@@ -507,10 +511,10 @@ dq_pmsg_free(pmsg_t *mb, gpointer arg)
 		g_assert(pmi->ttl >= (ttl & DQ_TTL_MASK));
 
 		if ((ttl & DQ_TTL_MASK) > 1) {
-			g_hash_table_replace(dq->queried, key,
+			htable_insert(dq->queried, key,
 				uint_to_pointer((ttl - 1) | (ttl & DQ_TTL_PROBE)));
 		} else {
-			g_hash_table_remove(dq->queried, key);
+			htable_remove(dq->queried, key);
 			nid_unref(key);
 		}
 
@@ -599,7 +603,7 @@ dq_pmsg_by_ttl(dquery_t *dq, int ttl)
 	 * Patch the TTL in the new data buffer.
 	 */
 	{
-		gnutella_header_t *header = cast_to_gpointer(pdata_start(db));
+		gnutella_header_t *header = cast_to_pointer(pdata_start(db));
 		gnutella_header_set_ttl(header, ttl);
 	}
 
@@ -723,7 +727,7 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 {
 	const GSList *sl;
 	int i = 0;
-	GHashTable *old = NULL;
+	htable_t *old = NULL;
 
 	dquery_check(dq);
 
@@ -736,11 +740,11 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 	if (dq->nv != NULL) {
 		int j;
 
-		old = g_hash_table_new(nid_hash, nid_equal);
+		old = htable_create_any(nid_hash, NULL, nid_equal);
 
 		for (j = 0; j < dq->nv_found; j++) {
 			struct next_up *nup = &dq->nv[j];
-			gm_hash_table_insert_const(old, nup->node_id, nup);
+			htable_insert(old, nup->node_id, nup);
 		}
 	}
 
@@ -751,8 +755,9 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 	GM_SLIST_FOREACH(node_all_ultranodes(), sl) {
 		struct next_up *nup, *old_nup;
 		struct gnutella_node *n;
-		void *knid, *ttlv;
-		gboolean found;
+		const void *knid;
+		void *ttlv;
+		bool found;
 
 		if (i >= ncount)
 			break;
@@ -771,7 +776,7 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 		 * Skip node if we already have a pending query.
 		 */
 
-		if (gm_hash_table_contains(dq->enqueued, NODE_ID(n)))
+		if (hset_contains(dq->enqueued, NODE_ID(n)))
 			continue;
 
 		/*
@@ -783,8 +788,7 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 		if (n->received == 0)
 			continue;
 
-		found = g_hash_table_lookup_extended(dq->queried, NODE_ID(n),
-			&knid, &ttlv);
+		found = htable_lookup_extended(dq->queried, NODE_ID(n), &knid, &ttlv);
 
 		if (found) {
 			if (!(n->attrs & NODE_A_DQ_PROBE))
@@ -815,10 +819,7 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 		nup->node_id = nid_ref(NODE_ID(n)); /* To be able to compare */
 		nup->qhv = dq->qhv;
 
-		if (
-			old &&
-			(old_nup = g_hash_table_lookup(old, nup->node_id))
-		) {
+		if (old && NULL != (old_nup = htable_lookup(old, nup->node_id))) {
 			g_assert(nid_equal(NODE_ID(n), old_nup->node_id));
 			nup->can_route = old_nup->can_route;
 		} else
@@ -832,7 +833,7 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 	if (old) {
 		g_assert(dq->nv != NULL);
 		dq_free_next_up(dq);
-		g_hash_table_destroy(old);
+		htable_free_null(&old);
 	}
 
 	dq->nv = nv;
@@ -849,11 +850,11 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 static void
 dq_sendto_leaves(dquery_t *dq, gnutella_node_t *source)
 {
-	gconstpointer head;
+	const void *head;
 	GSList *nodes;
 
 	dquery_check(dq);
-	head = cast_to_gconstpointer(pmsg_start(dq->mb));
+	head = cast_to_constpointer(pmsg_start(dq->mb));
 
 	/*
 	 * NB: In order to avoid qrt_build_query_target() selecting neighbouring
@@ -875,14 +876,13 @@ dq_sendto_leaves(dquery_t *dq, gnutella_node_t *source)
 	g_slist_free(nodes);
 }
 
-static gboolean
-free_node_id(gpointer key, gpointer unused_value, gpointer unused_udata)
+static void
+free_node_id(const void *key, void *unused_value, void *unused_udata)
 {
 	(void) unused_value;
 	(void) unused_udata;
 
 	nid_unref(key);
-	return TRUE;
 }
 
 /**
@@ -895,7 +895,7 @@ dq_free(dquery_t *dq)
 
 	dquery_check(dq);
 	g_assert((dq->flags & DQ_F_EXITING) ||
-		g_hash_table_lookup(dqueries, &dq->qid) == dq);
+		hevset_lookup(dqueries, &dq->qid) == dq);
 
 	if (GNET_PROPERTY(dq_debug) > 2)
 		g_debug("DQ[%s] %s(%d secs; +%d secs) node #%s ending: "
@@ -939,9 +939,9 @@ dq_free(dquery_t *dq)
 			gnet_stats_count_general(GNR_DYN_QUERIES_LINGER_RESULTS, 1);
 	}
 
-	g_hash_table_foreach_remove(dq->queried, free_node_id, NULL);
-	gm_hash_table_destroy_null(&dq->queried);
-	gm_hash_table_destroy_null(&dq->enqueued);
+	htable_foreach(dq->queried, free_node_id, NULL);
+	htable_free_null(&dq->queried);
+	hset_free_null(&dq->enqueued);
 
 	qhvec_free(dq->qhv);
 	dq_free_next_up(dq);
@@ -954,7 +954,7 @@ dq_free(dquery_t *dq)
 	}
 
 	if (!(dq->flags & DQ_F_EXITING))
-		g_hash_table_remove(dqueries, &dq->qid);
+		hevset_remove(dqueries, &dq->qid);
 
 	/*
 	 * Remove query from the `by_node_id' table but only if the node ID
@@ -967,12 +967,11 @@ dq_free(dquery_t *dq)
 	 */
 
 	if (!node_id_self(dq->node_id) && !(dq->flags & DQ_F_ID_CLEANING)) {
-		gpointer value;
-		gboolean found;
+		void *value;
+		bool found;
 		GSList *list;
 
-		found = g_hash_table_lookup_extended(by_node_id, dq->node_id,
-					NULL, &value);
+		found = htable_lookup_extended(by_node_id, dq->node_id, NULL, &value);
 
 		if (!found) {
 			g_error("%s: missing %s", G_STRLOC, nid_to_string(dq->node_id));
@@ -983,15 +982,14 @@ dq_free(dquery_t *dq)
 
 		if (list == NULL) {
 			/* Last item removed, get rid of the entry */
-			g_hash_table_remove(by_node_id, dq->node_id);
-			g_assert(!g_hash_table_lookup_extended(by_node_id, dq->node_id,
-						NULL, NULL));
+			htable_remove(by_node_id, dq->node_id);
+			g_assert(!htable_contains(by_node_id, dq->node_id));
 		} else if (list != value) {
 			dquery_t *key = list->data;
 
 			dquery_check(key);
-			gm_hash_table_replace_const(by_node_id, key->node_id, list);
-			g_assert(g_hash_table_lookup(by_node_id, dq->node_id) == list);
+			htable_insert(by_node_id, key->node_id, list);
+			g_assert(htable_lookup(by_node_id, dq->node_id) == list);
 		}
 	}
 
@@ -999,15 +997,16 @@ dq_free(dquery_t *dq)
 	 * Remove query's MUID.
 	 */
 	{
-		gpointer key, value;
-		gboolean found;
+		const void *key;
+		void *value;
+		bool found;
 
-		found = g_hash_table_lookup_extended(by_muid,
-					gnutella_header_get_muid(pmsg_start(dq->mb)), &key, &value);
+		found = htable_lookup_extended(by_muid,
+				gnutella_header_get_muid(pmsg_start(dq->mb)), &key, &value);
 
 		if (found) {		/* Could be missing if a MUID conflict occurred */
 			if (value == dq) {	/* Make sure it's for us in case of conflicts */
-				g_hash_table_remove(by_muid, key);
+				htable_remove(by_muid, key);
 				atom_guid_free(key);
 			}
 		}
@@ -1018,13 +1017,12 @@ dq_free(dquery_t *dq)
 	 */
 
 	if (dq->lmuid != NULL) {
-		gpointer key, value;
-		gboolean found;
+		void *value;
+		bool found;
 
-		found = g_hash_table_lookup_extended(
-			by_leaf_muid, dq->lmuid, &key, &value);
+		found = hikset_lookup_extended(by_leaf_muid, dq->lmuid, &value);
 		if (found && value == dq)
-			g_hash_table_remove(by_leaf_muid, key);
+			hikset_remove(by_leaf_muid, dq->lmuid);
 		atom_guid_free(dq->lmuid);
 	}
 
@@ -1040,7 +1038,7 @@ dq_free(dquery_t *dq)
  * Callout queue callback invoked when the dynamic query has expired.
  */
 static void
-dq_expired(cqueue_t *unused_cq, gpointer obj)
+dq_expired(cqueue_t *unused_cq, void *obj)
 {
 	dquery_t *dq = obj;
 
@@ -1076,14 +1074,14 @@ dq_expired(cqueue_t *unused_cq, gpointer obj)
  * Callout queue callback invoked when the result timer has expired.
  */
 static void
-dq_results_expired(cqueue_t *unused_cq, gpointer obj)
+dq_results_expired(cqueue_t *unused_cq, void *obj)
 {
 	dquery_t *dq = obj;
 	gnutella_node_t *n;
 	int timeout;
-	guint32 avg;
-	guint32 last;
-	gboolean was_waiting = FALSE;
+	uint32 avg;
+	uint32 last;
+	bool was_waiting = FALSE;
 
 	(void) unused_cq;
 	dquery_check(dq);
@@ -1290,8 +1288,8 @@ node_mq_cmp(const void *np1, const void *np2)
 static int
 node_mq_qrp_cmp(const void *np1, const void *np2)
 {
-	struct next_up *nu1 = deconstify_gpointer(np1);
-	struct next_up *nu2 = deconstify_gpointer(np2);
+	struct next_up *nu1 = deconstify_pointer(np1);
+	struct next_up *nu2 = deconstify_pointer(np2);
 	const gnutella_node_t *n1, *n2;
 	int qs1 = nu1->queue_pending;
 	int qs2 = nu2->queue_pending;
@@ -1341,13 +1339,13 @@ node_mq_qrp_cmp(const void *np1, const void *np2)
  * adjusted down accordingly.
  */
 static void
-dq_send_query(dquery_t *dq, gnutella_node_t *n, int ttl, gboolean probe)
+dq_send_query(dquery_t *dq, gnutella_node_t *n, int ttl, bool probe)
 {
 	struct dq_pmsg_info *pmi;
 	pmsg_t *mb;
 
 	dquery_check(dq);
-	g_assert(!g_hash_table_lookup(dq->queried, NODE_ID(n)));
+	g_assert(!htable_contains(dq->queried, NODE_ID(n)));
 	g_assert(NODE_IS_WRITABLE(n));
 
 	pmi = dq_pmi_alloc(dq, n->degree, MIN(n->max_ttl, ttl), NODE_ID(n), probe);
@@ -1400,8 +1398,8 @@ dq_send_next(dquery_t *dq)
 	int found;
 	int timeout;
 	int i;
-	gboolean sent = FALSE;
-	guint32 results;
+	bool sent = FALSE;
+	uint32 results;
 
 	dquery_check(dq);
 	g_assert(dq->results_ev == NULL);
@@ -1519,7 +1517,8 @@ dq_send_next(dquery_t *dq)
 	for (i = 0; i < found; i++) {
 		struct gnutella_node *node;
 		struct nid *nid = nv[i].node_id;
-		void *knid, *ttlv;
+		const void *knid;
+		void *ttlv;
 		unsigned ttl;
 
 		node = node_by_id(nid);
@@ -1530,7 +1529,7 @@ dq_send_next(dquery_t *dq)
 		 * a probe and the TTL is greater now.
 		 */
 
-		if (g_hash_table_lookup_extended(dq->queried, nid, &knid, &ttlv)) {
+		if (htable_lookup_extended(dq->queried, nid, &knid, &ttlv)) {
 			unsigned prev_ttl = pointer_to_uint(ttlv);
 
 			if (prev_ttl & DQ_TTL_PROBE) {
@@ -1544,7 +1543,7 @@ dq_send_next(dquery_t *dq)
 						nid_to_string2(NODE_ID(node)),
 						node_infostr(node), ttl, sttl);
 				}
-				g_hash_table_remove(dq->queried, knid);
+				htable_remove(dq->queried, knid);
 				nid_unref(knid);
 				g_assert(ttl > 1);
 			} else {
@@ -1592,7 +1591,7 @@ dq_send_next(dquery_t *dq)
 
 	timeout = dq->result_timeout;
 	if (dq->pending > 1) {
-		guint t = timeout;
+		uint t = timeout;
 
 		t += (dq->pending - 1) * DQ_PENDING_TIMEOUT;
 		timeout = t > UNSIGNED(timeout) ? t : INT_MAX;
@@ -1703,13 +1702,14 @@ dquery_id_create(void)
 static void
 dq_common_init(dquery_t *dq)
 {
-	gconstpointer head;
+	const void *head;
 	const guid_t *muid;
+	void *value;
 
 	dquery_check(dq);
 	dq->qid = dquery_id_create();
-	dq->queried = g_hash_table_new(nid_hash, nid_equal);
-	dq->enqueued = g_hash_table_new(nid_hash, nid_equal);
+	dq->queried = htable_create_any(nid_hash, NULL, nid_equal);
+	dq->enqueued = hset_create_any(nid_hash, NULL, nid_equal);
 	dq->result_timeout = DQ_QUERY_TIMEOUT;
 	dq->start = tm_time();
 
@@ -1724,19 +1724,17 @@ dq_common_init(dquery_t *dq)
 	 * Record the query as being "alive".
 	 */
 
-	g_hash_table_insert(dqueries, &dq->qid, dq);
+	hevset_insert_key(dqueries, &dq->qid);
 
 	/*
 	 * If query is not for the local node, insert it in `by_node_id'.
 	 */
 
 	if (!(dq->flags & DQ_F_LOCAL)) {
-		gpointer value;
-		gboolean found;
+		bool found;
 		GSList *list;
 
-		found = g_hash_table_lookup_extended(by_node_id, dq->node_id,
-					NULL, &value);
+		found = htable_lookup_extended(by_node_id, dq->node_id, NULL, &value);
 
 		if (found) {
 			list = value;
@@ -1744,8 +1742,8 @@ dq_common_init(dquery_t *dq)
 			g_assert(list == value);		/* Head not changed */
 		} else {
 			list = g_slist_prepend(NULL, dq);
-			gm_hash_table_replace_const(by_node_id, dq->node_id, list);
-			g_assert(g_hash_table_lookup(by_node_id, dq->node_id) == list);
+			htable_insert(by_node_id, dq->node_id, list);
+			g_assert(htable_lookup(by_node_id, dq->node_id) == list);
 		}
 	}
 
@@ -1753,14 +1751,19 @@ dq_common_init(dquery_t *dq)
 	 * Record the MUID of this query, warning if a conflict occurs.
 	 */
 
-	head = cast_to_gconstpointer(pmsg_start(dq->mb));
+	head = cast_to_constpointer(pmsg_start(dq->mb));
 	muid = gnutella_header_get_muid(head);
 
-	if (g_hash_table_lookup(by_muid, muid)) {
-		g_warning("conflicting MUID \"%s\" for dynamic query from %s, "
-			"ignoring.", guid_hex_str(muid), node_id_infostr(dq->node_id));
+	if (htable_lookup_extended(by_muid, muid, NULL, &value)) {
+		dquery_t *odq = value;
+		dquery_check(odq);
+		g_warning("ignoring conflicting MUID \"%s\" for dynamic query from %s, "
+			"already used by %s.",
+			guid_hex_str(muid), node_id_infostr(dq->node_id),
+			dq->node_id == odq->node_id ?
+				"same node" : node_id_infostr(odq->node_id));
 	} else {
-		gm_hash_table_insert_const(by_muid, atom_guid_get(muid), dq);
+		htable_insert(by_muid, atom_guid_get(muid), dq);
 	}
 
 	/*
@@ -1770,19 +1773,22 @@ dq_common_init(dquery_t *dq)
 	 */
 
 	if (dq->lmuid != NULL) {
-		if (g_hash_table_lookup(by_leaf_muid, dq->lmuid)) {
+		if (hikset_lookup_extended(by_leaf_muid, dq->lmuid, &value)) {
+			dquery_t *odq = value;
+			dquery_check(odq);
 			g_warning("ignoring conflicting leaf MUID \"%s\" for "
-				"dynamic query from %s",
-				guid_hex_str(dq->lmuid), node_id_infostr(dq->node_id));
+				"dynamic query from %s, already used by %s",
+				guid_hex_str(dq->lmuid), node_id_infostr(dq->node_id),
+				dq->node_id == odq->node_id ?
+					"same node" : node_id_infostr(odq->node_id));
 		} else {
-			g_hash_table_insert(by_leaf_muid,
-				deconstify_gpointer(dq->lmuid), dq);
+			hikset_insert_key(by_leaf_muid, &dq->lmuid);
 		}
 	}
 
 	if (GNET_PROPERTY(dq_debug) > 1) {
-		gconstpointer packet;
-		guint16 flags;
+		const void *packet;
+		uint16 flags;
 
 		packet = pmsg_start(dq->mb);
 		flags = gnutella_msg_search_get_flags(packet);
@@ -1820,8 +1826,8 @@ void
 dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv, unsigned media_types)
 {
 	dquery_t *dq;
-	guint16 flags;
-	gboolean flags_valid;
+	uint16 flags;
+	bool flags_valid;
 	const struct guid *leaf_muid;
 
 	/* Query from leaf node */
@@ -1866,7 +1872,7 @@ dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv, unsigned media_types)
 		!(dq->flags & DQ_F_LEAF_GUIDED) &&
 		NULL == oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header))
 	) {
-		gboolean proxied = FALSE;
+		bool proxied = FALSE;
 		if (
 			!GNET_PROPERTY(is_udp_firewalled) &&
 			GNET_PROPERTY(proxy_oob_queries) &&
@@ -2044,7 +2050,7 @@ dq_launch_local(gnet_search_t handle, pmsg_t *mb, query_hashvec_t *qhv)
 	if (GNET_PROPERTY(dq_debug) > 1) {
 		const char *qstr = gnutella_msg_search_get_text(pmsg_start(dq->mb));
 		char *safe_qstr = hex_escape(qstr, FALSE);
-		guint16 qflags = gnutella_msg_search_get_flags(pmsg_start(dq->mb));
+		uint16 qflags = gnutella_msg_search_get_flags(pmsg_start(dq->mb));
 		g_debug("DQ local %squeries \"%s\" for %u hits",
 			(qflags & QUERY_F_OOB_REPLY) ?  "OOB-" : "",
 			safe_qstr, dq->max_results);
@@ -2076,14 +2082,14 @@ ignore:
 void
 dq_node_removed(const struct nid *node_id)
 {
-	gpointer value;
+	void *value;
 	GSList *sl;
 
-	if (!g_hash_table_lookup_extended(by_node_id, node_id, NULL, &value))
+	if (!htable_lookup_extended(by_node_id, node_id, NULL, &value))
 		return;		/* No dynamic query for this node */
 
-	g_hash_table_remove(by_node_id, node_id);
-	g_assert(!g_hash_table_lookup_extended(by_node_id, node_id, NULL, NULL));
+	htable_remove(by_node_id, node_id);
+	g_assert(!htable_contains(by_node_id, node_id));
 
 	GM_SLIST_FOREACH(value, sl) {
 		dquery_t *dq = sl->data;
@@ -2100,7 +2106,7 @@ dq_node_removed(const struct nid *node_id)
 		dq_free(dq);
 	}
 
-	g_assert(!g_hash_table_lookup_extended(by_node_id, node_id, NULL, NULL));
+	g_assert(!htable_contains(by_node_id, node_id));
 	g_slist_free(value);
 }
 
@@ -2118,15 +2124,14 @@ dq_node_removed(const struct nid *node_id)
  * @return FALSE if the query was explicitly cancelled by the user or if we
  * should not forward the results anyway.
  */
-static gboolean
-dq_count_results(const struct guid *muid,
-	int count, guint16 status, gboolean oob)
+static bool
+dq_count_results(const struct guid *muid, int count, uint16 status, bool oob)
 {
 	dquery_t *dq;
 
 	g_assert(count > 0);		/* Query hits with no result are bad! */
 
-	dq = g_hash_table_lookup(by_muid, muid);
+	dq = htable_lookup(by_muid, muid);
 
 	if (dq == NULL)
 		return TRUE;
@@ -2224,8 +2229,8 @@ dq_count_results(const struct guid *muid,
  * results should be dropped, TRUE otherwise.  In other words, returns
  * whether we should forward the results.
  */
-gboolean
-dq_got_results(const struct guid *muid, guint count, guint32 status)
+bool
+dq_got_results(const struct guid *muid, uint count, uint32 status)
 {
 	return dq_count_results(muid, count, status, FALSE);
 }
@@ -2237,7 +2242,7 @@ dq_got_results(const struct guid *muid, guint count, guint32 status)
  * @return FALSE if the query was explicitly cancelled by the user and
  * results should not be claimed.
  */
-gboolean
+bool
 dq_oob_results_ind(const struct guid *muid, int count)
 {
 	return dq_count_results(muid, count, 0, TRUE);
@@ -2250,14 +2255,14 @@ dq_oob_results_ind(const struct guid *muid, int count)
  * were finally claimed and parsed).
  */
 void
-dq_oob_results_got(const struct guid *muid, guint count)
+dq_oob_results_got(const struct guid *muid, uint count)
 {
 	dquery_t *dq;
 
 	/* Query hits with no result are bad! */
 	g_assert(count > 0 && count <= INT_MAX);
 
-	dq = g_hash_table_lookup(by_muid, muid);
+	dq = htable_lookup(by_muid, muid);
 
 	if (dq == NULL)
 		return;
@@ -2289,11 +2294,11 @@ dq_oob_results_got(const struct guid *muid, guint count)
  */
 void
 dq_got_query_status(const struct guid *muid,
-	const struct nid *node_id, guint16 kept)
+	const struct nid *node_id, uint16 kept)
 {
 	dquery_t *dq;
 
-	dq = g_hash_table_lookup(by_muid, muid);
+	dq = htable_lookup(by_muid, muid);
 
 	/*
 	 * Could be an OOB-proxied query, but the leaf does not know the MUID
@@ -2301,7 +2306,7 @@ dq_got_query_status(const struct guid *muid,
 	 */
 
 	if (dq == NULL)
-		dq = g_hash_table_lookup(by_leaf_muid, muid);
+		dq = hikset_lookup(by_leaf_muid, muid);
 
 	if (dq == NULL)
 		return;
@@ -2390,13 +2395,12 @@ struct cancel_context {
  * -- hash table iterator callback
  */
 static void
-dq_cancel_local(gpointer key, gpointer value, gpointer udata)
+dq_cancel_local(void *value, void *udata)
 {
 	struct cancel_context *ctx = udata;
 	dquery_t *dq = value;
 
 	dquery_check(dq);
-	g_assert(&dq->qid == key);
 
 	if ((dq->flags & DQ_F_LOCAL) && dq->sh == ctx->handle) {
 		ctx->cancelled = g_slist_prepend(ctx->cancelled, dq);
@@ -2415,7 +2419,7 @@ dq_search_closed(gnet_search_t handle)
 	ctx.handle = handle;
 	ctx.cancelled = NULL;
 
-	g_hash_table_foreach(dqueries, dq_cancel_local, &ctx);
+	hevset_foreach(dqueries, dq_cancel_local, &ctx);
 
 	GM_SLIST_FOREACH(ctx.cancelled, sl) {
 		dq_free(sl->data);
@@ -2435,12 +2439,12 @@ dq_search_closed(gnet_search_t handle)
  * @return TRUE if the query is still active, FALSE if it does not exist
  * any more, in which case nothing is returned into `wanted'.
  */
-gboolean
-dq_get_results_wanted(const struct guid *muid, guint32 *wanted)
+bool
+dq_get_results_wanted(const struct guid *muid, uint32 *wanted)
 {
 	dquery_t *dq;
 
-	dq = g_hash_table_lookup(by_muid, muid);
+	dq = htable_lookup(by_muid, muid);
 
 	if (dq == NULL)
 		return FALSE;
@@ -2449,7 +2453,7 @@ dq_get_results_wanted(const struct guid *muid, guint32 *wanted)
 	if (dq->flags & DQ_F_USR_CANCELLED)
 		*wanted = 0;
 	else {
-		guint32 kept = dq_kept_results(dq);
+		uint32 kept = dq_kept_results(dq);
 
 		/*
 		 * d->kept_results is the true amount of total results they got, which
@@ -2485,10 +2489,12 @@ dq_get_results_wanted(const struct guid *muid, guint32 *wanted)
 G_GNUC_COLD void
 dq_init(void)
 {
-	dqueries = g_hash_table_new(nid_hash, nid_equal);
-	by_node_id = g_hash_table_new(nid_hash, nid_equal);
-	by_muid = g_hash_table_new(guid_hash, guid_eq);
-	by_leaf_muid = g_hash_table_new(guid_hash, guid_eq);
+	dqueries = hevset_create_any(
+		offsetof(struct dquery, qid), nid_hash, NULL, nid_equal);
+	by_node_id = htable_create_any(nid_hash, NULL, nid_equal);
+	by_muid = htable_create(HASH_KEY_FIXED, GUID_RAW_SIZE);
+	by_leaf_muid = hikset_create(
+		offsetof(struct dquery, lmuid), HASH_KEY_FIXED, GUID_RAW_SIZE);
 	fill_hosts();
 }
 
@@ -2496,12 +2502,11 @@ dq_init(void)
  * Hashtable iteration callback to free the dquery_t object held as the key.
  */
 static void
-free_query(gpointer key, gpointer value, gpointer unused_udata)
+free_query(void *value, void *unused_udata)
 {
 	dquery_t *dq = value;
 
 	dquery_check(dq);
-	g_assert(&dq->qid == key);
 	(void) unused_udata;
 
 	dq->flags |= DQ_F_EXITING;		/* So nothing is removed from the table */
@@ -2514,7 +2519,7 @@ free_query(gpointer key, gpointer value, gpointer unused_udata)
  * there should not be anything remaining, hence warn!
  */
 static void
-free_query_list(gpointer key, gpointer value, gpointer unused_udata)
+free_query_list(const void *key, void *value, void *unused_udata)
 {
 	GSList *sl, *list = value;
 	int count = g_slist_length(list);
@@ -2541,7 +2546,7 @@ free_query_list(gpointer key, gpointer value, gpointer unused_udata)
  * anything remaining, hence warn!
  */
 static void
-free_muid(gpointer key, gpointer unused_value, gpointer unused_udata)
+free_muid(const void *key, void *unused_value, void *unused_udata)
 {
 	(void) unused_value;
 	(void) unused_udata;
@@ -2557,12 +2562,13 @@ free_muid(gpointer key, gpointer unused_value, gpointer unused_udata)
  * anything remaining, hence warn!
  */
 static void
-free_leaf_muid(gpointer key, gpointer unused_value, gpointer unused_udata)
+free_leaf_muid(void *value, void *unused_udata)
 {
-	(void) unused_value;
+	const dquery_t *dq = value;
+
 	(void) unused_udata;
 	g_warning("remained un-freed leaf MUID \"%s\" in dynamic queries",
-		guid_hex_str(key));
+		guid_hex_str(dq->lmuid));
 }
 
 /**
@@ -2571,17 +2577,17 @@ free_leaf_muid(gpointer key, gpointer unused_value, gpointer unused_udata)
 G_GNUC_COLD void
 dq_close(void)
 {
-	g_hash_table_foreach(dqueries, free_query, NULL);
-	gm_hash_table_destroy_null(&dqueries);
+	hevset_foreach(dqueries, free_query, NULL);
+	hevset_free_null(&dqueries);
 
-	g_hash_table_foreach(by_node_id, free_query_list, NULL);
-	gm_hash_table_destroy_null(&by_node_id);
+	htable_foreach(by_node_id, free_query_list, NULL);
+	htable_free_null(&by_node_id);
 
-	g_hash_table_foreach(by_muid, free_muid, NULL);
-	gm_hash_table_destroy_null(&by_muid);
+	htable_foreach(by_muid, free_muid, NULL);
+	htable_free_null(&by_muid);
 
-	g_hash_table_foreach(by_leaf_muid, free_leaf_muid, NULL);
-	gm_hash_table_destroy_null(&by_leaf_muid);
+	hikset_foreach(by_leaf_muid, free_leaf_muid, NULL);
+	hikset_free_null(&by_leaf_muid);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
