@@ -51,10 +51,10 @@ void oob_shutdown(void);
 void oob_close(void);
 
 void oob_got_results(struct gnutella_node *n, GSList *files,
-		int count, host_addr_t addr, guint16 port,
-		gboolean secure_oob, unsigned flags);
+		int count, host_addr_t addr, uint16 port,
+		bool secure_oob, unsigned flags);
 void oob_deliver_hits(struct gnutella_node *n, const struct guid *muid,
-		guint8 wanted, const struct array *token);
+		uint8 wanted, const struct array *token);
 
 #endif /* _core_oob_h_ */
 
