@@ -51,6 +51,7 @@
 #include "lib/atoms.h"
 #include "lib/cq.h"
 #include "lib/endian.h"
+#include "lib/hikset.h"
 #include "lib/nid.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
@@ -88,7 +89,7 @@ oob_proxy_rec_check(const struct oob_proxy_rec * const opr)
 /**
  * Table recording the proxied OOB query MUID.
  */
-static GHashTable *proxied_queries;	/* New MUID => oob_proxy_rec */
+static hikset_t *proxied_queries;	/* New MUID => oob_proxy_rec */
 
 /*
  * High-level description of what's happening here.
@@ -151,7 +152,7 @@ static void
 oob_proxy_rec_free_remove(struct oob_proxy_rec *opr)
 {
 	oob_proxy_rec_check(opr);
-	g_hash_table_remove(proxied_queries, opr->proxied_muid);
+	hikset_remove(proxied_queries, opr->proxied_muid);
 	oob_proxy_rec_free(opr);
 }
 
@@ -159,7 +160,7 @@ oob_proxy_rec_free_remove(struct oob_proxy_rec *opr)
  * Callout queue callback to free OOB proxy record.
  */
 static void
-oob_proxy_rec_destroy(cqueue_t *unused_cq, gpointer obj)
+oob_proxy_rec_destroy(cqueue_t *unused_cq, void *obj)
 {
 	struct oob_proxy_rec *opr = obj;
 
@@ -180,13 +181,13 @@ oob_proxy_rec_destroy(cqueue_t *unused_cq, gpointer obj)
  *
  * @return TRUE on success, FALSE on MUID collision.
  */
-gboolean
+bool
 oob_proxy_create(gnutella_node_t *n)
 {
 	struct guid proxied_muid;
 	struct oob_proxy_rec *opr;
 	host_addr_t primary;
-	guint32 ipv4;
+	uint32 ipv4;
 
 	g_assert(gnutella_header_get_function(&n->header) == GTA_MSG_SEARCH);
 	g_assert(NODE_IS_LEAF(n));
@@ -220,7 +221,7 @@ oob_proxy_create(gnutella_node_t *n)
 	 * Otherwise we have an MUID collision.
 	 */
 
-	opr = g_hash_table_lookup(proxied_queries, &proxied_muid);
+	opr = hikset_lookup(proxied_queries, &proxied_muid);
 
 	if (opr != NULL) {
 		if (opr->node_id != NODE_ID(n)) {
@@ -241,7 +242,7 @@ oob_proxy_create(gnutella_node_t *n)
 
 		opr = oob_proxy_rec_make(gnutella_header_get_muid(&n->header),
 				&proxied_muid, NODE_ID(n));
-		gm_hash_table_insert_const(proxied_queries, opr->proxied_muid, opr);
+		hikset_insert_key(proxied_queries, &opr->proxied_muid);
 
 		opr->expire_ev = cq_main_insert(PROXY_EXPIRE_MS,
 			oob_proxy_rec_destroy, opr);
@@ -279,14 +280,14 @@ oob_proxy_create(gnutella_node_t *n)
  *
  * @return whether we know about OOB-proxied query `muid'.
  */
-gboolean
+bool
 oob_proxy_pending_results(
 	gnutella_node_t *n, const struct guid *muid,
-	int hits, gboolean uu_udp_firewalled, const struct array *token)
+	int hits, bool uu_udp_firewalled, const struct array *token)
 {
 	struct oob_proxy_rec *opr;
 	struct gnutella_node *leaf;
-	guint32 wanted;
+	uint32 wanted;
 	const char *msg = NULL;
 
 	(void) uu_udp_firewalled;
@@ -295,7 +296,7 @@ oob_proxy_pending_results(
 	g_assert(hits > 0);
 	g_assert(token);
 
-	opr = g_hash_table_lookup(proxied_queries, muid);
+	opr = hikset_lookup(proxied_queries, muid);
 	if (opr == NULL)
 		return FALSE;
 
@@ -428,8 +429,8 @@ ignore:
  * @return TRUE if we routed the packet, FALSE if we did not recognize
  * the MUID as one of the OOB-proxied queries.
  */
-gboolean
-oob_proxy_got_results(gnutella_node_t *n, guint results)
+bool
+oob_proxy_got_results(gnutella_node_t *n, uint results)
 {
 	struct oob_proxy_rec *opr;
 	struct gnutella_node *leaf;
@@ -437,8 +438,7 @@ oob_proxy_got_results(gnutella_node_t *n, guint results)
 	g_assert(gnutella_header_get_function(&n->header) == GTA_MSG_SEARCH_RESULTS);
 	g_assert(results > 0 && results <= INT_MAX);
 
-	opr = g_hash_table_lookup(proxied_queries,
-				gnutella_header_get_muid(&n->header));
+	opr = hikset_lookup(proxied_queries, gnutella_header_get_muid(&n->header));
 	if (opr == NULL)
 		return FALSE;
 
@@ -523,7 +523,7 @@ oob_proxy_muid_proxied(const struct guid *muid)
 {
 	const struct oob_proxy_rec *opr;
 	
-	opr = g_hash_table_lookup(proxied_queries, muid);
+	opr = hikset_lookup(proxied_queries, muid);
 	if (opr) {
 		oob_proxy_rec_check(opr);
 		return opr->leaf_muid;
@@ -538,18 +538,19 @@ oob_proxy_muid_proxied(const struct guid *muid)
 void
 oob_proxy_init(void)
 {
-	proxied_queries = g_hash_table_new(guid_hash, guid_eq);
+	proxied_queries = hikset_create(
+		offsetof(struct oob_proxy_rec, proxied_muid),
+		HASH_KEY_FIXED, GUID_RAW_SIZE);
 }
 
 /**
  * Cleanup servent -- hash table iterator callback
  */
 static void
-free_oob_proxy_kv(gpointer uu_key, gpointer value, gpointer uu_udata)
+free_oob_proxy_kv(void *value, void *uu_udata)
 {
 	struct oob_proxy_rec *opr = value;
 
-	(void) uu_key;
 	(void) uu_udata;
 	oob_proxy_rec_check(opr);
 	oob_proxy_rec_free(opr);
@@ -561,8 +562,8 @@ free_oob_proxy_kv(gpointer uu_key, gpointer value, gpointer uu_udata)
 void
 oob_proxy_close(void)
 {
-	g_hash_table_foreach(proxied_queries, free_oob_proxy_kv, NULL);
-	gm_hash_table_destroy_null(&proxied_queries);
+	hikset_foreach(proxied_queries, free_oob_proxy_kv, NULL);
+	hikset_free_null(&proxied_queries);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
