@@ -66,39 +66,38 @@ void parq_dl_remove(struct download *);
 void parq_dl_free(struct download *);
 
 void parq_download_retry_active_queued(struct download *);
-gboolean parq_download_supports_parq(header_t *);
-gboolean parq_download_parse_queue_status(struct download *, header_t *, guint);
-gboolean parq_download_is_active_queued(const struct download *);
+bool parq_download_supports_parq(header_t *);
+bool parq_download_parse_queue_status(struct download *, header_t *, uint);
+bool parq_download_is_active_queued(const struct download *);
 void parq_download_add_header(char *buf, size_t len, size_t *rw,
 	struct download *);
-gboolean parq_download_is_passive_queued(const struct download *);
+bool parq_download_is_passive_queued(const struct download *);
 void parq_download_queue_ack(struct gnutella_socket *);
 
 void parq_upload_timer(time_t now);
 
-size_t parq_upload_add_headers(char *buf, size_t size,
-	gpointer arg, guint32 flags);
+size_t parq_upload_add_headers(char *buf, size_t size, void *arg, uint32 flags);
 size_t parq_upload_add_header_id(char *buf, size_t size,
-	gpointer arg, guint32 flags);
+	void *arg, uint32 flags);
 
 struct parq_ul_queued *parq_upload_get(struct upload *, const header_t *);
-gboolean parq_upload_request(struct upload *);
-gboolean parq_upload_request_force(struct upload *, struct parq_ul_queued *);
-guint parq_upload_lookup_position(const struct upload *);
+bool parq_upload_request(struct upload *);
+bool parq_upload_request_force(struct upload *, struct parq_ul_queued *);
+uint parq_upload_lookup_position(const struct upload *);
 const struct guid *parq_upload_lookup_id(const struct upload *);
-gboolean parq_upload_queue_full(struct upload *);
-guint parq_upload_lookup_size(const struct upload *);
+bool parq_upload_queue_full(struct upload *);
+uint parq_upload_lookup_size(const struct upload *);
 void parq_upload_update_downloaded(const struct upload *u);
 
 time_t parq_upload_lifetime(const struct upload *);
 time_t parq_upload_retry(const struct upload *);
-guint parq_upload_lookup_eta(const struct upload *);
-guint parq_upload_lookup_queue_no(const struct upload *);
-gboolean parq_upload_lookup_quick(const struct upload *);
-gboolean parq_upload_lookup_frozen(const struct upload *);
+uint parq_upload_lookup_eta(const struct upload *);
+uint parq_upload_lookup_queue_no(const struct upload *);
+bool parq_upload_lookup_quick(const struct upload *);
+bool parq_upload_lookup_frozen(const struct upload *);
 
-gboolean parq_upload_queued(struct upload *);
-gboolean parq_upload_remove(struct upload *, gboolean, gboolean);
+bool parq_upload_queued(struct upload *);
+bool parq_upload_remove(struct upload *, bool, bool);
 void parq_upload_collect_stats(const struct upload *);
 void parq_upload_upload_got_freed(struct upload *);
 void parq_upload_upload_got_cloned(struct upload *u, struct upload *cu);
@@ -107,9 +106,9 @@ void parq_upload_add(struct upload *);
 void parq_upload_busy(struct upload *, struct parq_ul_queued *);
 void parq_upload_send_queue_conf(struct upload *);
 
-gboolean parq_ul_id_sent(const struct upload *);
+bool parq_ul_id_sent(const struct upload *);
 
 time_t parq_banned_source_expire(const host_addr_t);
-gboolean parq_is_enabled(void);
+bool parq_is_enabled(void);
 
 #endif /* _core_parq_h_ */
