@@ -51,33 +51,33 @@ extern struct in_addr *local_netmasks;
  * Global Functions
  */
 
-gboolean is_my_address(const host_addr_t addr);
-gboolean is_my_address_and_port(const host_addr_t addr, guint16 port);
+bool is_my_address(const host_addr_t addr);
+bool is_my_address_and_port(const host_addr_t addr, uint16 port);
 
 void settings_early_init(void);
-gboolean settings_is_unique_instance(void);
+bool settings_is_unique_instance(void);
 void settings_init(void);
 void settings_save_if_dirty(void);
 void settings_shutdown(void);
 void settings_addr_changed(const host_addr_t new_addr, const host_addr_t peer);
-guint32 settings_max_msg_size(void);
+uint32 settings_max_msg_size(void);
 void settings_add_randomness();
 void settings_close(void);
 void settings_terminate(void);
 
-guint32 get_average_servent_uptime(time_t now);
-guint32 get_average_ip_lifetime(time_t now, enum net_type net);
+uint32 get_average_servent_uptime(time_t now);
+uint32 get_average_ip_lifetime(time_t now, enum net_type net);
 
-gboolean settings_is_leaf(void);
-gboolean settings_is_ultra(void);
-gboolean settings_use_ipv4(void);
-gboolean settings_use_ipv6(void);
-gboolean settings_running_ipv4(void);
-gboolean settings_running_ipv6(void);
-gboolean settings_running_ipv4_and_ipv6(void);
-gboolean settings_running_ipv6_only(void);
-gboolean settings_running_same_net(const host_addr_t addr);
-gboolean settings_can_connect(const host_addr_t addr);
+bool settings_is_leaf(void);
+bool settings_is_ultra(void);
+bool settings_use_ipv4(void);
+bool settings_use_ipv6(void);
+bool settings_running_ipv4(void);
+bool settings_running_ipv6(void);
+bool settings_running_ipv4_and_ipv6(void);
+bool settings_running_ipv6_only(void);
+bool settings_running_same_net(const host_addr_t addr);
+bool settings_can_connect(const host_addr_t addr);
 
 host_addr_t listen_addr_primary(void);
 host_addr_t listen_addr_primary_net(host_net_t net);
