@@ -70,7 +70,10 @@
 #include "lib/file.h"
 #include "lib/glib-missing.h"
 #include "lib/halloc.h"
-#include "lib/hashlist.h"
+#include "lib/hashing.h"
+#include "lib/hikset.h"
+#include "lib/hset.h"
+#include "lib/htable.h"
 #include "lib/listener.h"
 #include "lib/mime_type.h"
 #include "lib/str.h"
@@ -105,13 +108,13 @@ struct shared_file {
 	time_t ctime;				/**< File creation time */
 
 	filesize_t file_size;		/**< File size in Bytes */
-	guint32 file_index;			/**< the files index within our local DB */
-	guint32 sort_index;			/**< the index for sorted listings */
+	uint32 file_index;			/**< the files index within our local DB */
+	uint32 sort_index;			/**< the index for sorted listings */
 
 	enum mime_type mime_type;	/* MIME type of the file */
 
 	int refcnt;					/**< Reference count */
-	guint32 flags;				/**< See below for definition */
+	uint32 flags;				/**< See below for definition */
 };
 
 /**
@@ -134,11 +137,11 @@ static const struct special_file specials[] = {
 /**
  * Maps special names (e.g. "/favicon.ico") to the shared_file_t structure.
  */
-static GHashTable *special_names;
+static htable_t *special_names;
 
-static GHashTable *extensions;	/* Shared filename extensions */
+static hset_t *extensions;	/* Shared filename extensions */
 static GSList *shared_dirs;
-static hash_list_t *partial_files;
+static hset_t *partial_files;
 static cevent_t *share_qrp_rebuild_ev;
 
 /*
@@ -149,23 +152,23 @@ static cevent_t *share_qrp_rebuild_ev;
  * the rebuilding process do we atomically update all of them with the new
  * values, freeing old content.
  */
-static guint64 files_scanned;	/* Amount of files shared in the library */
-static guint64 bytes_scanned;
+static uint64 files_scanned;	/* Amount of files shared in the library */
+static uint64 bytes_scanned;
 static GSList *shared_files;
 static search_table_t *search_table;
-static GHashTable *file_basenames;
+static htable_t *file_basenames;
 static search_table_t *partial_table;
 static shared_file_t **file_table;			/* Sorted by mtime */
 static shared_file_t **sorted_file_table;	/* Sorted by name */
 
 static struct recursive_scan *recursive_scan_context;
-static gboolean share_rebuilding;
+static bool share_rebuilding;
 
 /**
- * This tree maps a SHA1 hash (base-32 encoded) onto the corresponding
+ * This hash table maps a SHA1 hash (base-32 encoded) onto the corresponding
  * shared_file if we have one.
  */
-static GTree *sha1_to_share;
+static hikset_t *sha1_to_share;
 
 #define A	SEARCH_AUDIO_TYPE
 #define V	SEARCH_VIDEO_TYPE
@@ -301,19 +304,7 @@ static struct {
  * Hash table yielding the media type flags from a MIME type.
  * Built dynamically from media_type_map[].
  */
-static GHashTable *share_media_types;
-
-/**
- * Compare binary SHA1 hashes.
- * @return 0 if they're the same, a negative or positive number if s1 if greater
- * than s2 or s1 greater than s2, respectively.
- * Used to search the sha1_to_share tree.
- */
-static int
-compare_share_sha1(gconstpointer s1, gconstpointer s2)
-{
-	return memcmp(s1, s2, SHA1_RAW_SIZE);
-}
+static htable_t *share_media_types;
 
 /**
  * Reset sha1_to_share
@@ -321,10 +312,12 @@ compare_share_sha1(gconstpointer s1, gconstpointer s2)
 static void
 reinit_sha1_table(void)
 {
-	if (sha1_to_share)
-		g_tree_destroy(sha1_to_share);
-
-	sha1_to_share = g_tree_new(compare_share_sha1);
+	if G_UNLIKELY(NULL == sha1_to_share) {
+		sha1_to_share = hikset_create(
+			offsetof(shared_file_t, sha1), HASH_KEY_FIXED, SHA1_RAW_SIZE);
+	} else {
+		hikset_clear(sha1_to_share);
+	}
 }
 
 void
@@ -360,7 +353,7 @@ shared_file_deindex(shared_file_t *sf)
 
 	if (SHARE_F_BASENAME & sf->flags) {
 		if (file_basenames != NULL) {
-			g_hash_table_remove(file_basenames, sf->name_nfc);
+			htable_remove(file_basenames, sf->name_nfc);
 		}
 	}
 	sf->flags &= ~SHARE_F_BASENAME;
@@ -399,12 +392,10 @@ shared_file_deindex(shared_file_t *sf)
 
 	if (sf->sha1 != NULL) {
 		shared_file_t *current;
-		gpointer key;
 
-		key = deconstify_gpointer(sf->sha1);
-		current = g_tree_lookup(sha1_to_share, key);
+		current = hikset_lookup(sha1_to_share, sf->sha1);
 		if (current == sf) {
-			g_tree_remove(sha1_to_share, key);
+			hikset_remove(sha1_to_share, sf->sha1);
 		}
 	}
 }
@@ -442,7 +433,7 @@ shared_file_free(shared_file_t **sf_ptr)
  *
  * @return whether an error occurred
  */
-static gboolean
+static bool
 shared_file_set_names(shared_file_t *sf, const char *filename)
 {
   	shared_file_check(sf);	
@@ -470,7 +461,7 @@ shared_file_set_names(shared_file_t *sf, const char *filename)
 			name = g_strconcat(sf->relative_path, " ", sf->name_nfc,
 						(void *) 0);
 		} else {
-			name = deconstify_gchar(sf->name_nfc);
+			name = deconstify_char(sf->name_nfc);
 		}
 		name_canonic = UNICODE_CANONIZE(name);
 		sf->name_canonic = atom_str_get(name_canonic);
@@ -494,9 +485,9 @@ shared_file_set_names(shared_file_t *sf, const char *filename)
 	return FALSE;
 }
 
-static const guint FILENAME_CLASH = -1;		/**< Indicates basename clashes */
-static const guint PARTIAL_FILE = -2;		/**< Indicates partial file */
-static const guint SPECIAL_FILE = -3;		/**< Special served files */
+static const uint FILENAME_CLASH = -1;		/**< Indicates basename clashes */
+static const uint PARTIAL_FILE = -2;		/**< Indicates partial file */
+static const uint SPECIAL_FILE = -3;		/**< Special served files */
 
 /**
  * Initialize special file entry, returning shared_file_t structure if
@@ -559,8 +550,8 @@ done:
 
 void
 shared_files_match(const char *query,
-	st_search_callback callback, gpointer user_data,
-	int max_res, gboolean partials, query_hashvec_t *qhv)
+	st_search_callback callback, void *user_data,
+	int max_res, bool partials, query_hashvec_t *qhv)
 {
 	int n;
 	int remain;
@@ -596,15 +587,14 @@ shared_files_match(const char *query,
 static G_GNUC_COLD void
 share_special_init(void)
 {
-	guint i;
+	uint i;
 
-	special_names = g_hash_table_new(g_str_hash, g_str_equal);
+	special_names = htable_create(HASH_KEY_STRING, 0);
 
 	for (i = 0; i < G_N_ELEMENTS(specials); i++) {
 		shared_file_t *sf = share_special_load(&specials[i]);
 		if (sf != NULL)
-			g_hash_table_insert(special_names,
-				deconstify_gchar(specials[i].path), shared_file_ref(sf));
+			htable_insert(special_names, specials[i].path, shared_file_ref(sf));
 	}
 }
 
@@ -623,7 +613,7 @@ shared_special(const char *path)
 	shared_file_t *sf;
 	filestat_t file_stat;
 
-	sf = g_hash_table_lookup(special_names, path);
+	sf = htable_lookup(special_names, path);
 
 	if (sf == NULL)
 		return NULL;
@@ -656,7 +646,7 @@ shared_special(const char *path)
  * index) and SHARE_REBUILDING when we're rebuilding the library.
  */
 shared_file_t *
-shared_file(guint idx)
+shared_file(uint idx)
 {
 	/* @return shared file info for index `idx', or NULL if none */
 
@@ -675,7 +665,7 @@ shared_file(guint idx)
  * index) and SHARE_REBUILDING when we're rebuilding the library.
  */
 shared_file_t *
-shared_file_sorted(guint idx)
+shared_file_sorted(uint idx)
 {
 	/* @return shared file info for index `idx', or NULL if none */
 
@@ -692,12 +682,12 @@ shared_file_sorted(guint idx)
  * Get index of shared file identified by its name.
  * @return index > 0 if found, 0 if file is not known.
  */
-static guint
+static uint
 shared_file_get_index(const char *filename)
 {
-	guint idx;
+	uint idx;
 
-	idx = GPOINTER_TO_UINT(g_hash_table_lookup(file_basenames, filename));
+	idx = pointer_to_uint(htable_lookup(file_basenames, filename));
 	if (idx == 0 || idx == FILENAME_CLASH)
 		return 0;
 
@@ -715,7 +705,7 @@ shared_file_t *
 shared_file_by_name(const char *filename)
 {
 	shared_file_t *sf;
-	guint idx;
+	uint idx;
 
 	if (file_table == NULL)
 		return SHARE_REBUILDING;
@@ -732,10 +722,8 @@ shared_file_by_name(const char *filename)
 }
 
 static void
-free_extensions_helper(gpointer key,
-	gpointer unused_value, gpointer unused_data)
+free_extensions_helper(const void *key, void *unused_data)
 {
-	(void) unused_value;
 	(void) unused_data;
 	atom_str_free(key);
 }
@@ -746,8 +734,8 @@ static void
 free_extensions(void)
 {
 	if (extensions) {
-		g_hash_table_foreach(extensions, free_extensions_helper, NULL);
-		gm_hash_table_destroy_null(&extensions);
+		hset_foreach(extensions, free_extensions_helper, NULL);
+		hset_free_null(&extensions);
 	}
 }
 
@@ -759,10 +747,10 @@ parse_extensions(const char *str)
 {
 	char **exts = g_strsplit(str, ";", 0);
 	char *x, *s;
-	guint i;
+	uint i;
 
 	free_extensions();
-	extensions = g_hash_table_new(ascii_strcase_hash, ascii_strcase_eq);
+	extensions = hset_create_any(ascii_strcase_hash, NULL, ascii_strcase_eq);
 
 	for (i = 0; exts[i]; i++) {
 		char c;
@@ -780,9 +768,9 @@ parse_extensions(const char *str)
 					break;
 			}
 
-			if (*s && NULL == g_hash_table_lookup(extensions, s)) {
-				gconstpointer key = atom_str_get(s);
-				gm_hash_table_insert_const(extensions, key, key);
+			if (*s && !hset_contains(extensions, s)) {
+				const void *key = atom_str_get(s);
+				hset_insert(extensions, key);
 			}
 		}
 	}
@@ -834,12 +822,12 @@ shared_dirs_update_prop(void)
  * The given string was completely parsed, it returns TRUE, otherwise
  * it returns FALSE.
  */
-gboolean
+bool
 shared_dirs_parse(const char *str)
 {
 	char **dirs = g_strsplit(str, G_SEARCHPATH_SEPARATOR_S, 0);
-    gboolean ret = TRUE;
-	guint i;
+    bool ret = TRUE;
+	uint i;
 
 	/* FIXME: ESCAPING! */
 
@@ -848,7 +836,7 @@ shared_dirs_parse(const char *str)
 	for (i = 0; dirs[i]; i++) {
 		if (is_directory(dirs[i]))
 			shared_dirs = g_slist_prepend(shared_dirs,
-								deconstify_gchar(atom_str_get(dirs[i])));
+								deconstify_char(atom_str_get(dirs[i])));
         else
             ret = FALSE;
 	}
@@ -870,7 +858,7 @@ shared_dir_add(const char *pathname)
 			g_debug("%s: adding pathname=\"%s\"", G_STRFUNC, pathname);
 		}
         shared_dirs = g_slist_append(shared_dirs,
-						deconstify_gchar(atom_str_get(pathname)));
+						deconstify_char(atom_str_get(pathname)));
 	} else {
 		if (GNET_PROPERTY(share_debug) > 0) {
 			g_debug("%s: NOT adding pathname=\"%s\"", G_STRFUNC, pathname);
@@ -886,7 +874,7 @@ shared_dir_add(const char *pathname)
 shared_file_t *
 shared_file_ref(const shared_file_t *sf)
 {
-	shared_file_t *wsf = deconstify_gpointer(sf);
+	shared_file_t *wsf = deconstify_pointer(sf);
 
 	wsf->refcnt++;
 	return wsf;
@@ -922,7 +910,7 @@ shared_file_unref(shared_file_t **sf_ptr)
  *		 Keep it just in case that a platform has an fileoffset_t with more than
  *		 64 bits.
  */
-static inline gboolean
+static inline bool
 too_big_for_gnutella(fileoffset_t size)
 {
 	g_return_val_if_fail(size >= 0, TRUE);
@@ -938,7 +926,7 @@ too_big_for_gnutella(fileoffset_t size)
  *			returned. Otherwise, the pathname is considered OK and FALSE
  *			is returned.
  */
-static gboolean
+static bool
 contains_control_chars(const char *pathname)
 {
 	const char *s;
@@ -985,7 +973,7 @@ get_relative_path(const char *base_dir, const char *pathname)
  * @param filename  The name of the file to check.
  * @return TRUE if the file should be shared, FALSE if not.
  */
-static gboolean
+static bool
 shared_file_valid_extension(const char *filename)
 {
 	const char *filename_ext;
@@ -994,8 +982,8 @@ shared_file_valid_extension(const char *filename)
 		return FALSE;
 
 	if (
-		1 == g_hash_table_size(extensions) &&
-		g_hash_table_lookup(extensions, "--all--")
+		1 == hset_count(extensions) &&
+		hset_contains(extensions, "--all--")
     ) {
 		/*
 		 * An extension "--all--" matches all files, even those that don't
@@ -1018,9 +1006,8 @@ shared_file_valid_extension(const char *filename)
 		 * All valid extensions start with '.'.  Matching is case-insensitive
 		 */
 
-		if (g_hash_table_lookup(extensions, filename_ext)) {
+		if (hset_contains(extensions, filename_ext))
 			return TRUE;
-		}
 	}
 
 	return FALSE;
@@ -1130,7 +1117,7 @@ share_scan_add_file(const char *relative_path,
  * This is not meant to be exhaustive but test for common configuration
  * mistakes.
  */
-static gboolean
+static bool
 directory_is_unshareable(const char *dir)
 {
 	g_assert(dir);
@@ -1184,15 +1171,15 @@ struct recursive_scan {
 	slist_t *shared_files;		/* list of struct shared_file */
 	slist_t *partial_files;		/* list of struct shared_file */
 	slist_iter_t *iter;			/* list iterator */
-	GHashTable *words;			/* records words making up filenames, for QRP */
-	GHashTable *basenames;		/* known file basenames */
+	htable_t *words;			/* records words making up filenames, for QRP */
+	htable_t *basenames;		/* known file basenames */
 	GSList *shared;				/* the new shared_files variable */
 	shared_file_t **files;		/* the new file_table, sorted by mtime */
 	shared_file_t **sorted;		/* the new sorted_file_table, sorted by name */
 	search_table_t *search_tb;	/* the new search table */
 	search_table_t *partial_tb;	/* the new partial table */
-	guint64 files_scanned;		/* amount of files shared in the library */
-	guint64 bytes_scanned;		/* size of the library */
+	uint64 files_scanned;		/* amount of files shared in the library */
+	uint64 bytes_scanned;		/* size of the library */
 	int idx;					/* iterating index */
 	int ticks;					/* ticks used */
 };
@@ -1220,11 +1207,11 @@ recursive_scan_new(const GSList *base_dirs)
 	ctx->sub_dirs = slist_new();
 	ctx->shared_files = slist_new();
 	ctx->partial_files = slist_new();
-	ctx->words = g_hash_table_new(g_str_hash, g_str_equal);
-	ctx->basenames = g_hash_table_new(g_str_hash, g_str_equal);
+	ctx->words = htable_create(HASH_KEY_STRING, 0);
+	ctx->basenames = htable_create(HASH_KEY_STRING, 0);
 	for (iter = base_dirs; NULL != iter; iter = g_slist_next(iter)) {
 		const char *dir = atom_str_get(iter->data);
-		slist_append(ctx->base_dirs, deconstify_gchar(dir));
+		slist_append(ctx->base_dirs, deconstify_char(dir));
 	}
 	return ctx;
 }
@@ -1245,7 +1232,7 @@ recursive_scan_closedir(struct recursive_scan *ctx)
 	}
 }
 
-static void recursive_sf_unref(gpointer o)
+static void recursive_sf_unref(void *o)
 {
 	shared_file_t *sf = o;
 
@@ -1287,7 +1274,7 @@ recursive_scan_free(struct recursive_scan **ctx_ptr)
 		slist_free_all(&ctx->partial_files, recursive_sf_unref);
 		slist_iter_free(&ctx->iter);
 
-		gm_hash_table_destroy_null(&ctx->basenames);
+		htable_free_null(&ctx->basenames);
 		st_free(&ctx->search_tb);
 		st_free(&ctx->partial_tb);
 		atom_str_free_null(&ctx->base_dir);
@@ -1337,7 +1324,7 @@ share_free(void)
 	GSList *sl;
 
 	st_free(&search_table);
-	gm_hash_table_destroy_null(&file_basenames);
+	htable_free_null(&file_basenames);
 
 	for (sl = shared_files; sl; sl = g_slist_next(sl)) {
 		shared_file_t *sf = sl->data;
@@ -1589,7 +1576,7 @@ finish:
 /**
  * @return TRUE if finished.
  */
-static gboolean 
+static bool 
 recursive_scan_next_dir(struct recursive_scan *ctx)
 {
 	recursive_scan_check(ctx);
@@ -1740,7 +1727,7 @@ recursive_scan_step_build_basenames(struct bgtask *bt, void *data, int ticks)
 
 	while (UNSIGNED(ctx->idx) < ctx->files_scanned) {
 		shared_file_t *sf;
-		guint val;
+		uint val;
 		int i = ctx->idx++;
 
 	   	sf = ctx->files[i];
@@ -1765,16 +1752,14 @@ recursive_scan_step_build_basenames(struct bgtask *bt, void *data, int ticks)
 		 *		--RAM, 06/06/2002
 		 */
 
-		val = pointer_to_uint(
-				g_hash_table_lookup(ctx->basenames, sf->name_nfc));
+		val = pointer_to_uint(htable_lookup(ctx->basenames, sf->name_nfc));
 
 		/*
 		 * The following works because 0 cannot be a valid file index.
 		 */
 
 		val = (val != 0) ? FILENAME_CLASH : sf->file_index;
-		g_hash_table_insert(ctx->basenames, deconstify_gchar(sf->name_nfc),
-			GUINT_TO_POINTER(val));
+		htable_insert(ctx->basenames, sf->name_nfc, uint_to_pointer(val));
 
 		if (ctx->ticks++ >= ticks)
 			return BGR_MORE;
@@ -1963,14 +1948,15 @@ recursive_scan_step_load_partials(struct bgtask *bt, void *data, int ticks)
 	(void) ticks;
 
 	if (share_can_answer_partials()) {
-		hash_list_iter_t *iter = hash_list_iterator(partial_files);
+		hset_iter_t *iter = hset_iter_new(partial_files);
+		const void *item;
 
-		while (hash_list_iter_has_next(iter)) {
-			const shared_file_t *sf = hash_list_iter_next(iter);
+		while (hset_iter_next(iter, &item)) {
+			const shared_file_t *sf = item;
 			slist_append(ctx->partial_files, shared_file_ref(sf));
 		}
 
-		hash_list_iter_release(&iter);
+		hset_iter_release(&iter);
 	}
 
 	bg_task_ticks_used(bt, 0);
@@ -2098,7 +2084,7 @@ recursive_scan_step_update_qrp(struct bgtask *bt, void *data, int ticks)
 		 * when inserting partial files.
 		 */
 
-		sf->file_index = files_scanned + (hash_list_length(partial_files) -
+		sf->file_index = files_scanned + (hset_count(partial_files) -
 			slist_length(ctx->partial_files));
 
 		qrp_add_file(sf, ctx->words);
@@ -2205,7 +2191,7 @@ share_scan(void)
  *
  * @return whether QRP update task was launched.
  */
-static gboolean
+static bool
 share_update_qrp(void)
 {
 	/*
@@ -2227,7 +2213,7 @@ share_update_qrp(void)
  * Hash table iterator callback to free the value.
  */
 static void
-special_free_kv(gpointer unused_key, gpointer val, gpointer unused_udata)
+special_free_kv(const void *unused_key, void *val, void *unused_udata)
 {
 	shared_file_t *sf = val;
 
@@ -2244,8 +2230,8 @@ special_free_kv(gpointer unused_key, gpointer val, gpointer unused_udata)
 static void
 share_special_close(void)
 {
-	g_hash_table_foreach(special_names, special_free_kv, NULL);
-	gm_hash_table_destroy_null(&special_names);
+	htable_foreach(special_names, special_free_kv, NULL);
+	htable_free_null(&special_names);
 }
 
 /**
@@ -2265,9 +2251,10 @@ share_close(void)
 	oob_close();
 	qhit_close();
 	st_free(&partial_table);
-	gm_hash_table_destroy_null(&share_media_types);
-	hash_list_free(&partial_files);
+	htable_free_null(&share_media_types);
+	hset_free_null(&partial_files);
 	st_free(&partial_table);
+	hikset_free_null(&sha1_to_share);
 	cq_cancel(&share_qrp_rebuild_ev);
 }
 
@@ -2291,16 +2278,14 @@ shared_file_set_sha1(shared_file_t *sf, const struct sha1 *sha1)
 
 	if (sf->sha1) {
 		shared_file_t *current;
-		gpointer key;
 
-		key = deconstify_gpointer(sf->sha1);
-		current = g_tree_lookup(sha1_to_share, key);
+		current = hikset_lookup(sha1_to_share, sf->sha1);
 		if (current) {
 			shared_file_check(current);
 			g_assert(SHARE_F_INDEXED & current->flags);
 
 			if (sf == current) {
-				g_tree_remove(sha1_to_share, key);
+				hikset_remove(sha1_to_share, sf->sha1);
 			}
 		}
 	}
@@ -2315,10 +2300,8 @@ shared_file_set_sha1(shared_file_t *sf, const struct sha1 *sha1)
 
 	if ((SHARE_F_INDEXED & sf->flags) && sf->sha1) {
 		shared_file_t *current;
-		gpointer key;
 
-		key = deconstify_gpointer(sf->sha1);
-		current = g_tree_lookup(sha1_to_share, key);
+		current = hikset_lookup(sha1_to_share, sf->sha1);
 		if (current) {
 			shared_file_check(current);
 			g_assert(SHARE_F_INDEXED & current->flags);
@@ -2338,7 +2321,7 @@ shared_file_set_sha1(shared_file_t *sf, const struct sha1 *sha1)
 			 * Record in the set of shared SHA-1s and publish to the DHT.
 			 */
 		
-			g_tree_insert(sha1_to_share, deconstify_gpointer(sf->sha1), sf);
+			hikset_insert_key(sha1_to_share, &sf->sha1);
 			publisher_add(sf->sha1);
 		}
 	}
@@ -2367,7 +2350,7 @@ shared_file_set_modification_time(shared_file_t *sf, time_t mtime)
  *
  * Use sha1_hash_is_uptodate() to check for availability and accurateness.
  */
-gboolean
+bool
 sha1_hash_available(const shared_file_t *sf)
 {
 	shared_file_check(sf);
@@ -2382,7 +2365,7 @@ sha1_hash_available(const shared_file_t *sf)
  * NB: if the file is found to have changed, the background computation of
  * the SHA1 is requested.
  */
-gboolean
+bool
 sha1_hash_is_uptodate(shared_file_t *sf)
 {
 	filestat_t buf;
@@ -2448,21 +2431,21 @@ sha1_hash_is_uptodate(shared_file_t *sf)
 /**
  * Whether file is finished (i.e. either shared from the library or seeded).
  */
-gboolean
+bool
 shared_file_is_finished(const shared_file_t *sf)
 {
 	shared_file_check(sf);
 	return NULL == sf->fi || 0 != (sf->fi->flags & FI_F_SEEDING);
 }
 
-gboolean
+bool
 shared_file_is_partial(const shared_file_t *sf)
 {
 	shared_file_check(sf);
 	return NULL != sf->fi;
 }
 
-gboolean
+bool
 shared_file_is_shareable(const shared_file_t *sf)
 {
 	shared_file_check(sf);
@@ -2489,7 +2472,7 @@ shared_file_size(const shared_file_t *sf)
 	return sf->file_size;
 }
 
-guint32
+uint32
 shared_file_index(const shared_file_t *sf)
 {
 	shared_file_check(sf);
@@ -2598,7 +2581,7 @@ shared_file_creation_time(const shared_file_t *sf)
 	return sf->ctime;
 }
 
-guint32
+uint32
 shared_file_flags(const shared_file_t *sf)
 {
 	shared_file_check(sf);
@@ -2689,7 +2672,7 @@ shared_file_complete_by_sha1(const struct sha1 *sha1)
 	if (sha1_to_share == NULL)			/* Not even begun share_scan() yet */
 		return SHARE_REBUILDING;
 
-	f = g_tree_lookup(sha1_to_share, deconstify_gpointer(sha1));
+	f = hikset_lookup(sha1_to_share, sha1);
 	if (f) {
 		shared_file_check(f);
 	}
@@ -2801,7 +2784,7 @@ share_fill_newest(shared_file_t **sfvec, size_t sfcount, unsigned media_mask)
 /**
  * Is shared file belonging to the media types indicated by mask?
  */
-gboolean
+bool
 shared_file_has_media_type(const shared_file_t *sf, unsigned mask)
 {
 	unsigned type;
@@ -2809,7 +2792,7 @@ shared_file_has_media_type(const shared_file_t *sf, unsigned mask)
 	shared_file_check(sf);
 
 	type = pointer_to_uint(
-		g_hash_table_lookup(share_media_types, int_to_pointer(sf->mime_type)));
+		htable_lookup(share_media_types, int_to_pointer(sf->mime_type)));
 
 	return 0 != (type & mask);
 }
@@ -2829,7 +2812,7 @@ share_filename_media_mask(const char *filename)
 	const void *v;
 
 	mime = mime_type_from_filename(filename);
-	v = g_hash_table_lookup(share_media_types, int_to_pointer(mime));
+	v = htable_lookup(share_media_types, int_to_pointer(mime));
 
 	return pointer_to_uint(v);
 }
@@ -2837,7 +2820,7 @@ share_filename_media_mask(const char *filename)
 /**
  * Get accessor for ``kbytes_scanned''
  */
-guint64
+uint64
 shared_kbytes_scanned(void)
 {
 	return bytes_scanned / 1024;
@@ -2846,7 +2829,7 @@ shared_kbytes_scanned(void)
 /**
  * Get accessor for ``files_scanned''
  */
-guint64
+uint64
 shared_files_scanned(void)
 {
 	return files_scanned;
@@ -2899,10 +2882,10 @@ share_add_partial(const shared_file_t *sf)
 {
 	g_assert(shared_file_is_partial(sf));
 
-	if (hash_list_contains(partial_files, sf))
+	if (hset_contains(partial_files, sf))
 		return;
 
-	hash_list_append(partial_files, sf);
+	hset_insert(partial_files, sf);
 
 	/*
 	 * We added a new partial file, we need to rebuild the QRP table.
@@ -2924,7 +2907,7 @@ share_remove_partial(const shared_file_t *sf)
 {
 	g_assert(shared_file_is_partial(sf));
 
-	if (NULL == hash_list_remove(partial_files, sf))
+	if (!hset_remove(partial_files, sf))
 		return;
 
 	/*
@@ -2983,17 +2966,17 @@ share_init(void)
 	 * be applied to partial files).
 	 */
 
-	partial_files = hash_list_new(pointer_hash_func, NULL);
+	partial_files = hset_create(HASH_KEY_SELF, 0);
 	partial_table = st_create();
 
 	/*
 	 * Create the hash table yielding the media type flags from a MIME type.
 	 */
 
-	share_media_types = g_hash_table_new(NULL, NULL);
+	share_media_types = htable_create(HASH_KEY_SELF, 0);
 
 	for (i = 0; i < G_N_ELEMENTS(media_type_map); i++) {
-		g_hash_table_insert(share_media_types,
+		htable_insert(share_media_types,
 			int_to_pointer(media_type_map[i].type),
 			int_to_pointer(media_type_map[i].flags));
 	}
