@@ -45,9 +45,9 @@ const struct txdrv_ops *tx_deflate_get_ops(void);
  * Callbacks used by the deflating layer.
  */
 struct tx_deflate_cb {
-	void (*add_tx_deflated)(gpointer owner, int amount);
-	void (*shutdown)(gpointer owner, const char *reason, ...);
-	void (*flow_control)(gpointer owner, size_t amount);
+	void (*add_tx_deflated)(void *owner, int amount);
+	void (*shutdown)(void *owner, const char *reason, ...);
+	void (*flow_control)(void *owner, size_t amount);
 };
 
 /**
@@ -58,9 +58,9 @@ struct tx_deflate_args {
 	cqueue_t *cq;				/**< Callout queue to use */
 	size_t buffer_size;			/**< Internal buffer size to use */
 	size_t buffer_flush;		/**< Flush after that many bytes */
-	gboolean nagle;				/**< Whether to use Nagle or not */
-	gboolean gzip;				/**< Whether to use gzip encapsulation */
-	gboolean reduced;			/**< Whether to use reduced compression */
+	bool nagle;					/**< Whether to use Nagle or not */
+	bool gzip;					/**< Whether to use gzip encapsulation */
+	bool reduced;				/**< Whether to use reduced compression */
 };
 
 #endif	/* _core_tx_deflate_h_ */
