@@ -78,13 +78,16 @@
 
 #include "lib/ascii.h"
 #include "lib/atoms.h"
-#include "lib/cq.h"
 #include "lib/bstr.h"
+#include "lib/cq.h"
 #include "lib/dbmw.h"
 #include "lib/dbstore.h"
 #include "lib/glib-missing.h"
+#include "lib/hashing.h"
 #include "lib/host_addr.h"
+#include "lib/hset.h"
 #include "lib/log.h"				/* For log_file_printable() */
+#include "lib/mempcpy.h"
 #include "lib/parse.h"
 #include "lib/pmsg.h"
 #include "lib/stringify.h"
@@ -92,6 +95,7 @@
 #include "lib/unsigned.h"
 #include "lib/vendors.h"
 #include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 /*
@@ -139,22 +143,22 @@ struct valuedata {
 	kuid_t cid;					/**< The "secondary key" of the value */
 	vendor_code_t vcode;		/**< Vendor code who created the info */
 	host_addr_t addr;			/**< IP address of creator */
-	guint16 port;				/**< Port number of creator */
-	guint8 major;				/**< Version major of creator */
-	guint8 minor;				/**< Version minor of creator */
+	uint16 port;				/**< Port number of creator */
+	uint8 major;				/**< Version major of creator */
+	uint8 minor;				/**< Version minor of creator */
 	/* Value information */
 	dht_value_type_t type;		/**< Type of value stored */
-	guint8 value_major;			/**< Major version of value */
-	guint8 value_minor;			/**< Minor version of value */
-	guint16 length;				/**< Value length */
-	gboolean original;			/**< Whether we got data from creator */
+	uint8 value_major;			/**< Major version of value */
+	uint8 value_minor;			/**< Minor version of value */
+	uint16 length;				/**< Value length */
+	bool original;				/**< Whether we got data from creator */
 	/* Statistics */
 	time_t created;				/**< When we first created the value */
-	guint32 n_republish;		/**< Amount of republishing we had */
-	guint32 n_replication;		/**< Amount of replication we had */
-	guint32 s_elapsed_publish;	/**< Sum of elapsed time between publications */
-	guint32 s_elapsed_replicat;	/**< Sum of elapsed time between replications */
-	guint32 n_requests;			/**< Amount of time value was requested */
+	uint32 n_republish;			/**< Amount of republishing we had */
+	uint32 n_replication;		/**< Amount of replication we had */
+	uint32 s_elapsed_publish;	/**< Sum of elapsed time between publications */
+	uint32 s_elapsed_replicat;	/**< Sum of elapsed time between replications */
+	uint32 n_requests;			/**< Amount of time value was requested */
 };
 
 /**
@@ -162,7 +166,7 @@ struct valuedata {
  * These access keys are retrievable from another DB indexed by the primary
  * key of the value (see keys.c).
  */
-static guint64 valueid = 1;		/* 0 is not a valid key (used as marker) */
+static uint64 valueid = 1;		/* 0 is not a valid key (used as marker) */
 
 /**
  * Total amount of values currently managed.
@@ -173,14 +177,14 @@ static int values_managed = 0;
  * Counts number of values currently stored per IPv4 address and per class C
  * network.
  */
-static GHashTable *values_per_ip;
-static GHashTable *values_per_class_c;
+static acct_net_t *values_per_ip;
+static acct_net_t *values_per_class_c;
 
 /**
  * Records expired DB keys that have been identified but not physically
  * removed yet.
  */
-static GHashTable *expired;
+static hset_t *expired;
 
 /**
  * DBM wrapper to store valuedata.
@@ -220,7 +224,7 @@ values_count(void)
  * Serialization routine for valuedata.
  */
 static void
-serialize_valuedata(pmsg_t *mb, gconstpointer data)
+serialize_valuedata(pmsg_t *mb, const void *data)
 {
 	const struct valuedata *vd = data;
 
@@ -254,7 +258,7 @@ serialize_valuedata(pmsg_t *mb, gconstpointer data)
  * Deserialization routine for valuedata.
  */
 static void
-deserialize_valuedata(bstr_t *bs, gpointer valptr, size_t len)
+deserialize_valuedata(bstr_t *bs, void *valptr, size_t len)
 {
 	struct valuedata *vd = valptr;
 
@@ -312,7 +316,7 @@ static const char * const store_errstr[] = {
  * @return human-readable error string corresponding to error code `errnum'.
  */
 const char *
-dht_store_error_to_string(guint16 errnum)
+dht_store_error_to_string(uint16 errnum)
 {
 	if (errnum == 0 || errnum >= G_N_ELEMENTS(store_errstr))
 		return "Invalid error code";
@@ -327,10 +331,10 @@ struct dht_value {
 	const knode_t *creator;	/**< The creator of the value */
 	kuid_t *id;				/**< The key of the value (atom) */
 	dht_value_type_t type;	/**< Type of values */
-	guint8 major;			/**< Value's major version */
-	guint8 minor;			/**< Value's minor version */
-	guint16 length;			/**< Length of value */
-	gconstpointer data;		/**< The actual data value */
+	uint8 major;			/**< Value's major version */
+	uint8 minor;			/**< Value's minor version */
+	uint16 length;			/**< Length of value */
+	const void *data;		/**< The actual data value */
 };
 
 const kuid_t *
@@ -351,7 +355,7 @@ dht_value_type(const dht_value_t *v)
 	return v->type;
 }
 
-guint16
+uint16
 dht_value_length(const dht_value_t *v)
 {
 	return v->length;
@@ -383,7 +387,7 @@ dht_value_dump(FILE *out, const dht_value_t *v)
 dht_value_t *
 dht_value_make(const knode_t *creator,
 	const kuid_t *primary_key, dht_value_type_t type,
-	guint8 major, guint8 minor, gpointer data, guint16 length)
+	uint8 major, uint8 minor, void *data, uint16 length)
 {
 	dht_value_t *v;
 
@@ -432,17 +436,17 @@ dht_value_clone(const dht_value_t *v)
  * Free DHT value, optionally freeing the data as well.
  */
 void
-dht_value_free(dht_value_t *v, gboolean free_data)
+dht_value_free(dht_value_t *v, bool free_data)
 {
 	g_assert(v);
 	g_assert(equiv(v->length > 0, v->data != NULL));
 
-	knode_free(deconstify_gpointer(v->creator));
+	knode_free(deconstify_pointer(v->creator));
 	kuid_atom_free_null(&v->id);
 
 	if (free_data && v->data) {
 		g_assert(v->length && v->length <= DHT_VALUE_MAX_LEN);
-		wfree(deconstify_gpointer(v->data), v->length);
+		wfree(deconstify_pointer(v->data), v->length);
 	}
 
 	WFREE(v);
@@ -498,7 +502,7 @@ dht_value_lifetime(dht_value_type_t type)
  * @return length of the resulting string before potential truncation.
  */
 size_t
-dht_value_type_to_string_buf(guint32 type, char *buf, size_t size)
+dht_value_type_to_string_buf(uint32 type, char *buf, size_t size)
 {
 	if (type == DHT_VT_BINARY) {
 		return g_strlcpy(buf, "BIN.", size);
@@ -523,7 +527,7 @@ dht_value_type_to_string_buf(guint32 type, char *buf, size_t size)
  * @return pointer to static data
  */
 const char *
-dht_value_type_to_string(guint32 type)
+dht_value_type_to_string(uint32 type)
 {
 	static char buf[5];
 
@@ -537,7 +541,7 @@ dht_value_type_to_string(guint32 type)
  * @return pointer to static data
  */
 const char *
-dht_value_type_to_string2(guint32 type)
+dht_value_type_to_string2(uint32 type)
 {
 	static char buf[5];
 
@@ -600,10 +604,10 @@ dht_value_deserialize(bstr_t *bs)
 	dht_value_t *dv;
 	kuid_t id;
 	knode_t *creator;
-	guint8 major, minor;
-	guint16 length;
-	gpointer data = NULL;
-	guint32 type;
+	uint8 major, minor;
+	uint16 length;
+	void *data = NULL;
+	uint32 type;
 
 	creator = kmsg_deserialize_contact(bs);
 	if (!creator)
@@ -670,7 +674,7 @@ dht_value_hash(const void *key)
 /**
  * DHT value equality test.
  */
-gboolean
+bool
 dht_value_eq(const void *p1, const void *p2)
 {
 	const dht_value_t * const v1 = p1;
@@ -716,8 +720,8 @@ dht_value_fill_record(const dht_value_t *v, lookup_val_rc_t *rc)
 /**
  * Hash a pair of KUIDs.
  */
-static guint
-kuid_pair_hash(gconstpointer key)
+static uint
+kuid_pair_hash(const void *key)
 {
 	return binary_hash(key, 2 * KUID_RAW_SIZE);
 }
@@ -726,7 +730,7 @@ kuid_pair_hash(gconstpointer key)
  * Test equality of two KUID pairs.
  */
 static int
-kuid_pair_eq(gconstpointer a, gconstpointer b)
+kuid_pair_eq(const void *a, const void *b)
 {
 	return a == b || 0 == memcmp(a, b, 2 * KUID_RAW_SIZE);
 }
@@ -737,12 +741,14 @@ kuid_pair_eq(gconstpointer a, gconstpointer b)
 static void
 kuid_pair_fill(char *buf, size_t len, const kuid_t *key, const kuid_t *skey)
 {
+	void *p;
+
 	g_assert(len >= 2 * KUID_RAW_SIZE);
 
 	STATIC_ASSERT(sizeof(key->v) == KUID_RAW_SIZE);
 
-	memcpy(&buf[0], key->v, sizeof(key->v));
-	memcpy(&buf[KUID_RAW_SIZE], skey->v, sizeof(skey->v));
+	p = mempcpy(buf, key, KUID_RAW_SIZE);
+	memcpy(p, skey, KUID_RAW_SIZE);
 }
 
 /**
@@ -751,7 +757,7 @@ kuid_pair_fill(char *buf, size_t len, const kuid_t *key, const kuid_t *skey)
  * @param key		primary key
  * @param skey		secondary key
  */
-static gboolean
+static bool
 kuid_pair_was_expired(const kuid_t *key, const kuid_t *skey)
 {
 	char buf[2 * KUID_RAW_SIZE];
@@ -825,7 +831,7 @@ kuid_pair_was_republished(const kuid_t *key, const kuid_t *skey)
  * Get valuedata from database.
  */
 static struct valuedata *
-get_valuedata(guint64 dbkey)
+get_valuedata(uint64 dbkey)
 {
 	struct valuedata *vd;
 
@@ -852,7 +858,7 @@ get_valuedata(guint64 dbkey)
  * @param has_expired	whether deletion happens because value expired
  */
 static void
-delete_valuedata(guint64 dbkey, gboolean has_expired)
+delete_valuedata(uint64 dbkey, bool has_expired)
 {
 	const struct valuedata *vd;
 
@@ -879,12 +885,11 @@ delete_valuedata(guint64 dbkey, gboolean has_expired)
 /**
  * Hash table iterator callback to reclaim an expired DB key.
  */
-static gboolean
-reclaim_dbkey(gpointer key, gpointer u_value, gpointer u_data)
+static bool
+reclaim_dbkey(const void *key, void *u_data)
 {
-	guint64 *dbatom = key;
+	const uint64 *dbatom = key;
 
-	(void) u_value;
 	(void) u_data;
 
 	delete_valuedata(*dbatom, TRUE);
@@ -902,14 +907,14 @@ reclaim_dbkey(gpointer key, gpointer u_value, gpointer u_data)
 void
 values_reclaim_expired(void)
 {
-	g_hash_table_foreach_remove(expired, reclaim_dbkey, NULL);
+	hset_foreach_remove(expired, reclaim_dbkey, NULL);
 }
 
 /**
  *  Callout queue periodic event for value expiration.
  */
-static gboolean
-values_periodic_expire(gpointer unused_obj)
+static bool
+values_periodic_expire(void *unused_obj)
 {
 	(void) unused_obj;
 
@@ -921,7 +926,7 @@ values_periodic_expire(gpointer unused_obj)
  * Log statistics about an expired value.
  */
 static void
-log_expired_value_stats(guint64 dbkey, const struct valuedata *vd)
+log_expired_value_stats(uint64 dbkey, const struct valuedata *vd)
 {
 	if (NULL == vd)
 		return;
@@ -935,8 +940,8 @@ log_expired_value_stats(guint64 dbkey, const struct valuedata *vd)
 		(unsigned) vd->n_requests, uint64_to_string(dbkey));
 
 	if (GNET_PROPERTY(dht_storage_debug) > 1) {
-		guint32 avg_publish = 0;
-		guint32 avg_replicate = 0;
+		uint32 avg_publish = 0;
+		uint32 avg_replicate = 0;
 
 		if (vd->n_republish)
 			avg_publish = vd->s_elapsed_publish / vd->n_republish;
@@ -958,11 +963,11 @@ log_expired_value_stats(guint64 dbkey, const struct valuedata *vd)
  * to avoid perturbation in the caller's data structures.
  */
 static void
-values_expire(guint64 dbkey, const struct valuedata *vd)
+values_expire(uint64 dbkey, const struct valuedata *vd)
 {
-	const guint64 *dbatom;
+	const uint64 *dbatom;
 
-	if (g_hash_table_lookup(expired, &dbkey))
+	if (hset_contains(expired, &dbkey))
 		return;
 
 	if (GNET_PROPERTY(dht_storage_debug))
@@ -972,7 +977,7 @@ values_expire(guint64 dbkey, const struct valuedata *vd)
 		g_debug("DHT value DB-key %s expired", uint64_to_string(dbkey));
 
 	dbatom = atom_uint64_get(&dbkey);
-	gm_hash_table_insert_const(expired, dbatom, GINT_TO_POINTER(1));
+	hset_insert(expired, dbatom);
 }
 
 /**
@@ -983,19 +988,18 @@ values_expire(guint64 dbkey, const struct valuedata *vd)
  * happens asynchronously.
  */
 static void
-values_unexpire(guint64 dbkey)
+values_unexpire(uint64 dbkey)
 {
-	gpointer key, value;
+	const void *key;
 	
-	if (g_hash_table_lookup_extended(expired, &dbkey, &key, &value)) {
-		guint64 *dbatom = key;
+	if (hset_contains_extended(expired, &dbkey, &key)) {
+		const uint64 *dbatom = key;
 
-		g_hash_table_remove(expired, &dbkey);
+		hset_remove(expired, &dbkey);
 		atom_uint64_free(dbatom);
 
 		if (GNET_PROPERTY(dht_storage_debug) > 2)
-			g_debug("DHT value DB-key %s un-expired",
-				uint64_to_string(dbkey));
+			g_debug("DHT value DB-key %s un-expired", uint64_to_string(dbkey));
 	}
 }
 
@@ -1007,8 +1011,8 @@ values_unexpire(guint64 dbkey)
  * values_reclaim_expired() is called.  If FALSE is returned, the
  * actual expiration time is returned through `expire', if not NULL.
  */
-gboolean
-values_has_expired(guint64 dbkey, time_t now, time_t *expire)
+bool
+values_has_expired(uint64 dbkey, time_t now, time_t *expire)
 {
 	struct valuedata *vd;
 
@@ -1031,7 +1035,7 @@ values_has_expired(guint64 dbkey, time_t now, time_t *expire)
  * Validate that sender and valued's creator agree on other things than
  * just the KUID: they must agree on everything.
  */
-static gboolean
+static bool
 validate_creator(const knode_t *sender, const dht_value_t *v)
 {
 	const knode_t *creator = v->creator;
@@ -1090,11 +1094,11 @@ wrong:
  *
  * @return error code, STORE_SC_OK meaning we are neither full nor loaded.
  */
-static guint16
+static uint16
 validate_load(const dht_value_t *v)
 {
-	gboolean full;
-	gboolean loaded;
+	bool full;
+	bool loaded;
 
 	keys_get_status(v->id, &full, &loaded);
 
@@ -1114,7 +1118,7 @@ validate_load(const dht_value_t *v)
  *
  * @return error code, STORE_SC_OK meaning quotas are not reached yet.
  */
-static guint16
+static uint16
 validate_quotas(const dht_value_t *v)
 {
 	int count;
@@ -1127,7 +1131,7 @@ validate_quotas(const dht_value_t *v)
 	count = acct_net_get(values_per_class_c, c->addr, NET_CLASS_C_MASK);
 
 	if (GNET_PROPERTY(dht_storage_debug) > 2) {
-		guint32 net = host_addr_ipv4(c->addr) & NET_CLASS_C_MASK;
+		uint32 net = host_addr_ipv4(c->addr) & NET_CLASS_C_MASK;
 
 		g_debug("DHT STORE has %d/%d value%s for class C network %s",
 			count, MAX_VALUES_NET, 1 == count ? "" : "s",
@@ -1136,7 +1140,7 @@ validate_quotas(const dht_value_t *v)
 
 	if (count >= MAX_VALUES_NET) {
 		if (GNET_PROPERTY(dht_storage_debug)) {
-			guint32 net = host_addr_ipv4(c->addr) & NET_CLASS_C_MASK;
+			uint32 net = host_addr_ipv4(c->addr) & NET_CLASS_C_MASK;
 
 			g_debug("DHT STORE rejecting \"%s\": "
 				"has %d/%d value%s for class C network %s",
@@ -1178,10 +1182,10 @@ reject:
  * @return error code, STORE_SC_OK meaning we can accept the value, any
  * other code being an error condition that must be propagated back.
  */
-static guint16
+static uint16
 validate_new_acceptable(const dht_value_t *v)
 {
-	guint16 status;
+	uint16 status;
 
 	/*
 	 * Check whether we have already reached the maximum amount of values
@@ -1305,12 +1309,12 @@ fill_valuedata(struct valuedata *vd, const knode_t *cn, const dht_value_t *v)
  *
  * @return store status code that will be relayed back to the remote node.
  */
-static guint16
+static uint16
 values_remove(const knode_t *kn, const dht_value_t *v)
 {
 	const knode_t *cn = v->creator;
 	const char *reason = NULL;
-	guint64 dbkey;
+	uint64 dbkey;
 
 	if (!kuid_eq(kn->id, cn->id)) {
 		reason = "not from creator";
@@ -1377,7 +1381,7 @@ value_count_republish(struct valuedata *vd)
 {
 	time_t now = tm_time();
 
-	STATIC_ASSERT(DHT_VALUE_EXPIRE < MAX_INT_VAL(gint32));
+	STATIC_ASSERT(DHT_VALUE_EXPIRE < MAX_INT_VAL(int32));
 
 	/*
 	 * We only count a republishing if the value was an original, i.e. not
@@ -1385,11 +1389,11 @@ value_count_republish(struct valuedata *vd)
 	 */
 
 	if (vd->original) {
-		guint32 elapsed = (guint32) delta_time(now, vd->publish);
+		uint32 elapsed = (uint32) delta_time(now, vd->publish);
 
 		vd->n_republish++;
 		vd->s_elapsed_publish =
-			guint32_saturate_add(vd->s_elapsed_publish, elapsed);
+			uint32_saturate_add(vd->s_elapsed_publish, elapsed);
 
 		if (GNET_PROPERTY(dht_storage_debug)) {
 			g_debug("DHT STORE republishing of \"%s\" %s #%u after %s, "
@@ -1412,7 +1416,7 @@ value_count_replication(struct valuedata *vd)
 {
 	time_t now = tm_time();
 
-	STATIC_ASSERT(DHT_VALUE_EXPIRE < MAX_INT_VAL(gint32));
+	STATIC_ASSERT(DHT_VALUE_EXPIRE < MAX_INT_VAL(int32));
 
 	/*
 	 * Only update the sum of the elapsed time between replications if
@@ -1422,9 +1426,9 @@ value_count_replication(struct valuedata *vd)
 	vd->n_replication++;
 
 	if (0 != vd->replicated) {
-		guint32 elapsed = (guint32) delta_time(now, vd->replicated);
+		uint32 elapsed = (uint32) delta_time(now, vd->replicated);
 		vd->s_elapsed_replicat =
-			guint32_saturate_add(vd->s_elapsed_replicat, elapsed);
+			uint32_saturate_add(vd->s_elapsed_replicat, elapsed);
 
 		if (GNET_PROPERTY(dht_storage_debug))
 			g_debug("DHT STORE replication of \"%s\" %s #%u after %s",
@@ -1442,14 +1446,14 @@ value_count_replication(struct valuedata *vd)
  *
  * @return store status code that will be relayed back to the remote node.
  */
-static guint16
+static uint16
 values_publish(const knode_t *kn, const dht_value_t *v)
 {
-	guint64 dbkey;
+	uint64 dbkey;
 	const char *what;
 	struct valuedata *vd = NULL;
 	struct valuedata new_vd;
-	gboolean check_data = FALSE;
+	bool check_data = FALSE;
 
 	/*
 	 * Look whether we already hold this value (in which case it could
@@ -1460,7 +1464,7 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 
 	if (0 == dbkey) {
 		const knode_t *cn = v->creator;
-		guint16 acceptable;
+		uint16 acceptable;
 
 		acceptable = validate_new_acceptable(v);
 		if (acceptable != STORE_SC_OK)
@@ -1500,7 +1504,7 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 		gnet_stats_count_general(GNR_DHT_VALUES_HELD, +1);
 		gnet_stats_count_general(GNR_DHT_PUBLISHED, 1);
 	} else {
-		gboolean is_original = kuid_eq(kn->id, v->creator->id);
+		bool is_original = kuid_eq(kn->id, v->creator->id);
 
 		vd = get_valuedata(dbkey);
 
@@ -1585,7 +1589,7 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 
 	if (check_data) {
 		size_t length;
-		gpointer data;
+		void *data;
 
 		if (values_has_expired(dbkey, tm_time(), NULL))
 			goto expired;
@@ -1618,7 +1622,7 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 
 		g_assert(v->length == vd->length);	/* Ensured by preceding code */
 
-		dbmw_write(db_rawdata, &dbkey, deconstify_gpointer(v->data), v->length);
+		dbmw_write(db_rawdata, &dbkey, deconstify_pointer(v->data), v->length);
 	}
 
 	dbmw_write(db_valuedata, &dbkey, vd, sizeof *vd);
@@ -1659,10 +1663,10 @@ expired:
  *
  * @return store status code that will be relayed back to the remote node.
  */
-guint16
-values_store(const knode_t *kn, const dht_value_t *v, gboolean token)
+uint16
+values_store(const knode_t *kn, const dht_value_t *v, bool token)
 {
-	guint16 status = STORE_SC_OK;
+	uint16 status = STORE_SC_OK;
 
 	knode_check(kn);
 	g_assert(v);
@@ -1752,10 +1756,10 @@ done:
  * @return the DHT value, or NULL if type is not matching.
  */
 dht_value_t *
-values_get(guint64 dbkey, dht_value_type_t type)
+values_get(uint64 dbkey, dht_value_type_t type)
 {
 	struct valuedata *vd;
-	gpointer vdata = NULL;
+	void *vdata = NULL;
 	knode_t *creator;
 	dht_value_t *v;
 
@@ -1790,7 +1794,7 @@ values_get(guint64 dbkey, dht_value_type_t type)
 
 	if (vd->length) {
 		size_t length;
-		gpointer data;
+		void *data;
 
 		data = dbmw_read(db_rawdata, &dbkey, &length);
 
@@ -1817,8 +1821,8 @@ G_GNUC_COLD void
 values_init(void)
 {
 	dbstore_kv_t value_kv =
-		{ sizeof(guint64), NULL, sizeof(struct valuedata), 0 };
-	dbstore_kv_t raw_kv		= { sizeof(guint64), NULL, DHT_VALUE_MAX_LEN, 0 };
+		{ sizeof(uint64), NULL, sizeof(struct valuedata), 0 };
+	dbstore_kv_t raw_kv		= { sizeof(uint64), NULL, DHT_VALUE_MAX_LEN, 0 };
 	dbstore_kv_t expired_kv	= { 2 * KUID_RAW_SIZE, NULL, 0, 0 };
 	dbstore_packing_t value_packing =
 		{ serialize_valuedata, deserialize_valuedata, NULL };
@@ -1852,18 +1856,17 @@ values_init(void)
 
 	values_per_ip = acct_net_create();
 	values_per_class_c = acct_net_create();
-	expired = g_hash_table_new(uint64_hash, uint64_eq);
+	expired = hset_create_any(uint64_hash, NULL, uint64_eq);
 
 	values_expire_ev = cq_periodic_main_add(EXPIRE_PERIOD * 1000,
 		values_periodic_expire, NULL);
 }
 
 static void
-expired_free_kv(gpointer key, gpointer u_val, gpointer u_data)
+expired_free_k(const void *key, void *u_data)
 {
-	guint64 *dbkey = key;
+	const uint64 *dbkey = key;
 
-	(void) u_val;
 	(void) u_data;
 
 	atom_uint64_free(dbkey);
@@ -1879,15 +1882,15 @@ values_close(void)
 	dbstore_delete(db_rawdata);
 	dbstore_delete(db_expired);
 	db_valuedata = db_rawdata = db_expired = NULL;
-	acct_net_free(&values_per_ip);
-	acct_net_free(&values_per_class_c);
+	acct_net_free_null(&values_per_ip);
+	acct_net_free_null(&values_per_class_c);
 	cq_periodic_remove(&values_expire_ev);
 	values_managed = 0;
 
 	gnet_stats_set_general(GNR_DHT_VALUES_HELD, 0);
 
-	g_hash_table_foreach(expired, expired_free_kv, NULL);
-	gm_hash_table_destroy_null(&expired);
+	hset_foreach(expired, expired_free_k, NULL);
+	hset_free_null(&expired);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
