@@ -25,7 +25,7 @@
  * @ingroup lib
  * @file
  *
- * Arc4 random number generator.
+ * ARC4 random number generator.
  *
  * @author Raphael Manfredi
  * @date 2010
@@ -37,11 +37,14 @@
 #include "common.h"
 
 #ifndef HAS_ARC4RANDOM
-guint32 arc4random(void);
+uint32 arc4random(void);
 void arc4random_stir(void);
 void arc4random_addrandom(const unsigned char *dat, int datlen);
 #endif
 
+void arc4random_stir_once(void);
+uint64 arc4random_upto64(uint64 max);
+
 #endif /* _arc4random_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
