@@ -39,7 +39,19 @@
 #include "common.h"
 
 #include "atoms.h"
+#include "constants.h"
 #include "endian.h"
+#include "glib-missing.h"
+#include "hashing.h"
+#include "htable.h"
+#include "log.h"
+#include "misc.h"
+#include "once.h"
+#include "stringify.h"
+#include "walloc.h"
+#include "xmalloc.h"
+
+#include "override.h"		/* Must be the last header included */
 
 #if 0
 #define PROTECT_ATOMS
@@ -74,22 +86,11 @@ typedef enum {
 #define ATOM_SET_MAGIC(a)
 #endif	/* ATOMS_HAVE_MAGIC */
 
-
-#include "stringify.h"
-#include "glib-missing.h"
-#include "log.h"
-#include "walloc.h"
-#include "xmalloc.h"
-
-#include "override.h"		/* Must be the last header included */
-
 union mem_chunk {
   void *next;
   char align[MEM_ALIGNBYTES];
 };
 
-#define ATOM_TYPE_MASK	((size_t) 0x07)
-
 /**
  * Atoms are ref-counted.
  *
@@ -103,8 +104,8 @@ typedef struct atom {
 #endif /* ATOM_HAVE_MAGIC */
 	int refcnt;				/**< Amount of references */
 #ifdef TRACK_ATOMS
-	GHashTable *get;		/**< Allocation spots */
-	GHashTable *free;		/**< Free spots */
+	htable_t *get;			/**< Allocation spots */
+	htable_t *free;			/**< Free spots */
 #endif
 } atom_t;
 
@@ -113,7 +114,7 @@ typedef struct atom {
 		((sizeof(atom_t) % MEM_ALIGNBYTES) ? 1 : 0)))
 
 static inline atom_t *
-atom_from_arena(gconstpointer key)
+atom_from_arena(const void *key)
 {
 	return (atom_t *) (((char *) key) - ARENA_OFFSET);
 }
@@ -131,6 +132,8 @@ atom_check(const atom_t *a)
 	ATOM_MAGIC_CHECK(a);
 }
 
+static bool atoms_inited;
+
 #ifdef PROTECT_ATOMS
 struct mem_pool {
 	union mem_chunk chunks[1];
@@ -257,7 +260,7 @@ mem_free(struct mem_cache *mc, void *p)
 static struct mem_cache *
 atom_get_mc(size_t size)
 {
-	guint i;
+	uint i;
 
 	for (i = 0; i < G_N_ELEMENTS(mem_cache); i++) {
 		if (size <= mem_cache[i]->size)
@@ -271,9 +274,9 @@ atom_get_mc(size_t size)
  * @note All pages partially overlapped by the chunk will be affected.
  */
 static inline void
-mem_protect(gpointer ptr, size_t size)
+mem_protect(void *ptr, size_t size)
 {
-	gpointer p;
+	void *p;
 	size_t ps;
 
 	ps = compat_pagesize();
@@ -290,9 +293,9 @@ mem_protect(gpointer ptr, size_t size)
  * @note All pages partially overlapped by the chunk will be affected.
  */
 static inline void
-mem_unprotect(gpointer ptr, size_t size)
+mem_unprotect(void *ptr, size_t size)
 {
-	gpointer p;
+	void *p;
 	size_t ps;
 
 	ps = compat_pagesize();
@@ -400,60 +403,78 @@ atom_dealloc(atom_t *a, size_t size)
 
 #endif /* PROTECT_ATOMS */
 
-typedef size_t (*len_func_t)(gconstpointer v);
-typedef const char *(*str_func_t)(gconstpointer v);
+typedef size_t (*len_func_t)(const void *v);
+typedef const char *(*str_func_t)(const void *v);
 
 /**
  * Description of atom types.
  */
 typedef struct table_desc {
 	const char *type;			/**< Type of atoms */
-	GHashTable *table;			/**< Table of atoms: "atom value" => 1 */
-	GHashFunc hash_func;		/**< Hashing function for atoms */
-	GCompareFunc eq_func;		/**< Atom equality function */
+	htable_t *table;			/**< Table of atoms: "atom value" -> size */
+	hash_fn_t hash_func;		/**< Hashing function for atoms */
+	eq_fn_t eq_func;			/**< Atom equality function */
 	len_func_t len_func;		/**< Atom length function */
 	str_func_t str_func;		/**< Atom to human-readable string */
 } table_desc_t;
 
-static size_t str_len(gconstpointer v);
-static const char *str_str(gconstpointer v);
-static size_t guid_len(gconstpointer v);
-static const char *guid_str(gconstpointer v);
-static size_t sha1_len(gconstpointer v);
-static const char *sha1_str(gconstpointer v);
-static size_t tth_len(gconstpointer v);
-static const char *tth_str(gconstpointer v);
-static size_t uint64_len(gconstpointer v);
-static const char *uint64_str(gconstpointer v);
-static size_t filesize_len(gconstpointer v);
-static const char *filesize_str(gconstpointer v);
-static size_t uint32_len(gconstpointer v);
-static const char *uint32_str(gconstpointer v);
-static const char *gnet_host_str(gconstpointer v);
+static size_t str_len(const void *v);
+static const char *str_str(const void *v);
+static size_t guid_len(const void *v);
+static const char *guid_str(const void *v);
+static size_t sha1_len(const void *v);
+static const char *sha1_str(const void *v);
+static size_t tth_len(const void *v);
+static const char *tth_str(const void *v);
+static size_t uint64_len(const void *v);
+static const char *uint64_str(const void *v);
+static size_t filesize_len(const void *v);
+static const char *filesize_str(const void *v);
+static size_t uint32_len(const void *v);
+static const char *uint32_str(const void *v);
+static const char *gnet_host_str(const void *v);
+
+#define str_hash	string_mix_hash
+#define str_eq		string_eq
+#define fs_hash		filesize_hash
+#define fs_eq		filesize_eq
+#define fs_len		filesize_len
+#define fs_str		filesize_str
+#define gnh_hash	gnet_host_hash
+#define gnh_eq		gnet_host_eq
+#define gnh_len		gnet_host_length
+#define gnh_str		gnet_host_str
 
 /**
  * The set of all atom types we know about.
  */
 static table_desc_t atoms[] = {
-	{ "String",	NULL, g_str_hash,  g_str_equal, str_len,    str_str  },	/* 0 */
+	{ "String",	NULL, str_hash,    str_eq,      str_len,    str_str  },	/* 0 */
 	{ "GUID",	NULL, guid_hash,   guid_eq,	    guid_len,   guid_str },	/* 1 */
 	{ "SHA1",	NULL, sha1_hash,   sha1_eq,	    sha1_len,   sha1_str },	/* 2 */
 	{ "TTH",	NULL, tth_hash,    tth_eq,	    tth_len,    tth_str },	/* 3 */
 	{ "uint64",	NULL, uint64_hash, uint64_eq,   uint64_len, uint64_str},/* 4 */
-	{ "filesize", NULL,
-		filesize_hash, filesize_eq, filesize_len, filesize_str},		/* 5 */
+	{ "filesize", NULL, fs_hash,   fs_eq,       fs_len,     fs_str },	/* 5 */
 	{ "uint32",	NULL, uint32_hash, uint32_eq,   uint32_len, uint32_str},/* 6 */
-	{ "host", NULL,
-		gnet_host_hash, gnet_host_eq, gnet_host_length, gnet_host_str},	/* 7 */
+	{ "host",   NULL, gnh_hash,    gnh_eq,      gnh_len,    gnh_str },	/* 7 */
 };
 
-static GHashTable *ht_all_atoms;
+#undef str_hash
+#undef str_eq
+#undef fs_hash
+#undef fs_eq
+#undef fs_len
+#undef fs_str
+#undef gnh_hash
+#undef gnh_eq
+#undef gnh_len
+#undef gnh_str
 
 /**
  * @return length of string + trailing NUL.
  */
 static size_t 
-str_len(gconstpointer v)
+str_len(const void *v)
 {
 	return strlen((const char *) v) + 1;
 }
@@ -462,52 +483,16 @@ str_len(gconstpointer v)
  * @return printable form of a string, i.e. self.
  */
 static const char *
-str_str(gconstpointer v)
+str_str(const void *v)
 {
 	return (const char *) v;
 }
 
-/**
- * Hash `len' bytes starting from `data'.
- */
-G_GNUC_HOT guint
-binary_hash(gconstpointer data, size_t len)
-{
-	const unsigned char *key = data;
-	size_t i, remain, t4;
-	guint32 hash;
-
-	remain = len & 0x3;
-	t4 = len & ~0x3U;
-
-	g_assert(remain + t4 == len);
-	g_assert(remain <= 3);
-
-	hash = len;
-	for (i = 0; i < t4; i += 4) {
-		static const guint32 x[] = {
-			0xb0994420, 0x01fa96e3, 0x05066d0e, 0x50c3c22a,
-			0xec99f01f, 0xc0eaa79d, 0x157d4257, 0xde2b8419
-		};
-		hash ^= peek_le32(&key[i]);
-		hash += x[(i >> 2) & 0x7];
-		hash = (hash << 24) ^ (hash >> 8);
-	}
-
-	for (i = 0; i < remain; i++) {
-		hash += key[t4 + i];
-		hash ^= key[t4 + i] << (i * 8);
-		hash = (hash << 24) ^ (hash >> 8);
-	}
-
-	return pointer_hash_func((void *) (size_t) hash);
-}
-
 /**
  * Hash a GUID (16 bytes).
  */
-guint
-guid_hash(gconstpointer key)
+uint
+guid_hash(const void *key)
 {
 	return binary_hash(key, GUID_RAW_SIZE);
 }
@@ -516,7 +501,7 @@ guid_hash(gconstpointer key)
  * Test two GUIDs for equality.
  */
 int
-guid_eq(gconstpointer a, gconstpointer b)
+guid_eq(const void *a, const void *b)
 {
 	return a == b || 0 == memcmp(a, b, GUID_RAW_SIZE);
 }
@@ -525,7 +510,7 @@ guid_eq(gconstpointer a, gconstpointer b)
  * @return length of GUID.
  */
 static size_t 
-guid_len(gconstpointer unused_guid)
+guid_len(const void *unused_guid)
 {
 	(void) unused_guid;
 	return GUID_RAW_SIZE;
@@ -535,7 +520,7 @@ guid_len(gconstpointer unused_guid)
  * @return printable form of a GUID, as pointer to static data.
  */
 static const char *
-guid_str(gconstpointer v)
+guid_str(const void *v)
 {
 	const struct guid *guid = v;
 	return guid_hex_str(guid);
@@ -544,8 +529,8 @@ guid_str(gconstpointer v)
 /**
  * Hash a SHA1 (20 bytes).
  */
-guint
-sha1_hash(gconstpointer key)
+uint
+sha1_hash(const void *key)
 {
 	return binary_hash(key, SHA1_RAW_SIZE);
 }
@@ -554,7 +539,7 @@ sha1_hash(gconstpointer key)
  * Test two SHA1s for equality.
  */
 int
-sha1_eq(gconstpointer a, gconstpointer b)
+sha1_eq(const void *a, const void *b)
 {
 	return a == b || 0 == memcmp(a, b, SHA1_RAW_SIZE);
 }
@@ -563,7 +548,7 @@ sha1_eq(gconstpointer a, gconstpointer b)
  * @return length of SHA1.
  */
 static size_t 
-sha1_len(gconstpointer unused_sha1)
+sha1_len(const void *unused_sha1)
 {
 	(void) unused_sha1;
 	return SHA1_RAW_SIZE;
@@ -573,7 +558,7 @@ sha1_len(gconstpointer unused_sha1)
  * @return printable form of a SHA1, as pointer to static data.
  */
 static const char *
-sha1_str(gconstpointer sha1)
+sha1_str(const void *sha1)
 {
 	return sha1_base32(sha1);
 }
@@ -581,8 +566,8 @@ sha1_str(gconstpointer sha1)
 /**
  * Hash a TTH (24 bytes).
  */
-guint
-tth_hash(gconstpointer key)
+uint
+tth_hash(const void *key)
 {
 	return binary_hash(key, TTH_RAW_SIZE);
 }
@@ -594,7 +579,7 @@ tth_hash(gconstpointer key)
  * NB: This routine is visible for the download mesh.
  */
 int
-tth_eq(gconstpointer a, gconstpointer b)
+tth_eq(const void *a, const void *b)
 {
 	return a == b || 0 == memcmp(a, b, TTH_RAW_SIZE);
 }
@@ -603,7 +588,7 @@ tth_eq(gconstpointer a, gconstpointer b)
  * @return length of TTH.
  */
 static size_t 
-tth_len(gconstpointer unused_tth)
+tth_len(const void *unused_tth)
 {
 	(void) unused_tth;
 	return TTH_RAW_SIZE;
@@ -613,7 +598,7 @@ tth_len(gconstpointer unused_tth)
  * @return printable form of a TTH, as pointer to static data.
  */
 static const char *
-tth_str(gconstpointer tth)
+tth_str(const void *tth)
 {
 	return tth_base32(tth);
 }
@@ -622,17 +607,17 @@ tth_str(gconstpointer tth)
  * @return length of a 64-bit integer.
  */
 static size_t
-uint64_len(gconstpointer unused_v)
+uint64_len(const void *unused_v)
 {
 	(void) unused_v;
-	return sizeof(guint64);
+	return sizeof(uint64);
 }
 
 /**
  * @return length of a filesize_t.
  */
 static size_t
-filesize_len(gconstpointer unused_v)
+filesize_len(const void *unused_v)
 {
 	(void) unused_v;
 	return sizeof(filesize_t);
@@ -642,21 +627,21 @@ filesize_len(gconstpointer unused_v)
  * @return length of a 32-bit integer.
  */
 static size_t
-uint32_len(gconstpointer unused_v)
+uint32_len(const void *unused_v)
 {
 	(void) unused_v;
-	return sizeof(guint32);
+	return sizeof(uint32);
 }
 
 /**
  * @return printable form of a 64-bit integer, as pointer to static data.
  */
 static const char *
-uint64_str(gconstpointer v)
+uint64_str(const void *v)
 {
 	static char buf[UINT64_DEC_BUFLEN];
 
-	uint64_to_string_buf(*(const guint64 *) v, buf, sizeof buf);
+	uint64_to_string_buf(*(const uint64 *) v, buf, sizeof buf);
 	return buf;
 }
 
@@ -666,9 +651,9 @@ uint64_str(gconstpointer v)
  * @return whether both referenced 64-bit integers are equal.
  */
 int
-uint64_eq(gconstpointer a, gconstpointer b)
+uint64_eq(const void *a, const void *b)
 {
-	return *(const guint64 *) a == *(const guint64 *) b;
+	return *(const uint64 *) a == *(const uint64 *) b;
 }
 
 /**
@@ -676,11 +661,11 @@ uint64_eq(gconstpointer a, gconstpointer b)
  *
  * @return the 32-bit hash value for the referenced 64-bit integer.
  */
-guint
-uint64_hash(gconstpointer p)
+uint
+uint64_hash(const void *p)
 {
-	guint64 v = *(const guint64 *) p;
-	return v ^ (v >> 32);
+	uint64 v = *(const uint64 *) p;
+	return integer_hash(v ^ (v >> 32));
 }
 
 /**
@@ -690,9 +675,9 @@ uint64_hash(gconstpointer p)
  * @return whether both referenced 64-bit integers are equal.
  */
 int
-uint64_mem_eq(gconstpointer a, gconstpointer b)
+uint64_mem_eq(const void *a, const void *b)
 {
-	return a == b || 0 == memcmp(a, b, sizeof(guint64));
+	return a == b || 0 == memcmp(a, b, sizeof(uint64));
 }
 
 /**
@@ -701,17 +686,17 @@ uint64_mem_eq(gconstpointer a, gconstpointer b)
  *
  * @return the 32-bit hash value for the referenced 64-bit integer.
  */
-guint
-uint64_mem_hash(gconstpointer p)
+uint
+uint64_mem_hash(const void *p)
 {
-	return binary_hash(p, sizeof(guint64));
+	return binary_hash(p, sizeof(uint64));
 }
 
 /**
  * @return printable form of a filesize_t, as pointer to static data.
  */
 static const char *
-filesize_str(gconstpointer v)
+filesize_str(const void *v)
 {
 	static char buf[UINT64_DEC_BUFLEN];
 
@@ -723,7 +708,7 @@ filesize_str(gconstpointer v)
  * @return printable form of a gnet_host_t *, as pointer to static data.
  */
 static const char *
-gnet_host_str(gconstpointer v)
+gnet_host_str(const void *v)
 {
 	static char buf[HOST_ADDR_PORT_BUFLEN];
 
@@ -737,7 +722,7 @@ gnet_host_str(gconstpointer v)
  * @return whether both referenced 64-bit integers are equal.
  */
 int
-filesize_eq(gconstpointer a, gconstpointer b)
+filesize_eq(const void *a, const void *b)
 {
 	return *(const filesize_t *) a == *(const filesize_t *) b;
 }
@@ -747,10 +732,10 @@ filesize_eq(gconstpointer a, gconstpointer b)
  *
  * @return the 32-bit hash value for the referenced 64-bit integer.
  */
-guint
-filesize_hash(gconstpointer p)
+uint
+filesize_hash(const void *p)
 {
-	guint64 v = *(const filesize_t *) p;
+	uint64 v = *(const filesize_t *) p;
 	return v ^ (v >> 32);
 }
 
@@ -760,9 +745,9 @@ filesize_hash(gconstpointer p)
  * @return whether both referenced 32-bit integers are equal.
  */
 int
-uint32_eq(gconstpointer a, gconstpointer b)
+uint32_eq(const void *a, const void *b)
 {
-	return *(const guint32 *) a == *(const guint32 *) b;
+	return *(const uint32 *) a == *(const uint32 *) b;
 }
 
 /**
@@ -770,45 +755,41 @@ uint32_eq(gconstpointer a, gconstpointer b)
  *
  * @return the 32-bit hash value for the referenced 32-bit integer.
  */
-guint
-uint32_hash(gconstpointer p)
+uint
+uint32_hash(const void *p)
 {
-	guint32 v = *(const guint32 *) p;
-	return v;
+	uint32 v = *(const uint32 *) p;
+	return integer_hash(v);
 }
 
 /**
  * @return printable form of a 32-bit integer, as pointer to static data.
  */
 static const char *
-uint32_str(gconstpointer v)
+uint32_str(const void *v)
 {
 	static char buf[UINT32_DEC_BUFLEN];
 
-	uint32_to_string_buf(*(const guint32 *) v, buf, sizeof buf);
+	uint32_to_string_buf(*(const uint32 *) v, buf, sizeof buf);
 	return buf;
 }
 
 /**
  * Initialize atom structures.
  */
-void
-atoms_init(void)
+static G_GNUC_COLD void
+atoms_init_once(void)
 {
-	gboolean has_setting = FALSE;
+	bool has_setting = FALSE;
 	struct atom_settings {
-		guint8 track_atoms;
-		guint8 protect_atoms;
-		guint8 atoms_have_magic;
+		uint8 track_atoms;
+		uint8 protect_atoms;
+		uint8 atoms_have_magic;
 	} settings;
-	guint i;
-
-	if G_UNLIKELY(ht_all_atoms != NULL)
-		return;		/* Already initialized */
+	uint i;
 
 	ZERO(&settings);
 
-	STATIC_ASSERT(NUM_ATOM_TYPES <= (ATOM_TYPE_MASK + 1));
 	STATIC_ASSERT(NUM_ATOM_TYPES == G_N_ELEMENTS(atoms));
 
 #ifdef PROTECT_ATOMS
@@ -820,9 +801,8 @@ atoms_init(void)
 	for (i = 0; i < G_N_ELEMENTS(atoms); i++) {
 		table_desc_t *td = &atoms[i];
 
-		td->table = g_hash_table_new(td->hash_func, td->eq_func);
+		td->table = htable_create_any(td->hash_func, NULL, td->eq_func);
 	}
-	ht_all_atoms = g_hash_table_new(pointer_hash_func, NULL);
 
 	/*
 	 * Log atoms configuration.
@@ -847,33 +827,17 @@ atoms_init(void)
 			settings.protect_atoms ? "PROTECT_ATOMS " : "",
 			settings.atoms_have_magic ? "ATOMS_HAVE_MAGIC " : "");
 	}
+
+	log_atoms_inited();		/* Atom layer is up */
 }
 
 /**
- * Check whether ``key'' is an atom of type ``type''.
- *
- * @return the size of the atom if found, 0 otherwise.
+ * Initialize atom structures.
  */
-static inline size_t
-atom_is_registered(enum atom_type type, gconstpointer key)
+void
+atoms_init(void)
 {
-	gpointer value;
-
-	if (g_hash_table_lookup_extended(ht_all_atoms, key, NULL, &value)) {
-		/*
-		 * If the address is already registered in the global atom table,
-		 * this is definitely an atom. However, the same memory object
-		 * could be shared by atoms of different types (in theory at least),
-		 * thus we must check whether the types are identical.
-		 */
-
-		if (((size_t) value & ATOM_TYPE_MASK) == (guint) type) {
-			size_t size = (size_t) value & ~ATOM_TYPE_MASK;
-			g_assert(size >= ARENA_OFFSET);
-			return size;
-		}
-	}
-	return 0;
+	once_run(&atoms_inited, atoms_init_once);
 }
 
 /**
@@ -881,16 +845,15 @@ atom_is_registered(enum atom_type type, gconstpointer key)
  *
  * @return TRUE if ``key'' points to a ``type'' atom.
  */
-gboolean
-atom_exists(enum atom_type type, gconstpointer key)
+bool
+atom_exists(enum atom_type type, const void *key)
 {
 	g_assert(key != NULL);
 
-	if G_UNLIKELY(NULL == ht_all_atoms)
+	if G_UNLIKELY(!atoms_inited)
 		return 0;
 
-	return atom_is_registered(type, key) > 0 ||
-		gm_hash_table_contains(atoms[type].table, key);
+	return htable_contains(atoms[type].table, key);
 }
 
 /**
@@ -899,11 +862,12 @@ atom_exists(enum atom_type type, gconstpointer key)
  *
  * @return the atom's value.
  */
-gconstpointer
-atom_get(enum atom_type type, gconstpointer key)
+const void *
+atom_get(enum atom_type type, const void *key)
 {
 	table_desc_t *td;
-	gpointer orig_key;
+	const void *orig_key;
+	void *value;
 	size_t size;
 
 	STATIC_ASSERT(0 == ARENA_OFFSET % MEM_ALIGNBYTES);
@@ -912,33 +876,16 @@ atom_get(enum atom_type type, gconstpointer key)
     g_assert(key != NULL);
 	g_assert(UNSIGNED(type) < G_N_ELEMENTS(atoms));
 
-	if G_UNLIKELY(NULL == ht_all_atoms)
+	if G_UNLIKELY(!atoms_inited)
 		atoms_init();
 
 	td = &atoms[type];		/* Where atoms of this type are held */
 
-	size = atom_is_registered(type, key);
-	if (size > 0) {
-		/* Atom already exists for key and type */
-		orig_key = deconstify_gpointer(key);
+	if (htable_lookup_extended(td->table, key, &orig_key, &value)) {
+		size = pointer_to_size(value);
+		g_assert(size >= ARENA_OFFSET);
 	} else {
-		gpointer value;
-
-		/*
-		 * Normally, if atom exists, it will be found by atom_is_registered().
-		 * If not, look in the type-specific table.
-		 *
-		 * This happens when the first few bytes of the atom arena are shared
-		 * by atoms of different types (it creates conflicts in the global
-		 * ht_all_atoms table).
-		 */
-
-		if (g_hash_table_lookup_extended(td->table, key, &orig_key, &value)) {
-			size = (size_t) value;
-			g_assert(size >= ARENA_OFFSET);
-		} else {
-			size = 0;
-		}
+		size = 0;
 	}
 
 	/*
@@ -957,7 +904,6 @@ atom_get(enum atom_type type, gconstpointer key)
 		return orig_key;
 	} else {
 		size_t len;
-		gpointer value;
 		atom_t *a;
 
 		/*
@@ -966,7 +912,7 @@ atom_get(enum atom_type type, gconstpointer key)
 
 		len = (*td->len_func)(key);
 		g_assert(len < ((size_t) -1) - ARENA_OFFSET);
-		size = round_size_fast((ATOM_TYPE_MASK + 1), ARENA_OFFSET + len);
+		size = round_size_fast(MEM_ALIGNBYTES, ARENA_OFFSET + len);
 
 		a = atom_alloc(size);
 		a->refcnt = 1;
@@ -977,9 +923,7 @@ atom_get(enum atom_type type, gconstpointer key)
 		 * Insert atom in table.
 		 */
 
-		value = (gpointer) (size | (guint) type);
-		g_hash_table_insert(ht_all_atoms, atom_arena(a), value);
-		g_hash_table_insert(td->table, atom_arena(a), (gpointer) size);
+		htable_insert(td->table, atom_arena(a), size_to_pointer(size));
 
 		return atom_arena(a);
 	}
@@ -990,7 +934,7 @@ atom_get(enum atom_type type, gconstpointer key)
  * Dispose of atom if nobody references it anymore.
  */
 void
-atom_free(enum atom_type type, gconstpointer key)
+atom_free(enum atom_type type, const void *key)
 {
 	size_t size;
 	atom_t *a;
@@ -998,8 +942,8 @@ atom_free(enum atom_type type, gconstpointer key)
     g_assert(key != NULL);
 	g_assert(UNSIGNED(type) < G_N_ELEMENTS(atoms));
 
-	size = atom_is_registered(type, key);
-	g_assert(size > 0);
+	size = pointer_to_size(htable_lookup(atoms[type].table, key));
+	g_assert(size >= ARENA_OFFSET);
 
 	a = atom_from_arena(key);
 	g_assert(a->refcnt > 0);
@@ -1010,11 +954,7 @@ atom_free(enum atom_type type, gconstpointer key)
 
 	atom_unprotect(a, size);
 	if (--a->refcnt == 0) {
-		table_desc_t *td;
-
-		td = &atoms[type];		/* Where atoms of this type are held */
-		g_hash_table_remove(td->table, key);
-		g_hash_table_remove(ht_all_atoms, key);
+		htable_remove(atoms[type].table, key);
 		atom_dealloc(a, size);
 	} else {
 		atom_protect(a, size);
@@ -1033,14 +973,14 @@ struct spot {
  *
  * @returns the atom's value.
  */
-gconstpointer
-atom_get_track(enum atom_type type, gconstpointer key, char *file, int line)
+const void *
+atom_get_track(enum atom_type type, const void *key, char *file, int line)
 {
-	gconstpointer atom;
+	const void *atom;
 	atom_t *a;
 	char buf[512];
-	gpointer k;
-	gpointer v;
+	const void *k;
+	void *v;
 	struct spot *sp;
 
 	atom = atom_get(type, key);
@@ -1051,33 +991,34 @@ atom_get_track(enum atom_type type, gconstpointer key, char *file, int line)
 	 */
 
 	if (a->refcnt == 1) {
-		a->get = g_hash_table_new(g_str_hash, g_str_equal);
-		a->free = g_hash_table_new(g_str_hash, g_str_equal);
+		a->get = htable_create(HASH_KEY_STRING, 0);
+		a->free = htable_create(HASH_KEY_STRING, 0);
 	}
 
 	gm_snprintf(buf, sizeof(buf), "%s:%d", short_filename(file), line);
 
-	if (g_hash_table_lookup_extended(a->get, buf, &k, &v)) {
+	if (htable_lookup_extended(a->get, buf, &k, &v)) {
 		sp = (struct spot *) v;
 		sp->count++;
 	} else {
 		WALLOC(sp);
 		sp->count = 1;
-		g_hash_table_insert(a->get, g_strdup(buf), sp);
+		htable_insert(a->get, constant_str(buf), sp);
 	}
 
 	return atom;
 }
 
 /**
- * Free key/value pair of the tracking table.
+ * Free value from the tracking table.
  */
-static gboolean
-tracking_free_kv(gpointer key, gpointer value, gpointer uu_user)
+static void
+tracking_free_kv(const void *key, void *value, void *uu_user)
 {
 	(void) uu_user;
 
-	G_FREE_NULL(key);
+	/* The key is a constant */
+
 	wfree(value, sizeof(struct spot));
 	return TRUE;
 }
@@ -1086,10 +1027,10 @@ tracking_free_kv(gpointer key, gpointer value, gpointer uu_user)
  * Get rid of the tracking hash table.
  */
 static void
-destroy_tracking_table(GHashTable *h)
+destroy_tracking_table(htable_t *h)
 {
-	g_hash_table_foreach_remove(h, tracking_free_kv, NULL);
-	g_hash_table_destroy(h);
+	htable_foreach(h, tracking_free_kv, NULL);
+	htable_free_null(&h);
 }
 
 /**
@@ -1098,33 +1039,32 @@ destroy_tracking_table(GHashTable *h)
  * Dispose of atom if nobody references it anymore.
  */
 void
-atom_free_track(enum atom_type type, gconstpointer key, char *file, int line)
+atom_free_track(enum atom_type type, const void *key, char *file, int line)
 {
 	atom_t *a;
 	char buf[512];
-	gpointer k;
-	gpointer v;
+	void *v;
 	struct spot *sp;
 
 	a = atom_from_arena(key);
 
 	/*
-	 * If we're going to free the atom, dispose the tracking tables.
+	 * If we're going to free the atom, dispose of the tracking tables.
 	 */
 
-	if (a->refcnt == 1) {
+	if (1 == a->refcnt) {
 		destroy_tracking_table(a->get);
 		destroy_tracking_table(a->free);
 	} else {
 		gm_snprintf(buf, sizeof(buf), "%s:%d", short_filename(file), line);
 
-		if (g_hash_table_lookup_extended(a->free, buf, &k, &v)) {
+		if (htable_lookup_extended(a->free, buf, NULL, &v)) {
 			sp = (struct spot *) v;
 			sp->count++;
 		} else {
 			WALLOC(sp);
 			sp->count = 1;
-			g_hash_table_insert(a->free, g_strdup(buf), sp);
+			htable_insert(a->free, constant_str(buf), sp);
 		}
 	}
 
@@ -1136,10 +1076,10 @@ atom_free_track(enum atom_type type, gconstpointer key, char *file, int line)
  * amount of such operations.
  */
 static void
-dump_tracking_entry(gpointer key, gpointer value, gpointer user)
+dump_tracking_entry(const void *key, void *value, void *user)
 {
-	struct spot *sp = (struct spot *) value;
-	const char *what = (const char *) user;
+	struct spot *sp = value;
+	const char *what = user;
 
 	g_warning("%10d %s at \"%s\"", sp->count, what, (char *) key);
 }
@@ -1148,14 +1088,14 @@ dump_tracking_entry(gpointer key, gpointer value, gpointer user)
  * Dump the values held in the tracking table `h'.
  */
 static void
-dump_tracking_table(gpointer atom, GHashTable *h, char *what)
+dump_tracking_table(void *atom, htable_t *h, char *what)
 {
-	guint count = g_hash_table_size(h);
+	size_t count = htable_count(h);
 
-	g_warning("all %u %s spot%s for %p:",
+	g_warning("all %zu %s spot%s for %p:",
 		count, what, count == 1 ? "" : "s", atom);
 
-	g_hash_table_foreach(h, dump_tracking_entry, what);
+	htable_foreach(h, dump_tracking_entry, what);
 }
 
 #endif	/* TRACK_ATOMS */
@@ -1163,8 +1103,8 @@ dump_tracking_table(gpointer atom, GHashTable *h, char *what)
 /**
  * Warning about existing atom that should have been freed.
  */
-static gboolean
-atom_warn_free(gpointer key, gpointer unused_value, gpointer udata)
+static void
+atom_warn_free(const void *key, void *unused_value, void *udata)
 {
 	atom_t *a = atom_from_arena(key);
 	table_desc_t *td = udata;
@@ -1182,12 +1122,9 @@ atom_warn_free(gpointer key, gpointer unused_value, gpointer udata)
 #endif
 
 	/*
-	 * Don't free the entry, so that we know where the leak originates from
-	 * when running under -DUSE_DMALLOC or via valgrind.
+	 * Don't free the entry, so that we know where the leak originates from.
 	 *		--RAM, 02/02/2003
 	 */
-
-	return TRUE;
 }
 
 /**
@@ -1196,15 +1133,14 @@ atom_warn_free(gpointer key, gpointer unused_value, gpointer udata)
 void
 atoms_close(void)
 {
-	guint i;
+	uint i;
 
 	for (i = 0; i < G_N_ELEMENTS(atoms); i++) {
 		table_desc_t *td = &atoms[i];
 
-		g_hash_table_foreach_remove(td->table, atom_warn_free, td);
-		gm_hash_table_destroy_null(&td->table);
+		htable_foreach(td->table, atom_warn_free, td);
+		htable_free_null(&td->table);
 	}
-	gm_hash_table_destroy_null(&ht_all_atoms);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
