@@ -37,9 +37,9 @@
 #ifndef _compat_misc_h_
 #define _compat_misc_h_
 
-gboolean compat_is_superuser(void);
+bool compat_is_superuser(void);
 int compat_daemonize(const char *directory);
-gboolean compat_process_is_alive(pid_t pid);
+bool compat_process_is_alive(pid_t pid);
 
 void compat_fadvise_sequential(int fd, fileoffset_t offset, fileoffset_t size);
 void compat_fadvise_noreuse(int fd, fileoffset_t offset, fileoffset_t size);
