@@ -83,34 +83,14 @@
 #define m1mask 0xf
 #define hidden_bit 0x100000
 
-/* Little-Endian IEEE Double Floats */
-struct dblflt_le {
-    unsigned int m4: 16;
-    unsigned int m3: 16;
-    unsigned int m2: 16;
-    unsigned int m1: 4;
-    unsigned int e: 11;
-    unsigned int s: 1;
-};
-
-/* Big-Endian IEEE Double Floats */
-struct dblflt_be {
-    unsigned int s: 1;
-    unsigned int e: 11;
-    unsigned int m1: 4;
-    unsigned int m2: 16;
-    unsigned int m3: 16;
-    unsigned int m4: 16;
-};
-
 #define BIGSIZE 24
 #define MIN_E -1074
 #define MAX_FIVE 325
-#define B_P1 ((guint64)1 << 52)
+#define B_P1 ((uint64) 1 << 52)
 
 typedef struct {
    int l;
-   guint64 d[BIGSIZE];
+   uint64 d[BIGSIZE];
 } bignum_t;
 
 /*
@@ -137,8 +117,7 @@ static struct float_context {
 
 static bignum_t five[MAX_FIVE];
 static int recursion_level = -1;
-static gboolean float_inited;
-static gboolean float_little_endian;
+static bool float_inited;
 
 #define R			float_context[recursion_level].c_R
 #define S			float_context[recursion_level].c_S
@@ -156,7 +135,7 @@ static gboolean float_little_endian;
 #define MP			float_context[recursion_level].c_MP
 
 #define ADD(x, y, z, k) {\
-	guint64 x_add, z_add;\
+	uint64 x_add, z_add;\
 	x_add = (x);\
 	if ((k))\
 		z_add = x_add + (y) + 1, (k) = (z_add <= x_add);\
@@ -166,7 +145,7 @@ static gboolean float_little_endian;
 }
 
 #define SUB(x, y, z, b) {\
-	guint64 x_sub, y_sub;\
+	uint64 x_sub, y_sub;\
 	x_sub = (x); y_sub = (y);\
 	if ((b))\
 		(z) = x_sub - y_sub - 1, b = (y_sub >= x_sub);\
@@ -175,7 +154,7 @@ static gboolean float_little_endian;
 }
 
 #define MUL(x, y, z, k) {\
-	guint64 x_mul, low, high;\
+	uint64 x_mul, low, high;\
 	x_mul = (x);\
 	low = (x_mul & 0xffffffff) * (y) + (k);\
 	high = (x_mul >> 32) * (y) + (low >> 32);\
@@ -184,7 +163,7 @@ static gboolean float_little_endian;
 }
 
 #define SLL(x, y, z, k) {\
-	guint64 x_sll = (x);\
+	uint64 x_sll = (x);\
 	(z) = (x_sll << (y)) | (k);\
 	(k) = x_sll >> (64 - (y));\
 }
@@ -194,14 +173,14 @@ static void
 print_big(bignum_t *x)
 {
 	int i;
-	guint64 *p;
+	uint64 *p;
 
 	printf("#x");
 	i = x->l;
 	safety_assert(i < BIGSIZE);
 	p = &x->d[i];
 	for (p = &x->d[i]; i >= 0; i--) {
-		guint64 b = *p--;
+		uint64 b = *p--;
 		printf("%08x%08x", (int)(b >> 32), (int)(b & 0xffffffff));
 	}
 }
@@ -211,7 +190,7 @@ static void
 mul10(bignum_t *x)
 {
 	int i, l;
-	guint64 *p, k;
+	uint64 *p, k;
 
 	safety_assert(x->l < BIGSIZE);
 	safety_assert(x->l >= 0);
@@ -229,11 +208,11 @@ mul10(bignum_t *x)
 }
 
 static void
-big_short_mul(bignum_t *x, guint64 y, bignum_t *z)
+big_short_mul(bignum_t *x, uint64 y, bignum_t *z)
 {
 	int i, xl, zl;
-	guint64 *xp, *zp, k;
-	guint32 high, low;
+	uint64 *xp, *zp, k;
+	uint32 high, low;
 
 	safety_assert(x->l < BIGSIZE);
 	safety_assert(x->l >= 0);
@@ -245,7 +224,7 @@ big_short_mul(bignum_t *x, guint64 y, bignum_t *z)
 	high = y >> 32;
 	low = y & 0xffffffff;
 	for (i = xl, k = 0; i >= 0; i--, xp++, zp++) {
-		guint64 xlow, xhigh, z0, t, c, z1;
+		uint64 xlow, xhigh, z0, t, c, z1;
 		xlow = *xp & 0xffffffff;
 		xhigh = *xp >> 32;
 		z0 = (xlow * low) + k; /* Cout is (z0 < k) */
@@ -282,7 +261,7 @@ static void
 one_shift_left(int y, bignum_t *z)
 {
 	int n, m, i;
-	guint64 *zp;
+	uint64 *zp;
 
 	n = y / 64;
 	m = y % 64;
@@ -290,7 +269,7 @@ one_shift_left(int y, bignum_t *z)
 	zp = &z->d[0];
 	for (i = n; i > 0; i--)
 		*zp++ = 0;
-	*zp = (guint64)1 << m;
+	*zp = (uint64)1 << m;
 	z->l = n;
 
 	safety_assert(z->l >= 0);
@@ -298,10 +277,10 @@ one_shift_left(int y, bignum_t *z)
 }
 
 static void
-short_shift_left(guint64 x, int y, bignum_t *z)
+short_shift_left(uint64 x, int y, bignum_t *z)
 {
 	int n, m, i, zl;
-	guint64 *zp;
+	uint64 *zp;
 
 	n = y / 64;
 	m = y % 64;
@@ -313,7 +292,7 @@ short_shift_left(guint64 x, int y, bignum_t *z)
 	if (m == 0) {
 		*zp = x;
 	} else {
-		guint64 high = x >> (64 - m);
+		uint64 high = x >> (64 - m);
 		*zp = x << m;
 		if (high != 0)
 			*++zp = high, zl++;
@@ -328,7 +307,7 @@ static void
 big_shift_left(bignum_t *x, int y, bignum_t *z)
 {
 	int n, m, i, xl, zl;
-	guint64 *xp, *zp, k;
+	uint64 *xp, *zp, k;
 
 	n = y / 64;
 	m = y % 64;
@@ -359,7 +338,7 @@ static int
 big_comp(bignum_t *x, bignum_t *y)
 {
 	int i, xl, yl;
-	guint64 *xp, *yp;
+	uint64 *xp, *yp;
 
 	safety_assert(x->l < BIGSIZE);
 	safety_assert(x->l >= 0);
@@ -375,8 +354,8 @@ big_comp(bignum_t *x, bignum_t *y)
 	xp = &x->d[xl];
 	yp = &y->d[xl];
 	for (i = xl; i >= 0; i--, xp--, yp--) {
-		guint64 a = *xp;
-		guint64 b = *yp;
+		uint64 a = *xp;
+		uint64 b = *yp;
 
 		if (a > b) return 1;
 		else if (a < b) return -1;
@@ -388,7 +367,7 @@ static int
 sub_big(bignum_t *x, bignum_t *y, bignum_t *z)
 {
 	int xl, yl, zl, b, i;
-	guint64 *xp, *yp, *zp;
+	uint64 *xp, *yp, *zp;
 
 	safety_assert(x->l < BIGSIZE);
 	safety_assert(x->l >= 0);
@@ -406,7 +385,7 @@ sub_big(bignum_t *x, bignum_t *y, bignum_t *z)
 	for (i = yl, b = 0; i >= 0; i--)
 		SUB(*xp++, *yp++, *zp++, b);
 	for (i = xl-yl; b && i > 0; i--) {
-		guint64 x_sub;
+		uint64 x_sub;
 		x_sub = *xp++;
 		*zp++ = x_sub - 1;
 		b = (x_sub == 0);
@@ -430,7 +409,7 @@ static void
 add_big(bignum_t *x, bignum_t *y, bignum_t *z)
 {
 	int xl, yl, k, i;
-	guint64 *xp, *yp, *zp;
+	uint64 *xp, *yp, *zp;
 
 	safety_assert(x->l < BIGSIZE);
 	safety_assert(x->l >= 0);
@@ -453,7 +432,7 @@ add_big(bignum_t *x, bignum_t *y, bignum_t *z)
 	for (i = yl, k = 0; i >= 0; i--)
 		ADD(*xp++, *yp++, *zp++, k);
 	for (i = xl-yl; k && i > 0; i--) {
-		guint64 z_add;
+		uint64 z_add;
 		z_add = *xp++ + 1;
 		k = (z_add == 0);
 		*zp++ = z_add;
@@ -523,31 +502,7 @@ float_init(void)
 {
 	int n, i, l;
 	bignum_t *b;
-	guint64 *xp, *zp, k;
-
-	/*
-	 * Determine dynamically whether floats are stored as
-	 * little- or big-endian.  This will only impose negligeable
-	 * overhead to the formatting.
-	 *		--RAM, 2011-11-06
-	 */
-
-	{
-		double v = 4.0;
-		struct dblflt_le *le;
-		struct dblflt_be *be;
-
-		le = (struct dblflt_le *) &v;
-		be = (struct dblflt_be *) &v;
-
-		if (0 == le->s && 1025 == le->e) {
-			float_little_endian = TRUE;
-		} else if (0 == be->s && 1025 == be->e) {
-			float_little_endian = FALSE;
-		} else {
-			g_error("your double values are not stored in IEEE format");
-		}
-	}
+	uint64 *xp, *zp, k;
 
 	five[0].l = l = 0;
 	five[0].d[0] = 5;
@@ -593,33 +548,24 @@ add_cmp(int use_mp)
 	return big_comp(&sum, &S);
 }
 
-static guint64
+static uint64
 float_decompose(double v, int *sign, int *ep)
 {
-	guint64 f;
+	uint64 f;
 	int e;
+	union double_decomposition dc;
 
-	STATIC_ASSERT(sizeof v == sizeof(struct dblflt_le));
-	STATIC_ASSERT(sizeof(struct dblflt_le) == sizeof(struct dblflt_be));
+	STATIC_ASSERT(sizeof v == sizeof(dc.d));
 
 	/* decompose float into sign, mantissa & exponent */
-	if (float_little_endian) {
-		struct dblflt_le *x = (struct dblflt_le *)&v;
-		*sign = x->s;
-		e = x->e;
-		f = (guint64)(x->m1 << 16 | x->m2) << 32 |
-			(guint32)(x->m3 << 16 | x->m4);
-	} else {
-		struct dblflt_be *x = (struct dblflt_be *)&v;
-		*sign = x->s;
-		e = x->e;
-		f = (guint64)(x->m1 << 16 | x->m2) << 32 |
-			(guint32)(x->m3 << 16 | x->m4);
-	}
+	dc.value = v;
+	*sign = dc.d.s;
+	e = dc.d.e;
+	f = ((uint64) dc.d.mh << 32) | (uint32) dc.d.ml;
 
 	if (e != 0) {
 		*ep = e - bias - bitstoright;
-		f |= (guint64)hidden_bit << 32;
+		f |= (uint64) hidden_bit << 32;
 	} else if (f != 0) {
 		/* denormalized */
 		*ep = 1 - bias - bitstoright;
@@ -672,8 +618,8 @@ G_STMT_START {							\
 size_t
 float_dragon(char *dest, size_t len, double v, int *exponent)
 {
-	int sign, e, f_n, m_n, i, d, tc1, tc2;
-	guint64 f;
+	int sign, e = 0, f_n, m_n, i, d, tc1, tc2;
+	uint64 f;
 	int ruf, k, sl = 0, slr = 0;
 	int use_mp;
 	char *bp = dest;
@@ -703,9 +649,9 @@ float_dragon(char *dest, size_t len, double v, int *exponent)
 		k = estimate(e+52);
 	} else {
 		int n;
-		guint64 y;
+		uint64 y;
 
-		for (n = e+52, y = (guint64)1 << 52; f < y; n--)
+		for (n = e+52, y = (uint64)1 << 52; f < y; n--)
 			y >>= 1;
 		k = estimate(n);
 	}
@@ -794,20 +740,20 @@ again:
 		if (R.l < sl) {
 			d = 0;
 		} else if (R.l == sl) {
-			guint64 *p;
+			uint64 *p;
 
 			p = &R.d[sl];
 			d = *p >> slr;
-			*p &= ((guint64)1 << slr) - 1;
+			*p &= ((uint64)1 << slr) - 1;
 			for (i = sl; (i > 0) && (*p == 0); i--) p--;
 			R.l = i;
 		} else {
-			guint64 *p;
+			uint64 *p;
 
 			p = &R.d[sl+1];
 			d = *p << (64 - slr) | *(p-1) >> slr;
 			p--;
-			*p &= ((guint64)1 << slr) - 1;
+			*p &= ((uint64)1 << slr) - 1;
 			for (i = sl; (i > 0) && (*p == 0); i--) p--;
 			R.l = i;
 		}
@@ -878,8 +824,8 @@ done:
 size_t
 float_fixed(char *dest, size_t len, double v, int prec, int *exponent)
 {
-	int sign, e, f_n, i, d, n;
-	guint64 f;
+	int sign, e = 0, f_n, i, d, n;
+	uint64 f;
 	int k, sl = 0, slr = 0;
 	char *bp = dest;
 	size_t remain = len;
@@ -915,9 +861,9 @@ float_fixed(char *dest, size_t len, double v, int prec, int *exponent)
 	if (e > MIN_E) {
 		k = estimate(e+52);
 	} else {
-		guint64 y;
+		uint64 y;
 
-		for (n = e+52, y = (guint64)1 << 52; f < y; n--) y >>= 1;
+		for (n = e+52, y = (uint64)1 << 52; f < y; n--) y >>= 1;
 		k = estimate(n);
 	}
 
@@ -974,21 +920,21 @@ float_fixed(char *dest, size_t len, double v, int prec, int *exponent)
 			if (R.l < sl) {
 				d = 0;
 			} else if (R.l == sl) {
-				guint64 *p;
+				uint64 *p;
 
 				p = &R.d[sl];
 				d = *p >> slr;
-				*p &= ((guint64)1 << slr) - 1;
+				*p &= ((uint64)1 << slr) - 1;
 				for (i = sl; (i > 0) && (*p == 0); i--)
 					p--;
 				R.l = i;
 			} else {
-				guint64 *p;
+				uint64 *p;
 
 				p = &R.d[sl+1];
 				d = *p << (64 - slr) | *(p-1) >> slr;
 				p--;
-				*p &= ((guint64)1 << slr) - 1;
+				*p &= ((uint64)1 << slr) - 1;
 				for (i = sl; (i > 0) && (*p == 0); i--)
 					p--;
 				R.l = i;
