@@ -25,24 +25,20 @@
 #define _hashlist_h_
 
 #include "common.h"
-#include "glib-missing.h"
 
 typedef struct hash_list_iter hash_list_iter_t;
 typedef struct hash_list hash_list_t;
 
-typedef gboolean (*hashlist_cbr_t)(void *key, void *u);
-typedef void (*hashlist_destroy_cb)(void *data);
-
-hash_list_t *hash_list_new(GHashFunc, GEqualFunc);
+hash_list_t *hash_list_new(hash_fn_t, eq_fn_t);
 void hash_list_free(hash_list_t **);
-void hash_list_free_all(hash_list_t **hl_ptr, hashlist_destroy_cb freecb);
+void hash_list_free_all(hash_list_t **hl_ptr, free_fn_t freecb);
 void *hash_list_remove(hash_list_t *, const void *key);
 void *hash_list_remove_head(hash_list_t *);
 void *hash_list_remove_tail(hash_list_t *);
 void *hash_list_shift(hash_list_t *);
 void hash_list_append(hash_list_t *, const void *key);
 void hash_list_prepend(hash_list_t *, const void *key);
-void hash_list_insert_sorted(hash_list_t *, const void *key, GCompareFunc);
+void hash_list_insert_sorted(hash_list_t *, const void *key, cmp_fn_t);
 void hash_list_moveto_head(hash_list_t *, const void *key);
 void hash_list_moveto_tail(hash_list_t *, const void *key);
 void *hash_list_head(const hash_list_t *);
@@ -52,34 +48,35 @@ void *hash_list_previous(hash_list_t *, const void *key);
 void hash_list_clear(hash_list_t *hl);
 unsigned hash_list_length(const hash_list_t *);
 GList *hash_list_list(hash_list_t *);
-void hash_list_sort(hash_list_t *, GCompareFunc);
-void hash_list_sort_with_data(hash_list_t *, GCompareDataFunc, void *);
+void hash_list_sort(hash_list_t *, cmp_fn_t);
+void hash_list_sort_with_data(hash_list_t *, cmp_data_fn_t, void *);
+void hash_list_shuffle(hash_list_t *hl);
 
 hash_list_iter_t *hash_list_iterator(hash_list_t *);
 hash_list_iter_t *hash_list_iterator_tail(hash_list_t *);
 hash_list_iter_t *hash_list_iterator_at(hash_list_t *, const void *key);
 void hash_list_iter_release(hash_list_iter_t **);
-gboolean hash_list_iter_has_next(const hash_list_iter_t *) G_GNUC_PURE;
-gboolean hash_list_iter_has_previous(const hash_list_iter_t *) G_GNUC_PURE;
-gboolean hash_list_iter_has_more(const hash_list_iter_t *iter) G_GNUC_PURE;
+bool hash_list_iter_has_next(const hash_list_iter_t *) G_GNUC_PURE;
+bool hash_list_iter_has_previous(const hash_list_iter_t *) G_GNUC_PURE;
+bool hash_list_iter_has_more(const hash_list_iter_t *iter) G_GNUC_PURE;
 void *hash_list_iter_next(hash_list_iter_t *);
 void *hash_list_iter_previous(hash_list_iter_t *);
 void *hash_list_iter_move(hash_list_iter_t *iter);
 void *hash_list_iter_remove(hash_list_iter_t *iter);
 
-gboolean hash_list_find(hash_list_t *, const void *key, const void **orig_key);
-gboolean hash_list_contains(hash_list_t *, const void *key);
-void hash_list_foreach(const hash_list_t *, GFunc, void *user_data);
-size_t hash_list_foreach_remove(hash_list_t *hl, hashlist_cbr_t func, void *u);
+bool hash_list_find(hash_list_t *, const void *key, const void **orig_key);
+bool hash_list_contains(hash_list_t *, const void *key);
+void hash_list_foreach(const hash_list_t *, data_fn_t, void *);
+size_t hash_list_foreach_remove(hash_list_t *, data_rm_fn_t, void *);
 
 void *hash_list_remove_position(hash_list_t *hl, const void *key);
 void hash_list_insert_position(hash_list_t *hl, const void *key, void *pos);
 void hash_list_forget_position(void *position);
 
-static inline hashlist_destroy_cb
+static inline free_fn_t
 cast_to_hashlist_destroy(func_ptr_t fn)
 {
-	return (hashlist_destroy_cb) fn;
+	return (free_fn_t) fn;
 }
 
 #endif	/* _hashlist_h_ */
