@@ -36,6 +36,7 @@
 
 #include "common.h"
 
+#include "misc.h"
 #include "ascii.h"
 #include "atoms.h"
 #include "base16.h"
@@ -44,21 +45,22 @@
 #include "concat.h"
 #include "endian.h"
 #include "entropy.h"
+#include "glib-missing.h"
 #include "halloc.h"
+#include "htable.h"
 #include "html_entities.h"
 #include "log.h"				/* For log_file_printable() */
-#include "misc.h"
-#include "glib-missing.h"
-#include "sha1.h"
+#include "mempcpy.h"
 #include "parse.h"
 #include "path.h"
 #include "pow2.h"
 #include "random.h"
+#include "sha1.h"
 #include "stringify.h"
 #include "tm.h"
 #include "unsigned.h"
-#include "walloc.h"
 #include "utf8.h"
+#include "walloc.h"
 
 #include "if/core/guid.h"
 
@@ -133,7 +135,7 @@ is_strcaseprefix(const char *str, const char *prefix)
  * @param len		length of ``str'', (size_t)-1 means compute it
  * @param suffix	the suffix to look for (NUL-terminated string)
  */
-gboolean
+bool
 is_strsuffix(const char *str, size_t len, const char *suffix)
 {
 	size_t suffix_len;
@@ -278,7 +280,7 @@ strchomp(char *str, size_t len)
 /**
  * Check whether path is a directory.
  */
-gboolean
+bool
 is_directory(const char *pathname)
 {
 	filestat_t st;
@@ -290,7 +292,7 @@ is_directory(const char *pathname)
 /**
  * Check whether path points to a regular file.
  */
-gboolean
+bool
 is_regular(const char *pathname)
 {
 	filestat_t st;
@@ -302,7 +304,7 @@ is_regular(const char *pathname)
 /**
  * Check whether path is a symbolic link.
  */
-gboolean
+bool
 is_symlink(const char *pathname)
 #if defined(HAS_LSTAT)
 {
@@ -403,18 +405,15 @@ seek_to_filepos(int fd, filesize_t pos)
 filesize_t
 get_random_file_offset(const filesize_t size)
 {
-	filesize_t offset;
-
-	offset = 0;
-	if (size > 1) {
-		random_bytes(&offset, sizeof offset);
-		offset %= size - 1;
+	if (sizeof(size) == sizeof(uint64)) {
+		return random_value64(size - 1);
+	} else {
+		return random_value(size - 1);
 	}
-	return offset;
 }
 
-static inline guint
-filesize_fraction(filesize_t size, filesize_t part, guint base)
+static inline uint
+filesize_fraction(filesize_t size, filesize_t part, uint base)
 {
 	filesize_t x;
 
@@ -436,7 +435,7 @@ filesize_fraction(filesize_t size, filesize_t part, guint base)
 }
 
 #define GENERATE_FILESIZE_PER_X(base) \
-guint \
+uint \
 filesize_per_ ## base (filesize_t size, filesize_t part) \
 { \
 	return filesize_fraction(size, part, base); \
@@ -447,21 +446,21 @@ GENERATE_FILESIZE_PER_X(1000)
 GENERATE_FILESIZE_PER_X(10000)
 #undef GENERATE_FILESIZE_PER_X
 
-static inline guint
-kilo(gboolean metric)
+static inline uint
+kilo(bool metric)
 {
 	return metric ? 1000 : 1024;
 }
 
 static inline const char *
-byte_suffix(gboolean metric)
+byte_suffix(bool metric)
 {
 	static const char suffix[] = "iB";
 	return &suffix[metric ? 1 : 0];
 }
 
 static inline const char *
-scale_prefixes(gboolean metric)
+scale_prefixes(bool metric)
 {
 	return metric ? "\0kMGTPEZ" : "\0KMGTPEZ";
 }
@@ -470,34 +469,34 @@ scale_prefixes(gboolean metric)
  * Scales v so that quotient and reminder are both in the range "0..1023".
  *
  * @param v no document.
- * @param q pointer to a guint; will hold the quotient.
- * @param r pointer to a guint; will hold the reminder.
+ * @param q pointer to a uint; will hold the quotient.
+ * @param r pointer to a uint; will hold the reminder.
  * @param s a string holding the scale prefixes; must be sufficiently long.
  *
  * @return the appropriate prefix character from "s".
  */
 static inline char
-size_scale(guint64 v, guint *q, guint *r, const char *s, gboolean metric)
+size_scale(uint64 v, uint *q, uint *r, const char *s, bool metric)
 {
-	const guint base = kilo(metric);
+	const uint base = kilo(metric);
 
 	if (v < base) {
 		*q = v;
 		*r = 0;
 	} else {
-		const guint thresh = base * base;
+		const uint thresh = base * base;
 
 		for (s++; v >= thresh; v /= base)
 			s++;
 	
-		*q = (guint) v / base;
-		*r = (guint) v % base;
+		*q = (uint) v / base;
+		*r = (uint) v % base;
 	}
 	return *s;
 }
 
 static inline char
-norm_size_scale(guint64 v, guint *q, guint *r, gboolean metric)
+norm_size_scale(uint64 v, uint *q, uint *r, bool metric)
 {
 	return size_scale(v, q, r, scale_prefixes(metric), metric);
 }
@@ -507,9 +506,9 @@ norm_size_scale(guint64 v, guint *q, guint *r, gboolean metric)
  * by 1024 (binary).
  */
 static inline char
-kib_size_scale(guint64 v, guint *q, guint *r, gboolean metric)
+kib_size_scale(uint64 v, uint *q, uint *r, bool metric)
 {
-	if (metric && v < ((guint64) -1) / 1024) {
+	if (metric && v < ((uint64) -1) / 1024) {
 		v = (v * 1024) / 1000;
 	}
 	return size_scale(v, q, r, scale_prefixes(metric) + 1, metric);
@@ -528,13 +527,13 @@ kib_size_scale(guint64 v, guint *q, guint *r, gboolean metric)
  * @return The length of the resulting string assuming ``size'' is sufficient.
  */
 size_t
-short_size_to_string_buf(guint64 size, gboolean metric, char *dst, size_t len)
+short_size_to_string_buf(uint64 size, bool metric, char *dst, size_t len)
 {
 	if (size < kilo(metric)) {
-		guint n = size;
+		uint n = size;
 		return gm_snprintf(dst, len, NG_("%u Byte", "%u Bytes", n), n);
 	} else {
-		guint q, r;
+		uint q, r;
 		char c;
 
 		c = norm_size_scale(size, &q, &r, metric);
@@ -545,7 +544,7 @@ short_size_to_string_buf(guint64 size, gboolean metric, char *dst, size_t len)
 }
 
 const char *
-short_size(guint64 size, gboolean metric)
+short_size(uint64 size, bool metric)
 {
 	static char b[SIZE_FIELD_MAX];
 
@@ -554,15 +553,24 @@ short_size(guint64 size, gboolean metric)
 }
 
 const char *
-short_frequency(guint64 freq)
+short_size2(uint64 size, bool metric)
+{
+	static char b[SIZE_FIELD_MAX];
+
+	short_size_to_string_buf(size, metric, b, sizeof b);
+	return b;
+}
+
+const char *
+short_frequency(uint64 freq)
 {
 	static char b[SIZE_FIELD_MAX];
 
 	if (freq < kilo(TRUE)) {
-		guint n = freq;
+		uint n = freq;
 		gm_snprintf(b, sizeof b, "%u Hz", n);
 	} else {
-		guint q, r;
+		uint q, r;
 		char c;
 
 		c = norm_size_scale(freq, &q, &r, TRUE);
@@ -577,15 +585,15 @@ short_frequency(guint64 freq)
  * Like short_size() but with unbreakable space between the digits and unit.
  */
 const char *
-short_html_size(guint64 size, gboolean metric)
+short_html_size(uint64 size, bool metric)
 {
 	static char b[SIZE_FIELD_MAX];
 
 	if (size < kilo(metric)) {
-		guint n = size;
+		uint n = size;
 		gm_snprintf(b, sizeof b, NG_("%u&nbsp;Byte", "%u&nbsp;Bytes", n), n);
 	} else {
-		guint q, r;
+		uint q, r;
 		char c;
 
 		c = norm_size_scale(size, &q, &r, metric);
@@ -598,15 +606,15 @@ short_html_size(guint64 size, gboolean metric)
 }
 
 size_t
-short_byte_size_to_buf(guint64 size, gboolean metric, char *buf, size_t buflen)
+short_byte_size_to_buf(uint64 size, bool metric, char *buf, size_t buflen)
 {
 	size_t w;
 
 	if (size < kilo(metric)) {
-		guint n = size;
+		uint n = size;
 		w = gm_snprintf(buf, buflen, "%u B", n);
 	} else {
-		guint q, r;
+		uint q, r;
 		char c;
 
 		c = norm_size_scale(size, &q, &r, metric);
@@ -623,7 +631,7 @@ short_byte_size_to_buf(guint64 size, gboolean metric, char *buf, size_t buflen)
  * is less than a kilo.
  */
 const char *
-short_byte_size(guint64 size, gboolean metric)
+short_byte_size(uint64 size, bool metric)
 {
 	static char b[SIZE_FIELD_MAX];
 
@@ -632,7 +640,7 @@ short_byte_size(guint64 size, gboolean metric)
 }
 
 const char *
-short_byte_size2(guint64 size, gboolean metric)
+short_byte_size2(uint64 size, bool metric)
 {
 	static char b[SIZE_FIELD_MAX];
 
@@ -641,15 +649,15 @@ short_byte_size2(guint64 size, gboolean metric)
 }
 
 size_t
-short_kb_size_to_buf(guint64 size, gboolean metric, char *buf, size_t buflen)
+short_kb_size_to_buf(uint64 size, bool metric, char *buf, size_t buflen)
 {
 	size_t w;
 
 	if (size < kilo(metric)) {
 		w = gm_snprintf(buf, buflen,
-				"%u %s", (guint) size, metric ? "kB" : "KiB");
+				"%u %s", (uint) size, metric ? "kB" : "KiB");
 	} else {
-		guint q, r;
+		uint q, r;
 		char c;
 
 		c = kib_size_scale(size, &q, &r, metric);
@@ -666,7 +674,7 @@ short_kb_size_to_buf(guint64 size, gboolean metric, char *buf, size_t buflen)
  * kibibytes, not bytes.
  */
 const char *
-short_kb_size(guint64 size, gboolean metric)
+short_kb_size(uint64 size, bool metric)
 {
 	static char b[SIZE_FIELD_MAX];
 
@@ -675,7 +683,7 @@ short_kb_size(guint64 size, gboolean metric)
 }
 
 const char *
-short_kb_size2(guint64 size, gboolean metric)
+short_kb_size2(uint64 size, bool metric)
 {
 	static char b[SIZE_FIELD_MAX];
 
@@ -687,14 +695,14 @@ short_kb_size2(guint64 size, gboolean metric)
  * @return a number of Kbytes in a compact readable form
  */
 const char *
-compact_kb_size(guint32 size, gboolean metric)
+compact_kb_size(uint32 size, bool metric)
 {
 	static char b[SIZE_FIELD_MAX];
 
 	if (size < kilo(metric)) {
-		gm_snprintf(b, sizeof b, "%u%s", (guint) size, metric ? "kB" : "KiB");
+		gm_snprintf(b, sizeof b, "%u%s", (uint) size, metric ? "kB" : "KiB");
 	} else {
-		guint q, r;
+		uint q, r;
 		char c;
 
 		c = kib_size_scale(size, &q, &r, metric);
@@ -706,7 +714,7 @@ compact_kb_size(guint32 size, gboolean metric)
 }
 
 const char *
-nice_size(guint64 size, gboolean metric)
+nice_size(uint64 size, bool metric)
 {
 	static char buf[256];
 	char bytes[UINT64_DEC_BUFLEN];
@@ -718,12 +726,12 @@ nice_size(guint64 size, gboolean metric)
 }
 
 char *
-compact_value(char *buf, size_t size, guint64 v, gboolean metric)
+compact_value(char *buf, size_t size, uint64 v, bool metric)
 {
 	if (v < kilo(metric)) {
-		gm_snprintf(buf, size, "%u", (guint) v);
+		gm_snprintf(buf, size, "%u", (uint) v);
 	} else {
-		guint q, r;
+		uint q, r;
 		char c;
 
 		c = norm_size_scale(v, &q, &r, metric);
@@ -735,12 +743,12 @@ compact_value(char *buf, size_t size, guint64 v, gboolean metric)
 }
 
 char *
-short_value(char *buf, size_t size, guint64 v, gboolean metric)
+short_value(char *buf, size_t size, uint64 v, bool metric)
 {
 	if (v < kilo(metric)) {
-		gm_snprintf(buf, size, "%u ", (guint) v);
+		gm_snprintf(buf, size, "%u ", (uint) v);
 	} else {
-		guint q, r;
+		uint q, r;
 		char c;
 
 		c = norm_size_scale(v, &q, &r, metric);
@@ -752,7 +760,7 @@ short_value(char *buf, size_t size, guint64 v, gboolean metric)
 }
 
 const char *
-compact_size(guint64 size, gboolean metric)
+compact_size(uint64 size, bool metric)
 {
 	static char buf[SIZE_FIELD_MAX];
 
@@ -762,7 +770,7 @@ compact_size(guint64 size, gboolean metric)
 }
 
 const char *
-compact_size2(guint64 size, gboolean metric)
+compact_size2(uint64 size, bool metric)
 {
 	static char buf[SIZE_FIELD_MAX];
 
@@ -772,7 +780,7 @@ compact_size2(guint64 size, gboolean metric)
 }
 
 const char *
-compact_rate(guint64 rate, gboolean metric)
+compact_rate(uint64 rate, bool metric)
 {
 	static char buf[SIZE_FIELD_MAX];
 
@@ -783,7 +791,7 @@ compact_rate(guint64 rate, gboolean metric)
 }
 
 static size_t
-short_rate_to_string_buf(guint64 rate, gboolean metric, char *dst, size_t size)
+short_rate_to_string_buf(uint64 rate, bool metric, char *dst, size_t size)
 {
 	short_value(dst, size, rate, metric);
 	/* TRANSLATORS: Don't translate 'B', just 's' is allowed. */
@@ -791,7 +799,7 @@ short_rate_to_string_buf(guint64 rate, gboolean metric, char *dst, size_t size)
 }
 
 short_string_t
-short_rate_get_string(guint64 rate, gboolean metric)
+short_rate_get_string(uint64 rate, bool metric)
 {
 	short_string_t buf;
 	short_rate_to_string_buf(rate, metric, buf.str, sizeof buf.str);
@@ -799,7 +807,7 @@ short_rate_get_string(guint64 rate, gboolean metric)
 }
 
 const char *
-short_rate(guint64 rate, gboolean metric)
+short_rate(uint64 rate, bool metric)
 {
 	static short_string_t buf;
 	buf = short_rate_get_string(rate, metric);
@@ -867,11 +875,11 @@ guid_hex_str(const struct guid *guid)
 	return buf;
 }
 
-static gint8 char2int_tabs[3][(size_t) (guchar) -1 + 1];
+static int8 char2int_tabs[3][(size_t) (uchar) -1 + 1];
 
-const gint8 *hex2int_tab = char2int_tabs[0];
-const gint8 *dec2int_tab = char2int_tabs[1];
-const gint8 *alnum2int_tab = char2int_tabs[2];
+const int8 *hex2int_tab = char2int_tabs[0];
+const int8 *dec2int_tab = char2int_tabs[1];
+const int8 *alnum2int_tab = char2int_tabs[2];
 
 /**
  * Converts a hexadecimal char (0-9, A-F, a-f) to an integer.
@@ -883,7 +891,7 @@ const gint8 *alnum2int_tab = char2int_tabs[2];
  * @return "0..15" for valid hexadecimal ASCII characters.
  */
 int
-hex2int(guchar c)
+hex2int(uchar c)
 {
 	int ret;
 	
@@ -902,7 +910,7 @@ hex2int(guchar c)
  * @return "0..9" for valid decimal ASCII characters.
  */
 static int
-dec2int(guchar c)
+dec2int(uchar c)
 {
 	int ret;
 	
@@ -921,7 +929,7 @@ dec2int(guchar c)
  * @return "0..36" for valid decimal ASCII characters.
  */
 static int
-alnum2int(guchar c)
+alnum2int(uchar c)
 {
 	int ret;
 	
@@ -949,7 +957,7 @@ hex2int_init(void)
 	
 	/* Check consistency of hex2int_tab */
 
-	for (i = 0; i <= (guchar) -1; i++)
+	for (i = 0; i <= (uchar) -1; i++)
 		switch (i) {
 		case '0': g_assert(0 == hex2int(i)); break;
 		case '1': g_assert(1 == hex2int(i)); break;
@@ -997,7 +1005,7 @@ dec2int_init(void)
 	
 	/* Check consistency of hex2int_tab */
 
-	for (i = 0; i <= (guchar) -1; i++)
+	for (i = 0; i <= (uchar) -1; i++)
 		switch (i) {
 		case '0': g_assert(0 == dec2int(i)); break;
 		case '1': g_assert(1 == dec2int(i)); break;
@@ -1033,7 +1041,7 @@ alnum2int_init(void)
 	
 	/* Check consistency of hex2int_tab */
 
-	for (i = 0; i <= (guchar) -1; i++) {
+	for (i = 0; i <= (uchar) -1; i++) {
 		const char *p = i ? strchr(abc, ascii_tolower(i)): NULL;
 		int v = p ? (p - abc) : -1;
 	
@@ -1051,7 +1059,7 @@ alnum2int_init(void)
  *
  * @return TRUE if OK.
  */
-gboolean
+bool
 hex_to_guid(const char *hexguid, struct guid *guid)
 {
 	size_t ret;
@@ -1231,9 +1239,7 @@ bitprint_to_urn_string(const struct sha1 *sha1, const struct tth *tth)
 		const char * const end = &buf[sizeof buf];
 		char *p = buf;
 
-		memcpy(p, prefix, CONST_STRLEN(prefix));
-		p += CONST_STRLEN(prefix);
-		
+		p = mempcpy(p, prefix, CONST_STRLEN(prefix));
 		base32_encode(p, end - p, sha1->data, sizeof sha1->data);
 		p += SHA1_BASE32_SIZE;
 
@@ -1373,10 +1379,10 @@ tth_to_urn_string(const struct tth *tth)
  */
 G_GNUC_HOT size_t
 common_leading_bits(
-	gconstpointer k1, size_t k1bits, gconstpointer k2, size_t k2bits)
+	const void *k1, size_t k1bits, const void *k2, size_t k2bits)
 {
-	const guint8 *p1 = k1;
-	const guint8 *p2 = k2;
+	const uint8 *p1 = k1;
+	const uint8 *p2 = k2;
 	size_t cbits;			/* Total amount of bits to compare */
 	size_t bytes;			/* Amount of bytes to compare */
 	size_t bits;			/* Remaining bits in last byte */
@@ -1393,7 +1399,7 @@ common_leading_bits(
 	bytes = cbits >> 3;
 
 	for (i = 0; i < bytes; i++) {
-		guint8 diff = *p1++ ^ *p2++;
+		uint8 diff = *p1++ ^ *p2++;
 		if (diff)
 			return i * 8 + 7 - highest_bit_set(diff);
 	}
@@ -1401,8 +1407,8 @@ common_leading_bits(
 	bits = cbits & 0x7;
 
 	if (bits != 0) {
-		guint8 mask = ~((1 << (8 - bits)) - 1);
-		guint8 diff = (*p1 & mask) ^ (*p2 & mask);
+		uint8 mask = ~((1 << (8 - bits)) - 1);
+		uint8 diff = (*p1 & mask) ^ (*p2 & mask);
 		if (diff)
 			return bytes * 8 + 7 - highest_bit_set(diff);
 	}
@@ -1436,7 +1442,7 @@ force_range(float val, float min, float max)
  * Check whether buffer contains printable data, suitable for "%s" printing.
  * If not, consider dump_hex().
  */
-gboolean
+bool
 is_printable(const char *buf, int len)
 {
 	const char *p = buf;
@@ -1472,7 +1478,7 @@ dump_hex_line(FILE *out, const char *data, size_t length, size_t offset)
 			*p++ = ' ';
 		}
 		if (i < length) {	
-			guchar c;
+			uchar c;
 
 			c = data[i];
 			i++;
@@ -1491,7 +1497,7 @@ dump_hex_line(FILE *out, const char *data, size_t length, size_t offset)
 	*p = '\0';
 	*q = '\0';
 
-	fprintf(out, "%5u %s  %s\n", (guint) (offset & 0xffff), hex_buf, char_buf);
+	fprintf(out, "%5u %s  %s\n", (uint) (offset & 0xffff), hex_buf, char_buf);
 }
 
 /**
@@ -1499,7 +1505,7 @@ dump_hex_line(FILE *out, const char *data, size_t length, size_t offset)
  * Displays the "title" then the characters in "s", # of bytes to print in "b"
  */
 void
-dump_hex(FILE *out, const char *title, gconstpointer data, int length)
+dump_hex(FILE *out, const char *title, const void *data, int length)
 {
 	int i;
 
@@ -1554,7 +1560,7 @@ dump_string(FILE *out, const char *str, size_t len, const char *trailer)
  * Is string made-up of printable ISO-8859 characters?
  * If not, consider dump_hex().
  */
-gboolean
+bool
 is_printable_iso8859_string(const char *s)
 {
 	int c;
@@ -1579,7 +1585,7 @@ void
 locale_strlower(char *dst, const char *src)
 {
 	do {
-		*dst++ = tolower((guchar) *src);
+		*dst++ = tolower((uchar) *src);
 	} while (*src++);
 }
 
@@ -1665,11 +1671,11 @@ failure:
 /**
  * Find amount of common leading bits between two IP addresses.
  */
-static G_GNUC_HOT guint8
-find_common_leading(guint32 ip1, guint32 ip2)
+static G_GNUC_HOT uint8
+find_common_leading(uint32 ip1, uint32 ip2)
 {
-	guint8 n;
-	guint32 mask;
+	uint8 n;
+	uint32 mask;
 
 	for (n = 0, mask = 0x80000000; n < 32; n++, mask |= (mask >> 1)) {
 		if ((ip1 & mask) != (ip2 & mask))
@@ -1699,11 +1705,11 @@ find_common_leading(guint32 ip1, guint32 ip2)
  */
 void
 ip_range_split(
-	guint32 lower_ip, guint32 upper_ip, cidr_split_t cb, gpointer udata)
+	uint32 lower_ip, uint32 upper_ip, cidr_split_t cb, void *udata)
 {
-	guint8 bits;
-	guint32 mask;
-	guint32 trailing;
+	uint8 bits;
+	uint32 mask;
+	uint32 trailing;
 
 	g_assert(lower_ip <= upper_ip);
 
@@ -1726,7 +1732,7 @@ ip_range_split(
 
 			(*cb)(lower_ip, bits, udata);
 		} else {
-			guint32 cut;
+			uint32 cut;
 
 			/*
 			 * Start filling after the first 1 bit in lower_ip.
@@ -1745,7 +1751,7 @@ ip_range_split(
 			ip_range_split(cut + 1, upper_ip, cb, udata);
 		}
 	} else {
-		guint32 cut;
+		uint32 cut;
 
 		/*
 		 * We can't cover the full range.
@@ -1771,18 +1777,6 @@ ip_range_split(
 	}
 }
 
-/*
- * Hashing of pointers.
- *
- * The identity function makes a poor hash for pointers.
- */
-unsigned
-pointer_hash_func(const void *p)
-{
-	unsigned long v = pointer_to_ulong(p);
-	return (((guint64) 0x4F1BBCDCUL * v) >> 32) ^ v;
-}
-
 static inline const char *
 html_escape_replacement(char c, size_t *len)
 {
@@ -1824,7 +1818,7 @@ html_escape(const char *src, char *dst, size_t dst_size)
 {
 	char *d = dst;
 	const char *s = src;
-	guchar c;
+	uchar c;
 
 	g_assert(0 == dst_size || NULL != dst);
 	g_assert(NULL != src);
@@ -1854,16 +1848,16 @@ html_escape(const char *src, char *dst, size_t dst_size)
 	return d - dst;
 }
 
-static GHashTable *html_entities_lut;
+static htable_t *html_entities_lut;
 
 static G_GNUC_COLD void
 html_entities_init(void)
 {
 	size_t i;
 
-	html_entities_lut = g_hash_table_new(g_str_hash, g_str_equal);
+	html_entities_lut = htable_create(HASH_KEY_STRING, 0);
 	for (i = 0; i < G_N_ELEMENTS(html_entities); i++) {
-		gm_hash_table_insert_const(html_entities_lut, html_entities[i].name,
+		htable_insert(html_entities_lut, html_entities[i].name,
 			uint_to_pointer(html_entities[i].uc));
 	}
 }
@@ -1871,7 +1865,7 @@ html_entities_init(void)
 static void
 html_entities_close(void)
 {
-	gm_hash_table_destroy_null(&html_entities_lut);
+	htable_free_null(&html_entities_lut);
 }
 
 /**
@@ -1881,10 +1875,10 @@ html_entities_close(void)
  * @param endptr If not NULL, it will be set to point either to
  *		 		 the original string or the next character after
  *				 the entity.
- * @return		 On failure (guint32)-1 is returned, on success the
+ * @return		 On failure (uint32)-1 is returned, on success the
  *				 Unicode codepoint.
  */
-guint32
+uint32
 html_decode_entity(const char * const src, const char **endptr)
 {
 	if ('&' != src[0])
@@ -1893,7 +1887,7 @@ html_decode_entity(const char * const src, const char **endptr)
 	if ('#' == src[1]) {
 		const char *ep, *p;
 		int base, error;
-		guint32 v;
+		uint32 v;
 
 		switch (src[2]) {
 		case 'x':
@@ -1934,7 +1928,7 @@ html_decode_entity(const char * const src, const char **endptr)
 		if (!html_entities_lut) {
 			html_entities_init();
 		}
-		value = g_hash_table_lookup(html_entities_lut, name);
+		value = htable_lookup(html_entities_lut, name);
 		if (NULL == value)
 			goto failure;
 
@@ -1948,7 +1942,7 @@ failure:
 	if (endptr) {
 		*endptr = src;
 	}
-	return (guint32) -1;
+	return (uint32) -1;
 }
 
 /**
@@ -1977,8 +1971,8 @@ G_GNUC_HOT int
 bitcmp(const void *s1, const void *s2, size_t n)
 {
 	int i, bytes, remain;
-	const guint8 *p1 = s1, *p2 = s2;
-	guint8 mask, c1, c2;
+	const uint8 *p1 = s1, *p2 = s2;
+	uint8 mask, c1, c2;
 
 	bytes = n / 8;				/* First bytes to compare */
 
@@ -1994,7 +1988,7 @@ bitcmp(const void *s1, const void *s2, size_t n)
 	if (0 == remain)
 		return 0;
 
-	mask = (guint8) -1 << (8 - remain);
+	mask = (uint8) -1 << (8 - remain);
 
 	c1 = *p1 & mask;
 	c2 = *p2 & mask;
@@ -2184,8 +2178,8 @@ misc_init(void)
 	{
 		static const struct {
 			const char *s;
-			const guint64 v;
-			const guint base;
+			const uint64 v;
+			const uint base;
 			const int error;
 		} tests[] = {
 			{ "", 					0,				10, EINVAL },
@@ -2198,7 +2192,7 @@ misc_init(void)
 			{ "ffff",				0xffff,			16, 0 },
 			{ "fffff",				0xfffff,		16, 0 },
 			{ "ffffffff",			0xffffffffU,	16, 0 },
-			{ "ffffffffffffffff",	(guint64) -1,	16, 0 },
+			{ "ffffffffffffffff",	(uint64) -1,	16, 0 },
 			{ "1111111111111111",	0xffff,			2,  0 },
 			{ "11111111111111111",	0x1ffff,		2,  0 },
 			{ "111111111111111111",	0x3ffff,		2,  0 },
@@ -2212,12 +2206,12 @@ misc_init(void)
 			{ "8",					0,				8, EINVAL },
 			{ "9",					0,				9, EINVAL },
 		};
-		guint i;
+		uint i;
 
 		for (i = 0; i < G_N_ELEMENTS(tests); i++) {
 			const char *endptr;
 			int error;
-			guint64 v;
+			uint64 v;
 
 			g_assert((0 == tests[i].v) ^ (0 == tests[i].error));
 			
@@ -2230,7 +2224,7 @@ misc_init(void)
 			error = EAGAIN;
 			endptr = GINT_TO_POINTER(-1);
 			v = parse_uint32(tests[i].s, &endptr, tests[i].base, &error);
-			if (tests[i].v > (guint32) -1) {
+			if (tests[i].v > (uint32) -1) {
 				g_assert(0 == v);
 				g_assert(ERANGE == error);
 			} else {
@@ -2241,7 +2235,7 @@ misc_init(void)
 			error = EAGAIN;
 			endptr = GINT_TO_POINTER(-1);
 			v = parse_uint16(tests[i].s, &endptr, tests[i].base, &error);
-			if (tests[i].v > (guint16) -1) {
+			if (tests[i].v > (uint16) -1) {
 				g_assert(0 == v);
 				g_assert(ERANGE == error);
 			} else {
