@@ -41,13 +41,16 @@
  */
 
 void random_init(void);
-guint32 random_value(guint32 max) WARN_UNUSED_RESULT;
-guint32 random_u32(void) WARN_UNUSED_RESULT;
+uint32 random_value(uint32 max) WARN_UNUSED_RESULT;
+uint64 random_value64(uint64 max) WARN_UNUSED_RESULT;
+uint32 random_u32(void) WARN_UNUSED_RESULT;
 void random_bytes(void *dst, size_t size);
-guint32 random_cpu_noise(void);
+uint32 random_cpu_noise(void);
 void random_collect(void (*cb)(void));
 void random_pool_append(void *buf, size_t len, void (*cb)(void));
 void random_add(const void *data, size_t datalen);
+double random_double_generate(random_fn_t rf);
+double random_double(void);
 
 #endif /* _random_h_ */
 
