@@ -33,10 +33,10 @@
 
 #include "common.h"
 
-#include "misc.h"
+#include "sorted_array.h"
 #include "halloc.h"
+#include "misc.h"
 #include "walloc.h"
-#include "lib/sorted_array.h"
 
 #include "override.h"		/* Must be the last header included */
 
