@@ -71,10 +71,10 @@
 /*
  * The following include space for NUL, too.
  */
-#define UINT8_DEC_BUFLEN	TYPE_DEC_BUFLEN(guint8)
-#define UINT16_DEC_BUFLEN	TYPE_DEC_BUFLEN(guint16)
-#define UINT32_DEC_BUFLEN	TYPE_DEC_BUFLEN(guint32)
-#define UINT64_DEC_BUFLEN	TYPE_DEC_BUFLEN(guint64)
+#define UINT8_DEC_BUFLEN	TYPE_DEC_BUFLEN(uint8)
+#define UINT16_DEC_BUFLEN	TYPE_DEC_BUFLEN(uint16)
+#define UINT32_DEC_BUFLEN	TYPE_DEC_BUFLEN(uint32)
+#define UINT64_DEC_BUFLEN	TYPE_DEC_BUFLEN(uint64)
 #define OFF_T_DEC_BUFLEN	TYPE_DEC_BUFLEN(fileoffset_t)
 #define TIME_T_DEC_BUFLEN	TYPE_DEC_BUFLEN(time_t)
 #define SIZE_T_DEC_BUFLEN	TYPE_DEC_BUFLEN(size_t)
@@ -82,19 +82,22 @@
 #define UINT_DEC_BUFLEN		TYPE_DEC_BUFLEN(unsigned int)
 #define ULONG_DEC_BUFLEN	TYPE_DEC_BUFLEN(unsigned long)
 
-#define UINT8_HEX_BUFLEN	TYPE_HEX_BUFLEN(guint8)
-#define UINT16_HEX_BUFLEN	TYPE_HEX_BUFLEN(guint16)
-#define UINT32_HEX_BUFLEN	TYPE_HEX_BUFLEN(guint32)
-#define UINT64_HEX_BUFLEN	TYPE_HEX_BUFLEN(guint64)
+#define INT_DEC_BUFLEN		TYPE_DEC_BUFLEN(int)
+#define LONG_DEC_BUFLEN		TYPE_DEC_BUFLEN(long)
+
+#define UINT8_HEX_BUFLEN	TYPE_HEX_BUFLEN(uint8)
+#define UINT16_HEX_BUFLEN	TYPE_HEX_BUFLEN(uint16)
+#define UINT32_HEX_BUFLEN	TYPE_HEX_BUFLEN(uint32)
+#define UINT64_HEX_BUFLEN	TYPE_HEX_BUFLEN(uint64)
 #define ULONG_HEX_BUFLEN	TYPE_HEX_BUFLEN(unsigned long)
 
 #define POINTER_BUFLEN			TYPE_HEX_BUFLEN(unsigned long)
 #define HOST_ADDR_BUFLEN		(MAX(IPV4_ADDR_BUFLEN, IPV6_ADDR_BUFLEN))
 #define HOST_ADDR_PORT_BUFLEN	(HOST_ADDR_BUFLEN + sizeof ":[65535]")
 
-#define UINT16_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(guint16)
-#define UINT32_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(guint32)
-#define UINT64_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(guint64)
+#define UINT16_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(uint16)
+#define UINT32_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(uint32)
+#define UINT64_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(uint64)
 #define OFF_T_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(fileoffset_t)
 #define TIME_T_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(time_t)
 #define SIZE_T_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(size_t)
@@ -102,27 +105,28 @@
 #define UINT_DEC_GRP_BUFLEN		GROUPPED_DEC_BUFLEN(unsigned int)
 #define ULONG_DEC_GRP_BUFLEN	GROUPPED_DEC_BUFLEN(unsigned long)
 
-size_t int32_to_string_buf(gint32 v, char *dst, size_t size);
-size_t uint32_to_string_buf(guint32 v, char *dst, size_t size);
-size_t uint64_to_string_buf(guint64 v, char *dst, size_t size);
+size_t int32_to_string_buf(int32 v, char *dst, size_t size);
+size_t uint32_to_string_buf(uint32 v, char *dst, size_t size);
+size_t uint64_to_string_buf(uint64 v, char *dst, size_t size);
 size_t uint_to_string_buf(unsigned v, char *dst, size_t size);
+size_t ulong_to_string_buf(unsigned long v, char *dst, size_t size);
 size_t fileoffset_t_to_string_buf(fileoffset_t v, char *dst, size_t size);
 size_t size_t_to_string_buf(size_t v, char *dst, size_t size);
 size_t pointer_to_string_buf(const void *ptr, char *dst, size_t size);
-const char *uint32_to_string(guint32);
-const char *uint64_to_string(guint64);
-const char *uint64_to_string2(guint64);
+const char *uint32_to_string(uint32);
+const char *uint64_to_string(uint64);
+const char *uint64_to_string2(uint64);
 const char *uint_to_string(unsigned v);
 const char *fileoffset_t_to_string(fileoffset_t);
 const char *size_t_to_string(size_t);
 const char *pointer_to_string(const void *);
 const char *filesize_to_string(filesize_t);
 const char *filesize_to_string2(filesize_t);
-const char *ipv6_to_string(const guint8 *ipv6);
-const char *ipv6_to_string2(const guint8 *ipv6);
-size_t ipv6_to_string_buf(const guint8 *ipv6, char *dst, size_t size);
+const char *ipv6_to_string(const uint8 *ipv6);
+const char *ipv6_to_string2(const uint8 *ipv6);
+size_t ipv6_to_string_buf(const uint8 *ipv6, char *dst, size_t size);
 
-char *hex_escape(const char *name, gboolean strict);
+char *hex_escape(const char *name, bool strict);
 char *control_escape(const char *s);
 const char *lazy_string_to_printf_escape(const char *src);
 
@@ -130,13 +134,14 @@ const char *lazy_string_to_printf_escape(const char *src);
  * Groupped by thousands integer values.
  */
 
-size_t int32_to_gstring_buf(gint32 v, char *dst, size_t size);
-size_t uint32_to_gstring_buf(guint32 v, char *dst, size_t size);
-size_t uint64_to_gstring_buf(guint64 v, char *dst, size_t size);
+size_t int32_to_gstring_buf(int32 v, char *dst, size_t size);
+size_t uint32_to_gstring_buf(uint32 v, char *dst, size_t size);
+size_t uint64_to_gstring_buf(uint64 v, char *dst, size_t size);
 size_t uint_to_gstring_buf(unsigned v, char *dst, size_t size);
+size_t ulong_to_gstring_buf(unsigned long v, char *dst, size_t size);
 size_t size_t_to_gstring_buf(size_t v, char *dst, size_t size);
-const char *uint32_to_gstring(guint32);
-const char *uint64_to_gstring(guint64);
+const char *uint32_to_gstring(uint32);
+const char *uint64_to_gstring(uint64);
 const char *uint_to_gstring(unsigned v);
 const char *size_t_to_gstring(size_t v);
 
