@@ -53,6 +53,8 @@
 #define REMAP_ZALLOC
 #endif
 
+size_t walloc_blocksize(size_t size) G_GNUC_PURE;
+
 #ifdef REMAP_ZALLOC
 
 #ifdef TRACK_ZALLOC
@@ -60,22 +62,22 @@
 #endif
 
 #if 0 && GLIB_CHECK_VERSION(2, 10, 0)
-static inline gpointer walloc(size_t size) { return g_slice_alloc(size); }
-static inline gpointer walloc0(size_t size) { return g_slice_alloc0(size); }
-static inline void wfree(gpointer p, size_t size) { g_slice_free1(size, p); }
+static inline void *walloc(size_t size) { return g_slice_alloc(size); }
+static inline void *walloc0(size_t size) { return g_slice_alloc0(size); }
+static inline void wfree(void *p, size_t size) { g_slice_free1(size, p); }
 
-static inline gpointer
-wcopy(gconstpointer p, size_t size)
+static inline void *
+wcopy(const void *p, size_t size)
 {
-	gpointer x = g_slice_alloc(size);
+	void *x = g_slice_alloc(size);
 	memcpy(x, p, size);
 	return x;
 }
 
-static inline gpointer
-wrealloc(gpointer p, size_t old_size, size_t new_size)
+static inline void *
+wrealloc(void *p, size_t old_size, size_t new_size)
 {
-	gpointer x = g_slice_alloc(new_size);
+	void *x = g_slice_alloc(new_size);
 	memcpy(x, p, MIN(new_size, old_size));
 	g_slice_free1(old_size, p);
 	return x;
@@ -85,28 +87,28 @@ wrealloc(gpointer p, size_t old_size, size_t new_size)
 #define walloc(s)			g_malloc(s)
 #define walloc0(s)			g_malloc0(s)
 
-static inline gpointer
-wcopy(gconstpointer p, size_t size)
+static inline void *
+wcopy(const void *p, size_t size)
 {
 	return g_memdup(p, size);
 }
 
 static inline void
-wfree(gpointer p, size_t size)
+wfree(void *p, size_t size)
 {
 	(void) size;
 	g_free(p);
 }
 
 static inline void
-wfree0(gpointer p, size_t size)
+wfree0(void *p, size_t size)
 {
 	memset(p, 0, size);
 	g_free(p);
 }
 
-static inline gpointer
-wrealloc(gpointer p, size_t o, size_t n)
+static inline void *
+wrealloc(void *p, size_t o, size_t n)
 {
 	(void) o;
 	return g_realloc(p, n);
@@ -122,23 +124,23 @@ wmove(void *p, size_t n)
 
 #else	/* !REMAP_ZALLOC */
 
-gpointer walloc(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
-gpointer walloc0(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
-void wfree(gpointer ptr, size_t size);
-void wfree0(gpointer ptr, size_t size);
-gpointer wrealloc(gpointer old, size_t old_size, size_t new_size)
+void *walloc(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void *walloc0(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void wfree(void *ptr, size_t size);
+void wfree0(void *ptr, size_t size);
+void *wrealloc(void *old, size_t old_size, size_t new_size)
 			WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void *wmove(void *ptr, size_t size) WARN_UNUSED_RESULT;
 
 /* Don't define both an inline routine and a macro... */
 #ifndef TRACK_ZALLOC
-static inline gpointer wcopy(gconstpointer ptr, size_t size)
+static inline void *wcopy(const void *ptr, size_t size)
 			WARN_UNUSED_RESULT G_GNUC_MALLOC;
 
-static inline gpointer
-wcopy(gconstpointer ptr, size_t size)
+static inline void *
+wcopy(const void *ptr, size_t size)
 {
-	gpointer cp = walloc(size);
+	void *cp = walloc(size);
 	memcpy(cp, ptr, size);
 	return cp;
 }
@@ -153,10 +155,10 @@ wcopy(gconstpointer ptr, size_t size)
 #define walloc0(s)			walloc0_track(s, _WHERE_, __LINE__)
 #define wrealloc(p,o,n)		wrealloc_track(p, o, n, _WHERE_, __LINE__)
 
-gpointer walloc_track(size_t size, const char *file, int line);
-gpointer walloc0_track(size_t size, const char *file, int line);
-gpointer wcopy_track(gconstpointer, size_t size, const char *file, int line);
-gpointer wrealloc_track(gpointer old, size_t old_size, size_t new_size,
+void *walloc_track(size_t size, const char *file, int line);
+void *walloc0_track(size_t size, const char *file, int line);
+void *wcopy_track(const void *, size_t size, const char *file, int line);
+void *wrealloc_track(void *old, size_t old_size, size_t new_size,
 	const char *file, int line);
 
 #endif	/* TRACK_ZALLOC */
