@@ -40,11 +40,12 @@
 
 struct logagent;
 
-void set_xmalloc_debug(guint32 level);
+void set_xmalloc_debug(uint32 level);
+void xmalloc_crash_mode(void);
 void xmalloc_vmm_inited(void);
 void xmalloc_pre_close(void);
 void xmalloc_post_init(void);
-gboolean xmalloc_is_malloc(void) G_GNUC_CONST;
+bool xmalloc_is_malloc(void) G_GNUC_CONST;
 void xmalloc_show_settings(void);
 void xmalloc_show_settings_log(struct logagent *la);
 void xmalloc_stop_freeing(void);
@@ -53,17 +54,24 @@ void xmalloc_dump_stats(void);
 void xmalloc_dump_stats_log(struct logagent *la, unsigned options);
 void xmalloc_dump_usage_log(struct logagent *la, unsigned options);
 void xmalloc_dump_freelist_log(struct logagent *la);
+size_t xmalloc_freelist_check(struct logagent *la, bool verbose);
+
+void xgc(void);
 
 void *xmalloc(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void *xmalloc0(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void *xpmalloc(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void *xpmalloc0(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void *xhmalloc(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void *xcalloc(size_t nmemb, size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void *xrealloc(void *ptr, size_t size) WARN_UNUSED_RESULT;
 void *xprealloc(void *ptr, size_t size) WARN_UNUSED_RESULT;
 void xfree(void *ptr);
 char *xstrdup(const char *str) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 char *xpstrdup(const char *str) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+char *xstrndup(const char *str, size_t n) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+char *xpstrndup(const char *str, size_t n) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+size_t xallocated(const void *p);
 
 static inline void * WARN_UNUSED_RESULT G_GNUC_MALLOC
 xcopy(const void *p, size_t size)
@@ -81,6 +89,26 @@ xpcopy(const void *p, size_t size)
 	return cp;
 }
 
+#define XMALLOC(p)			\
+G_STMT_START {				\
+	p = xmalloc(sizeof *p);	\
+} G_STMT_END
+
+#define XMALLOC0(p)				\
+G_STMT_START {					\
+	p = xmalloc0(sizeof *p);	\
+} G_STMT_END
+
+#define XPMALLOC(p)				\
+G_STMT_START {					\
+	p = xpmalloc(sizeof *p);	\
+} G_STMT_END
+
+#define XPMALLOC0(p)			\
+G_STMT_START {					\
+	p = xpmalloc0(sizeof *p);	\
+} G_STMT_END
+
 #define XFREE_NULL(p)	\
 G_STMT_START {			\
 	if (p) {			\
