@@ -137,6 +137,7 @@ general_type_str(gint value)
 		N_("\"What's New?\" queries"),
 		N_("GUESS queries"),
 		N_("GUESS queries (0.2)"),
+		N_("GUESS link cache size"),
 		N_("GUESS cached query keys held"),
 		N_("GUESS cached 0.2 hosts held"),
 		N_("GUESS locally generated queries"),
@@ -213,6 +214,7 @@ general_type_str(gint value)
 		N_("Re-seeding of orphan downloads through query hits"),
 		N_("Re-seeding of orphan downloads through upload requests"),
 		N_("DHT estimated amount of nodes"),
+		N_("DHT standard error of estimated amount of nodes"),
 		N_("DHT k-ball theoretical frontier (bits)"),
 		N_("DHT k-ball furthest frontier (bits)"),
 		N_("DHT k-ball closeest frontier (bits)"),
