@@ -1,6 +1,4 @@
 /*
- * $Id: hcache_common.h 6145 2004-08-24 22:16:09Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -29,6 +27,9 @@
 #include "gui.h"
 #include "if/core/hcache.h"
 
-const gchar *get_hcache_name(hcache_type_t type);
+const gchar *get_hcache_name(hcache_type_t);
+void hcache_gui_timer(time_t now);
+void hcache_gui_update_display(void);
 
 #endif /* _gtk_hcache_common_h_ */
+/* vi: set ts=4 sw=4 cindent: */
