@@ -51,6 +51,7 @@
 #include "natpmp.h"
 #include "service.h"
 
+#include "core/inet.h"			/* For inet_router_configured() */
 #include "core/settings.h"		/* For listen_addr() */
 
 #include "if/gnet_property.h"
@@ -58,8 +59,9 @@
 
 #include "lib/atoms.h"
 #include "lib/cq.h"
-#include "lib/glib-missing.h"
+#include "lib/hashing.h"
 #include "lib/host_addr.h"
+#include "lib/htable.h"
 #include "lib/product.h"		/* For product_get_build() */
 #include "lib/stacktrace.h"
 #include "lib/walloc.h"
@@ -85,7 +87,7 @@
 static struct {
 	upnp_device_t *dev;			/**< Our Internet Gateway Device */
 	upnp_ctrl_t *monitor;		/**< Regular monitoring event */
-	guint32 rcvd_pkts;			/**< Amount of received packets */
+	uint32 rcvd_pkts;			/**< Amount of received packets */
 	unsigned delete_pending;	/**< Amount of pending mapping deletes */
 	time_delta_t lease_time;	/**< Lease time to request */
 	unsigned discover:1;		/**< Force discovery again */
@@ -121,7 +123,7 @@ struct upnp_mapping {
 	enum upnp_mapping_magic magic;
 	enum upnp_map_proto proto;	/**< Network protocol used */
 	enum upnp_method method;	/**< Method used to publish mapping */
-	guint16 port;				/**< Port to map */
+	uint16 port;				/**< Port to map */
 	cevent_t *install_ev;		/**< Periodic install event */
 	upnp_ctrl_t *rpc;			/**< Pending control RPC */
 	time_delta_t lease_time;	/**< Requested lease time */
@@ -135,7 +137,7 @@ upnp_mapping_check(const struct upnp_mapping * const um)
 	g_assert(UPNP_MAPPING_MAGIC == um->magic);
 }
 
-static GHashTable *upnp_mappings;	/**< Tracks requested UPnP mappings */
+static htable_t *upnp_mappings;		/**< Tracks requested UPnP mappings */
 static host_addr_t upnp_local_addr;	/**< Computed local IP address */
 
 static const char UPNP_CONN_IP_ROUTED[]	= "IP_Routed";
@@ -154,7 +156,7 @@ upnp_igd_ip_routed(void)
 /**
  * Do we have port mapping deletion in progress?
  */
-gboolean
+bool
 upnp_delete_pending(void)
 {
 	if (GNET_PROPERTY(shutdown_debug) > 1) {
@@ -195,7 +197,18 @@ upnp_mapping_hash(const void *p)
 {
 	const struct upnp_mapping *um = p;
 
-	return ((unsigned) um->proto * 0xa79dU) ^ (unsigned) um->port;
+	return integer_hash2(um->proto) ^ port_hash(um->port);
+}
+
+/**
+ * Hash a UPnP mapping (secondary hash).
+ */
+static unsigned
+upnp_mapping_hash2(const void *p)
+{
+	const struct upnp_mapping *um = p;
+
+	return integer_hash(um->proto) ^ port_hash2(um->port);
 }
 
 /**
@@ -214,7 +227,7 @@ upnp_mapping_eq(const void *a, const void *b)
  * Create a new UPnP mapping record.
  */
 static struct upnp_mapping *
-upnp_mapping_alloc(enum upnp_map_proto proto, guint16 port)
+upnp_mapping_alloc(enum upnp_map_proto proto, uint16 port)
 {
 	struct upnp_mapping * um;
 
@@ -231,7 +244,7 @@ upnp_mapping_alloc(enum upnp_map_proto proto, guint16 port)
  * Free a UPnP mapping record.
  */
 static void
-upnp_mapping_free(struct upnp_mapping *um, gboolean in_shutdown)
+upnp_mapping_free(struct upnp_mapping *um, bool in_shutdown)
 {
 	upnp_mapping_check(um);
 
@@ -382,7 +395,7 @@ upnp_dev_igd_make(const char *desc_url, GSList *services, host_addr_t wan_ip,
 static void
 upnp_check_new_wan_addr(host_addr_t addr)
 {
-	gboolean learnt_external_ip = FALSE;
+	bool learnt_external_ip = FALSE;
 
 	switch (host_addr_net(addr)) {
 	case NET_TYPE_IPV4:
@@ -519,7 +532,7 @@ done:
  * @param arg		user-defined argument
  */
 static void
-upnp_natpmp_discovered(gboolean ok, natpmp_t *gateway, void *arg)
+upnp_natpmp_discovered(bool ok, natpmp_t *gateway, void *arg)
 {
 	(void) arg;
 
@@ -549,7 +562,7 @@ upnp_natpmp_discovered(gboolean ok, natpmp_t *gateway, void *arg)
 static void
 upnp_launch_discovery(void)
 {
-	static gboolean retrying;
+	static bool retrying;
 
 	igd.discovery_done = FALSE;
 	gw.discovery_done = FALSE;
@@ -625,7 +638,7 @@ upnp_packets_igd_callback(int code, void *value, size_t size, void *unused_arg)
  * @param arg		user-defined argument
  */
 static void
-upnp_monitor_natpmp_callback(gboolean ok, natpmp_t *gateway, void *unused_arg)
+upnp_monitor_natpmp_callback(bool ok, natpmp_t *gateway, void *unused_arg)
 {
 	(void) unused_arg;
 
@@ -793,7 +806,7 @@ rediscover:
  *
  * @return TRUE when we are firewalled and in need for port mappings.
  */
-static gboolean
+static bool
 upnp_port_mapping_required(void)
 {
 	if (GNET_PROPERTY(is_firewalled) || GNET_PROPERTY(is_udp_firewalled)) {
@@ -806,9 +819,9 @@ upnp_port_mapping_required(void)
 }
 
 static void
-upnp_count_mapping_kv(void *key, void *u_value, void *data)
+upnp_count_mapping_kv(const void *key, void *u_value, void *data)
 {
-	struct upnp_mapping *um = key;
+	const struct upnp_mapping *um = key;
 	unsigned *count = data;
 
 	(void) u_value;
@@ -828,7 +841,7 @@ upnp_published_mappings(void)
 	g_assert(upnp_mappings != NULL);
 
 	count = 0;
-	g_hash_table_foreach(upnp_mappings, upnp_count_mapping_kv, &count);
+	htable_foreach(upnp_mappings, upnp_count_mapping_kv, &count);
 	return count;
 }
 
@@ -898,8 +911,8 @@ upnp_map_try_publish_all(void)
  * Callout queue periodic event to monitor presence of the Internet Gateway
  * Device or the NAT-PMP gateway we are using and detect configuration changes.
  */
-static gboolean
-upnp_monitor_drivers(gpointer unused_obj)
+static bool
+upnp_monitor_drivers(void *unused_obj)
 {
 	(void) unused_obj;
 
@@ -966,7 +979,7 @@ done:
 	 */
 
 	gnet_prop_set_boolean_val(PROP_PORT_MAPPING_SUCCESSFUL,
-		g_hash_table_size(upnp_mappings) == upnp_published_mappings());
+		htable_count(upnp_mappings) == upnp_published_mappings());
 
 	/*
 	 * Publish mappings if needed.
@@ -997,6 +1010,8 @@ upnp_map_publish_reply(int code, void *value, size_t size, void *arg)
 			g_message("UPNP successfully published UPnP mapping for %s port %u",
 				upnp_map_proto_to_string(um->proto), um->port);
 		}
+		if (!um->published)
+			inet_router_configured();	/* First time we're publishing */
 		um->published = TRUE;
 		um->method = UPNP_M_UPNP;
 	} else {
@@ -1027,7 +1042,7 @@ upnp_map_publish_reply(int code, void *value, size_t size, void *arg)
 	}
 
 	gnet_prop_set_boolean_val(PROP_PORT_MAPPING_SUCCESSFUL,
-		g_hash_table_size(upnp_mappings) == upnp_published_mappings());
+		htable_count(upnp_mappings) == upnp_published_mappings());
 }
 
 /**
@@ -1035,7 +1050,7 @@ upnp_map_publish_reply(int code, void *value, size_t size, void *arg)
  */
 static void
 upnp_map_natpmp_publish_reply(int code,
-	guint16 port, unsigned lifetime, void *arg)
+	uint16 port, unsigned lifetime, void *arg)
 {
 	struct upnp_mapping *um = arg;
 
@@ -1047,6 +1062,8 @@ upnp_map_natpmp_publish_reply(int code,
 				"for %s port %u, lease = %u s",
 				upnp_map_proto_to_string(um->proto), um->port, lifetime);
 		}
+		if (!um->published)
+			inet_router_configured();	/* First time we're publishing */
 		um->published = TRUE;
 		um->method = UPNP_M_NATPMP;
 		cq_resched(um->install_ev, lifetime / 2 * 1000);
@@ -1064,7 +1081,7 @@ upnp_map_natpmp_publish_reply(int code,
 	}
 
 	gnet_prop_set_boolean_val(PROP_PORT_MAPPING_SUCCESSFUL,
-		g_hash_table_size(upnp_mappings) == upnp_published_mappings());
+		htable_count(upnp_mappings) == upnp_published_mappings());
 }
 
 /**
@@ -1185,7 +1202,7 @@ upnp_map_publish(cqueue_t *unused_cq, void *obj)
  * Record port mapping addition.
  */
 static void
-upnp_map_add(enum upnp_map_proto proto, guint16 port)
+upnp_map_add(enum upnp_map_proto proto, uint16 port)
 {
 	struct upnp_mapping key;
 	struct upnp_mapping *um;
@@ -1193,7 +1210,7 @@ upnp_map_add(enum upnp_map_proto proto, guint16 port)
 	key.proto = proto;
 	key.port = port;
 
-	if (gm_hash_table_contains(upnp_mappings, &key))
+	if (htable_contains(upnp_mappings, &key))
 		return;		/* Already known */
 
 	/*
@@ -1209,7 +1226,7 @@ upnp_map_add(enum upnp_map_proto proto, guint16 port)
 	um->install_ev = cq_main_insert(1, upnp_map_publish, um);
 	um->lease_time = UPNP_UNDEFINED_LEASE;
 
-	g_hash_table_insert(upnp_mappings, um, um);
+	htable_insert(upnp_mappings, um, um);
 }
 
 /**
@@ -1248,7 +1265,7 @@ upnp_map_delete_reply(int code, void *value, size_t size, void *arg)
  *
  * @return TRUE if we can dispose of the mapping record.
  */
-static gboolean
+static bool
 upnp_map_unpublish(struct upnp_mapping *um)
 {
 	upnp_mapping_check(um);
@@ -1305,7 +1322,7 @@ upnp_map_unpublish(struct upnp_mapping *um)
  * Remove port mapping.
  */
 static void
-upnp_map_remove(enum upnp_map_proto proto, guint16 port)
+upnp_map_remove(enum upnp_map_proto proto, uint16 port)
 {
 	struct upnp_mapping key;
 	struct upnp_mapping *um;
@@ -1313,7 +1330,7 @@ upnp_map_remove(enum upnp_map_proto proto, guint16 port)
 	key.proto = proto;
 	key.port = port;
 
-	um = g_hash_table_lookup(upnp_mappings, &key);
+	um = htable_lookup(upnp_mappings, &key);
 
 	if (NULL == um) {
 		if (GNET_PROPERTY(upnp_debug)) {
@@ -1322,7 +1339,7 @@ upnp_map_remove(enum upnp_map_proto proto, guint16 port)
 		}
 	} else {
 		upnp_mapping_check(um);
-		g_hash_table_remove(upnp_mappings, um);
+		htable_remove(upnp_mappings, um);
 
 		if (upnp_map_unpublish(um)) {
 			upnp_mapping_free(um, FALSE);
@@ -1368,9 +1385,9 @@ upnp_map_mapping_deleted(int code, void *value, size_t size, void *arg)
  * Remove published mapping of the specified kind.
  */
 static void
-upnp_remove_mapping_kv(void *key, void *u_value, void *data)
+upnp_remove_mapping_kv(const void *key, void *u_value, void *data)
 {
-	struct upnp_mapping *um = key;
+	struct upnp_mapping *um = deconstify_gpointer(key);
 	enum upnp_method method = pointer_to_int(data);
 
 	(void) u_value;
@@ -1414,7 +1431,7 @@ void
 upnp_disabled(void)
 {
 	if (igd.dev != NULL) {
-		g_hash_table_foreach(upnp_mappings, upnp_remove_mapping_kv,
+		htable_foreach(upnp_mappings, upnp_remove_mapping_kv,
 			int_to_pointer(UPNP_M_UPNP));
 		gnet_prop_set_boolean_val(PROP_PORT_MAPPING_SUCCESSFUL, FALSE);
 	}
@@ -1436,7 +1453,7 @@ void
 upnp_natpmp_disabled(void)
 {
 	if (gw.gateway != NULL) {
-		g_hash_table_foreach(upnp_mappings, upnp_remove_mapping_kv,
+		htable_foreach(upnp_mappings, upnp_remove_mapping_kv,
 			int_to_pointer(UPNP_M_NATPMP));
 		gnet_prop_set_boolean_val(PROP_PORT_MAPPING_SUCCESSFUL, FALSE);
 	}
@@ -1455,9 +1472,9 @@ upnp_natpmp_disabled(void)
  * before.
  */
 static void
-upnp_publish_mapping_kv(void *key, void *u_value, void *u_data)
+upnp_publish_mapping_kv(const void *key, void *u_value, void *u_data)
 {
-	struct upnp_mapping *um = key;
+	struct upnp_mapping *um = deconstify_gpointer(key);
 
 	(void) u_value;
 	(void) u_data;
@@ -1485,14 +1502,14 @@ upnp_map_publish_all(void)
 {
 	g_assert(igd.dev != NULL || gw.gateway != NULL);
 
-	g_hash_table_foreach(upnp_mappings, upnp_publish_mapping_kv, NULL);
+	htable_foreach(upnp_mappings, upnp_publish_mapping_kv, NULL);
 }
 
 /**
  * Add TCP port redirection on the IGD device to this machine.
  */
 void
-upnp_map_tcp(guint16 port)
+upnp_map_tcp(uint16 port)
 {
 	upnp_map_add(UPNP_MAP_TCP, port);
 }
@@ -1501,7 +1518,7 @@ upnp_map_tcp(guint16 port)
  * Add UDP port redirection on the IGD device to this machine.
  */
 void
-upnp_map_udp(guint16 port)
+upnp_map_udp(uint16 port)
 {
 	upnp_map_add(UPNP_MAP_UDP, port);
 }
@@ -1510,7 +1527,7 @@ upnp_map_udp(guint16 port)
  * Remove TCP port redirection on the IGD device.
  */
 void
-upnp_unmap_tcp(guint16 port)
+upnp_unmap_tcp(uint16 port)
 {
 	upnp_map_remove(UPNP_MAP_TCP, port);
 }
@@ -1519,7 +1536,7 @@ upnp_unmap_tcp(guint16 port)
  * Remove UDP port redirection on the IGD device.
  */
 void
-upnp_unmap_udp(guint16 port)
+upnp_unmap_udp(uint16 port)
 {
 	upnp_map_remove(UPNP_MAP_UDP, port);
 }
@@ -1559,7 +1576,8 @@ upnp_init(void)
 	cq_periodic_main_add(UPNP_MONITOR_DELAY_MS,
 		upnp_monitor_drivers, NULL);
 
-	upnp_mappings = g_hash_table_new(upnp_mapping_hash, upnp_mapping_eq);
+	upnp_mappings = htable_create_any(upnp_mapping_hash,
+		upnp_mapping_hash2, upnp_mapping_eq);
 }
 
 /**
@@ -1575,10 +1593,10 @@ upnp_post_init(void)
  * Free mappings still present, warning about them since normal cleanup
  * should remove them.
  */
-static gboolean
-upnp_free_mapping_kv(void *key, void *u_value, void *u_data)
+static bool
+upnp_free_mapping_kv(const void *key, void *u_value, void *u_data)
 {
-	struct upnp_mapping *um = key;
+	struct upnp_mapping *um = deconstify_gpointer(key);
 
 	(void) u_value;
 	(void) u_data;
@@ -1603,8 +1621,8 @@ upnp_close(void)
 	upnp_dev_free_null(&igd.dev);
 	natpmp_free_null(&gw.gateway);
 	upnp_ctrl_cancel_null(&igd.monitor, FALSE);
-	g_hash_table_foreach_remove(upnp_mappings, upnp_free_mapping_kv, NULL);
-	gm_hash_table_destroy_null(&upnp_mappings);
+	htable_foreach_remove(upnp_mappings, upnp_free_mapping_kv, NULL);
+	htable_free_null(&upnp_mappings);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
